# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the thermal scalar management.

This module contains the following classes and function:
- DefBCModel
- MultiSelect
- CreateHlistWidgets
- DefBCView
- PageText
- DefBCTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import sys
import Tix, unittest
from Tkconstants import *
from math import *
import tkFileDialog

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Model
from Pages.RadiativeBoundaries import RadiativeBoundariesModel
from Pages.PreProcessingInformations import Informations, preprocessorFile

#-------------------------------------------------------------------------------
# Multi-selection string
#-------------------------------------------------------------------------------


_MULTISEL = "************"

#-------------------------------------------------------------------------------
# Thermal scalar model class
#-------------------------------------------------------------------------------


class DefBCModel:
    """
    """
    def __init__(self, case):
        """
        Constuctor.
        """
        self.case = case
        self.node_scalar   = self.case.xmlGetNode('additional_scalars')
        self.node_bc       = self.case.xmlGetNode('boundary_conditions')
        self.node_def      = self.node_bc.xmlInitNode('boundary_definition')

        self.natureNumber  = {}
        self.zoneMaxNumber = 0

        self.updateDicos()


    def updateDicos(self):
        """
        Get list of labels for boundaries conditions
        """
        self.dicoLabel2Nature = {}
        self.dicoLabel2Zone   = {}
        self.dicoLocal = []
        self.zoneNumber = 0

        for nature in ('inlet', 'outlet', 'wall', 'symmetry'):
            self.natureNumber[nature] = 0
            for node in self.node_def.xmlGetNodeList(nature):
                self.zoneNumber = self.zoneNumber + 1
                self.zoneMaxNumber = max(self.zoneMaxNumber, self.zoneNumber)
                self.dicoLabel2Nature[node['label']] = nature
                self.dicoLabel2Zone[node['label']] = node['zone']
                local = node.xmlGetTextNode()
                self.dicoLocal.append(local)
                self.natureNumber[nature] = self.natureNumber[nature] + 1


    def getListLabel(self):
        """
        Get list of labels for boundaries conditions
        """
        list = []
        for nature in ('inlet', 'outlet', 'wall', 'symmetry'):
            for node in self.node_def.xmlGetNodeList(nature):
                list.append(node['label'])
        return list
        
        
    def updateBoundaryConditions(self):
        """
        Update boundary conditions according to boundary definitions in XML file.
        """
        for nature in ('inlet', 'outlet', 'wall', 'symmetry'):
            for node in self.node_def.xmlGetChildNodeList(nature):
                label = node['label']
                self.setBoundaryConditions(nature, label)
        self.updateDicos()


    def setBoundaries(self, label, zone, nature, local):
        """
        Add to the XML doc the new values.
        """
        # 1 - In < boundary_definition > markup : creation of the nature markup
        # and the reference and/or group markup
        node = self.node_def.xmlInitNode(nature, label=label, zone=zone)
        if local not in self.dicoLocal:
            node.xmlSetTextNode(local)
            
        self.setBoundaryConditions(nature, label)


    def setBoundaryConditions(self, nature, label):
        """
        Add boundary conditions if not exist into the XML file.
        """
        # 2 - In < boundary_conditions > markup : creation of the nature markup
        # with the default values for velocity, turbulence, pressure and scalars
        node_cond = self.node_bc.xmlInitChildNode(nature, label=label)

        # if nature = 'wall', if thermal radiation, creation with default 
        # values for radiative transfer (NP 27-12-06)
        if nature == 'wall':
            ray_model = RadiativeBoundariesModel(self.case).getRadiativeTransfertModel()
            if ray_model != 'off':
                RadiativeBoundariesModel(self.case).getTypeOfCondition(label)

        if nature == 'inlet':
            self.initCondInletVelocity(node_cond)
            from Turbulence import TurbulenceModel
            list = TurbulenceModel(self.case).getTurbulenceVariable()
            del TurbulenceModel
            if list: 
                self.initCondTurbulence(node_cond, '1.0')
            self.setCondScalar(node_cond, 'dirichlet', '0') 
            
        if nature == 'outlet':
            self.setCondScalar(node_cond, 'neumann', '0')


    def initCondInletVelocity(self, node):
        """
        Initialization velocity's boundary conditions for inlet
        """
        c = node.xmlInitNode('velocity_pressure', 'choice') 
        if not c['choice']: 
            self.setDirichletVelocity(node, '0.0', '0.0', '0.0')


    def setDirichletVelocity(self, node, u, v, w):
        """
        Put thermal scalar node for <type label> under boundary conditions
        """
        n = node.xmlGetNode('velocity_pressure', 'choice')
        n['choice'] = 'dirichlet'
        n.xmlInitNode('dirichlet', name='velocity_U').xmlSetTextNode(u)
        n.xmlInitNode('dirichlet', name='velocity_V').xmlSetTextNode(v)
        n.xmlInitNode('dirichlet', name='velocity_W').xmlSetTextNode(w)


    def initCondTurbulence(self, node, value):
        """
        Initialization turbulence's boundary conditions
        """
        c = node.xmlInitNode('turbulence', 'choice')
        if not c['choice']:
            c['choice'] = "hydraulic_diameter"
            c.xmlSetData('hydraulic_diameter', value)


    def setDirichletTurbulence(self, node, tag, value):
        """
        Put thermal scalar node for <type label> under boundary conditions
        """
        n = node.xmlGetNode('turbulence', 'choice')
        n['choice'] = 'dirichlet'
        n.xmlInitNode('dirichlet', name=tag).xmlSetTextNode(value)


    def setCondScalar(self, node, choice, value):
        """
        Put scalar nodes for <type label> under boundary conditions
        """
        for scal in self.node_scalar.xmlGetNodeList('scalar'):
            label = scal['label']
            type = scal['type']
            n = node.xmlGetChildNode('scalar', label=label, type=type)
            if not n:
                node.xmlInitChildNode('scalar', choice=choice, label=label, type=type).xmlSetData(choice,value)


    def replaceBoundaryCondition(self, old_label, old_zone, old_nature, label, zone, nature, local):
        """
        Replace label,wall, and local for boundary condition 
        """
        node_def = self.node_def.xmlGetChildNode(old_nature, label=old_label, zone=old_zone)
        if node_def:
            node_def.xmlRemoveNode()
            self.updateDicos()
            self.setBoundaries(label, zone, nature, local)

        self.updateDicos()


    def cancelBoundaryCondition(self, label, nature):
        """
        Remove definition and boundary conditions nodes for nature-label
        """
        self.node_def.xmlRemoveChild(nature, label=label)
        self.updateDicos()
 

    def getLocalisation(self, label, nature):
        """
        Get value of localissation in self.case in XML file 
        """
        node = self.node_def.xmlGetChildNode(nature, label=label)
        local = node.xmlGetTextNode()
        return local

#-------------------------------------------------------------------------------
# Hlist multi-selection class
#-------------------------------------------------------------------------------


class MultiSelect:
    """
    This class is design in order to manage the storage of the selected items in
    the Hlist.
    """
    def __init__(self, Hlist, natureList, natureColor):
        """
        Constructor
        """
        self.Hlist = Hlist
        self.natureList  = natureList
        self.natureColor = natureColor


    def _initList(self):
        """
        For each mouse selection we do reinitialize all lists.
        """
        self.labelL   = []
        self.zoneL    = []
        self.natureL  = []
        self.localL   = []


    def boundaryInfo(self, entry):
        """
        Return info from the argument entry.
        """
        # Here we must call the tk function directly without passing in
        # Tix.py, because of a bug in tix8.1 in the Hlist.item_cget function
        #
        h = self.Hlist
        label  = h.tk.call(h, 'item', 'cget', entry, 0, '-text')
        zone   = h.tk.call(h, 'item', 'cget', entry, 1, '-text')
        nature = h.tk.call(h, 'item', 'cget', entry, 2, '-text')
        local  = h.tk.call(h, 'item', 'cget', entry, 3, '-text')

        for key in self.natureList.keys():
            if self.natureList[key] == nature: nature = key

        return label, zone, nature, local


    def setItems(self, entriesL):
        """
        Store values when a new selection is done.
        """
        self._initList()

        for entry in entriesL:
            label, zone, nature, local = self.boundaryInfo(entry)
            self.labelL.append(label)
            self.zoneL.append(zone)
            self.natureL.append(nature)
            self.localL.append(local)


    def backgroundColor(self):
        """
        Find the right color for the selection background.
        """
        nature = self.natureL[0]
        for i in self.natureL:
            if nature == i:
                iok = 1
            else:
                iok = 0

        if iok:
            self.Hlist.config(selectbackground=self.natureColor[nature])
        else:
            self.Hlist.config(selectbackground=wm['selectbackground'])


    def widgetText(self):
        """
        Find the right texts for the widget layout.
        """
        label, zone, nature, local = self.labelL[0], self.zoneL[0], self.natureL[0], self.localL[0]

        for i in self.labelL:
            if label != i: label = _MULTISEL
        for i in self.zoneL:
            if zone != i: zone = _MULTISEL
        for i in self.natureL:
            if nature != i: nature = _MULTISEL
        for i in self.localL:
            if local != i: local = _MULTISEL

        return label, zone, nature, local


#-------------------------------------------------------------------------------
# Create Hlist Widget Class
#-------------------------------------------------------------------------------


class CreateHlistWidgets:
    """
    This class create Hlist View used by BoundConditions.py, ScalarsBoundaries.py 
    and RadiativeBoundaries.py.
    """
    def __init__(self, parent, balloon, selectmode, selectHlist):
        self.frame = parent
        self.balloon = balloon
        self.selectmode = selectmode
        self.selectHlist = selectHlist

        t = PageText()
        self.natureList =  {'wall':      t.WALL      ,
                            'inlet':     t.INLET     ,
                            'outlet':    t.OUTLET    ,
                            'symmetry':  t.SYMMETRY  }

        # Definition of color
        #
        self.natureColor = {'wall':      "blue"   ,
                            'inlet':     "red"    ,
                            'outlet':    "brown"  ,
                            'symmetry':  "green"  }

        self._createHlistWidgets(self.selectHlist)


    def _make_style(self):
        """
        Define beautyful style in order to display columns in the Hlist.
        """
        self.style={}

        self.style['header'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.w1,
                                                anchor=CENTER,
                                                padx=5, pady=2,
                                                font=fB)

        for nature in self.natureList.keys():
            self.style[nature+'b'] = Tix.DisplayStyle(Tix.TEXT,
                                                    refwindow=self.w1,
                                                    padx=5,
                                                    fg=self.natureColor[nature],
                                                    selectforeground='#fefefe',
                                                    font=fB)
            self.style[nature] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.w1,
                                                anchor=CENTER,
                                                padx=5,
                                                fg=self.natureColor[nature],
                                                selectforeground='#fefefe',
                                                font=fN)


    def _createHlistWidgets(self, selectHlist):
        t = PageText()

        self.w1 = Tix.Frame(self.frame, relief=FLAT)
        self.w1.pack(side=TOP, fill=X)


        # Put a simple hierachy into the Hlist (two levels).
        #
        self.h = Tix.ScrolledHList(self.w1, options='hlist.columns 5 hlist.header 1 ' )
        self.h.config(scrollbar="auto +y +x")

        # selectmode='single'
        # selectmode='browse'
        # selectmode='extended' -> multi select with <Shift> and <Ctrl>
        # selectmode='multiple' -> multi select only with <Shift>
        #
        self.h.hlist.config(selectmode=self.selectmode,
                            browsecmd=selectHlist,
                            bg=wm['hlistbackground'])
        self.h.hsb.config(width=10, bd=1)
        self.h.vsb.config(width=10, bd=1)
        self.h.pack(expand=1, fill=BOTH, padx=10, pady=10, side=TOP)

        self.balloon.bind_widget(self.h.hlist, statusmsg=t.MSG_HLIST)

        # First some styles for the headers
        #
        self._make_style()

        # Create the headers
        #
        self.h.hlist.header_create(0, itemtype=Tix.TEXT, text=t.LABEL,
                                   style=self.style['header'])
        self.h.hlist.header_create(1, itemtype=Tix.TEXT, text=t.ZONE,
                                   style=self.style['header'])
        self.h.hlist.header_create(2, itemtype=Tix.TEXT, text=t.NATURE,
                                   style=self.style['header'])
        self.h.hlist.header_create(3, itemtype=Tix.TEXT, text=t.LOCAL,
                                   style=self.style['header'])

        # Notice that we use 5 columns in the Hlist widget. This way when the user
        # expands the windows wide, the right side of the header doesn't look
        # chopped off. The following line ensures that the 5 column header is
        # not shown unless the Hlist window is wider than its contents.
        #
        self.h.hlist.column_width(2,0)

        # Let configure the appearance of the Hlist subwidget 
        #
        self.h.hlist.config(separator='.', width=25, height=11, drawbranch=0, indent=10)

        # Let configure the width of the columns
        #
        self.h.hlist.column_width(0, chars=16)
        self.h.hlist.column_width(1, chars=6)
        self.h.hlist.column_width(2, chars=9)
        self.h.hlist.column_width(3, chars=30)


#-------------------------------------------------------------------------------
# Main View Class
#-------------------------------------------------------------------------------


class DefBCView(TkPage.Page):
    """
    Class to open Turbulence Page.
    """
    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.label  = Tix.StringVar()
        self.zone   = Tix.StringVar()
        self.local  = Tix.StringVar()
        self.nature = Tix.StringVar()
        self.color  = Tix.StringVar()


    def _pageControl(self):
        """
        Instantiate the definition boundary modelling class.
        """
        self.mdl = DefBCModel(self.case)

        #  Initialize variables concerning the display of the Hlist
        #
        self.selectedEntry = None
        self.currentEntry = None
        self.entriesNumber = 0


    def getNature(self, event=None):
        """
        Return the selected nature of boundary condition. Whatever the language,
        type is allways in ('wall','inlet','outlet','symmetry').
        """
        nature = self.nature.get()
        for key in self.natureList.keys():
            if self.natureList[key] == nature: nature = key
        return nature


    def getLabel(self, event=None):
        """
        Return the name of the boundary condition. It 's not allowed to have
        blank or not ASCII caracters in this name.
        """
        label = self.label.get()
        if len(label) > 32:
            label = label[:LABEL_LENGTH_MAX]
            if label in self.mdl.getListLabel():
                default = {}
                default['label'] = label
                default['list'] = self.mdl.getListLabel()
                dialog = Dialog.VerifyExistenceLabel(self.myPage, default=default)
                select = dialog.result.copy()
                label = select['label']
        return  string.join(string.split(label), '_')


    def testInt(self, entry, var):
        """
        Verify if the 'entry' contents an integer. If not, signal an error
        and return None.
        """
        try:
            i = int(var.get())
            if i <= 0: 
                self.check2.error(entry)
                i = -999
        except:
            self.check2.error(entry)
            i = -999

        return i



    def string2int(self, entry, var):
        """
        If the StringVar() 'var' associated to the 'entry', is not a '_MULTISEL',
        convert it to an integer.
        """
        self.check2.begin(entry)

        # Several selections in Hlist

        if self.currentEntry and len(self.currentEntry) > 1:
            if var.get() != _MULTISEL:
                i = self.testInt(entry, var)
            else:
                i = _MULTISEL

        # Only one selection in Hlist
        else:
            i = self.testInt(entry, var)

        return i
        

    def getZone(self, event=None):
        """
        Return an integer for number of voluminal zone.
        """
        if self.zone.get():
            zone = self.string2int(self.ezone, self.zone)
        else:
            zone = None
        return zone


    def getLocalisation(self, event=None):
        """
        Return a string with references and perhaps some words.
        The separation will be done later.
        """
        return self.local.get()


    def eraseEntries(self, event=None):
        """
        This button is for delete all caracters in the three entries.
        """
        self.elabel.delete(0, END)
        self.ezone.delete(0, END)
        self.elocal.delete(0, END)


    def insertHlist(self, label, zone, nature, local):
        """
        Create the item Hlist associated with the new boundary definition.
        """
        self.stbar.busy()

        self.entriesNumber = self.entriesNumber +1
        self.mdl.natureNumber[nature] = self.mdl.natureNumber[nature] +1
        self.mdl.zoneNumber = self.mdl.zoneNumber +1

        iok = 0
        name = 'item' + repr(self.entriesNumber)
        n = repr(self.entriesNumber)
        if not zone:
            zone = str(n)
            if zone in self.mdl.dicoLabel2Zone.values():
                for i in range(self.mdl.zoneMaxNumber):
                    i = i+1
                    if repr(i) not in self.mdl.dicoLabel2Zone.values() and iok == 0:
                        n = repr(i)
                        iok = 1
                        
            zone = str(n)
            name = 'item' + n

        if not label: 
            label = nature + "_" + n
            if label in self.mdl.dicoLabel2Nature.keys():
                for j in range(self.mdl.natureNumber[nature]):
                    j = j+1
                    if repr(j) not in self.mdl.dicoLabel2Nature.keys():
                        label = nature + "_" + j

        self.h.hlist.add(name, itemtype=Tix.TEXT, text=label,
                         style=self.style[nature+'b'])

        self.h.hlist.item_create(name, 1, itemtype=Tix.TEXT,
                                 text=zone,
                                 style=self.style[nature])

        self.h.hlist.item_create(name, 2, itemtype=Tix.TEXT,
                                 text=self.natureList[nature],
                                 style=self.style[nature])

        self.h.hlist.item_create(name, 3, itemtype=Tix.TEXT,
                                 text=local,
                                 style=self.style[nature])

        self.stbar.idle()
        return label, zone


    def createHlist(self, event=None):
        """
        Upload the contents of the definition widget
        when the 'create' button is pushed.
        Checking consistency of values and interpreting
        complexe entry for reference if necessary.
        """
        self.stbar.busy()
        t = PageText()

        label = self.getLabel()
        if label == _MULTISEL:
            self.stbar.idle()
            return

        nature  = self.getNature()

        zone = self.getZone()
        if zone == -999: 
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.MSG_ZONE)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

        if zone == _MULTISEL:
            self.stbar.idle()
            return

        local = self.getLocalisation()
        if not local:
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.NO_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

        if label in self.mdl.getListLabel():
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LABEL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

        if str(zone) in self.mdl.dicoLabel2Zone.values():
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_ZONE)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

        if local in self.mdl.dicoLocal:
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

        try:
            label, zone = self.insertHlist(label, zone, nature, local)
            self.mdl.setBoundaries(label, str(zone), nature, local)
            self.mdl.updateDicos()
        except:
            pass

        self.mdl.updateBoundaryConditions()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def replaceHlist(self, entry, label, zone, nature, local):
        """
        Add the item Hlist associated with the new boundary definition.
        """
        self.stbar.busy()
        # Update the Hlist
        self.h.hlist.item_configure(entry, 0, text=label, style=self.style[nature+'b'])
        self.h.hlist.item_configure(entry, 1, text=zone, style=self.style[nature])
        self.h.hlist.item_configure(entry, 2, text=self.natureList[nature], style=self.style[nature])
        self.h.hlist.item_configure(entry, 3, text=local, style=self.style[nature])
        self.h.hlist.config(selectbackground=self.natureColor[nature])

        self.stbar.idle()


    def setEditedValueHlist(self, entry, old_label, old_zone, old_nature, old_local):
        """
        Checking consistency of values and interpreting
        complexe entry for reference if necessary.
        """
        t = PageText()
        new_label = self.getLabel()
        if new_label == _MULTISEL or not new_label:
            new_label = old_label

        new_zone = self.getZone()
        if new_zone == _MULTISEL or not new_zone:
            new_zone = old_zone
            if new_zone == -999: 
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.MSG_ZONE)
                return
                
        new_nature = self.b1.menubutton['text']

        if new_nature == _MULTISEL:
            new_nature = old_nature
        else:
            new_nature = self.getNature()


        if len(self.currentEntry) == 1:
            new_local   = self.getLocalisation()
        else:
            new_local   = old_local

        if not new_local: 
# ajouter popup ????
            return
        if new_label != old_label and len(self.currentEntry) == 1:
            if new_label in self.mdl.getListLabel():
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LABEL)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return

        if str(new_zone) != old_zone:
            if str(new_zone) in self.mdl.dicoLabel2Zone.values():
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_ZONE)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return

        if new_local != old_local:
            if new_local in self.mdl.dicoLocal:
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return
                
        self.mdl.replaceBoundaryCondition(old_label, old_zone, old_nature, 
                                          new_label, str(new_zone), new_nature, new_local)
        self.replaceHlist(entry, new_label, str(new_zone), new_nature, new_local)
        

    def editHlist(self, event=None):
        """
        Modify previous selected entries. A new node is created with
        the new specifications and the previous ChildNodes are cloned
        in order to taking into account the user's boundary conditions
        if they exist.
        """
        if not self.currentEntry: return
        self.stbar.busy()
        t = PageText()

        for entry in self.currentEntry:
            label, zone, nature, local = self.select.boundaryInfo(entry)
            self.setEditedValueHlist(entry, label, zone, nature, local) 

        if len(self.currentEntry) >1:
            label0, zone0, nature0, local0 = self.select.widgetText()
            entry0 = self.currentEntry[0]
            if nature0 != _MULTISEL:
                label = self.getLabel()
                if label != _MULTISEL :
                    new_local = string.join(self.select.localL, ' or ')
                    new_zone = min(self.select.zoneL)
                    
                    i = 0
                    for entry in self.currentEntry:
                        self.mdl.cancelBoundaryCondition(label, self.getNature())
                        if entry != entry0:
                            i = i + 1
                            self.h.hlist.delete_entry(entry)
                    self.mdl.setBoundaries(label, new_zone, self.getNature(), new_local)
                    self.replaceHlist(entry0, label, new_zone, self.getNature(), new_local)
                    

        self.mdl.updateBoundaryConditions()
        self.mdl.updateDicos()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def cancelHlist(self, event=None):
        """
        Just delete the current selected entries from the Hlist and
        of course from the XML file.
        """
        if not self.currentEntry: return
        self.stbar.busy()
        for entry in self.currentEntry:
            label, zone, nature, local = self.select.boundaryInfo(entry)
            self.mdl.cancelBoundaryCondition(label, nature)

            self.entriesNumber = self.entriesNumber -1
            self.mdl.natureNumber[nature] = self.mdl.natureNumber[nature] -1
##            self.mdl.zoneNumber = self.mdl.zoneNumber -1
            self.h.hlist.delete_entry(entry)
        self.mdl.updateDicos()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def importHlist(self, event=None):
        """
        Import from SALOME.
        """
        from CFDSTUDYGUI_DataModel import _getStudy,_getEngine
        from CFDSTUDYGUI_Commons import sg, sgPyQt
        from salome import lcc
        import smesh
        import GEOM

        aStudy = _getStudy()
        aSMESH_SO = aStudy.FindComponent("SMESH")
        aGEOM_SO = aStudy.FindComponent("GEOM")

        if aSMESH_SO == None and aGEOM_SO == None:
            return

        #loading IORs
        builder = aStudy.NewBuilder()
        if aSMESH_SO != None:
            aSMESHEngine = lcc.FindOrLoadComponent( "FactoryServer", "SMESH" )
            builder.LoadWith( aSMESH_SO , aSMESHEngine )
        if aGEOM_SO != None:
            aGEOMEngine = lcc.FindOrLoadComponent( "FactoryServer", "GEOM" )
            builder.LoadWith( aGEOM_SO , aGEOMEngine )
            
        local = ""
        if sg.SelectedCount() > 0:
            for i in range (sg.SelectedCount()):
                entry = sg.getSelected( i )
                if entry != '':
                    sobj = aStudy.FindObjectID( entry )
                    if sobj !=  None:
                        #print "sobj=", sobj
                        anObjectDS = sobj.GetObject()
                        if anObjectDS !=  None:

                            #check for smesh group
                            aSmeshObject = anObjectDS._narrow( smesh.SMESH_Group )
                            if aSmeshObject != None and aSmeshObject.GetType() == smesh.FACE:
			        if not local:
                                    local = aSmeshObject.GetName()
			        else:
                                    local = local + ' or ' + aSmeshObject.GetName()

                            #check for geom group of faces
                            aGeomObject = anObjectDS._narrow( GEOM.GEOM_Object )
                            if aGeomObject != None and aGeomObject.GetType() == 37:
                                #check the group
                                #get all possible faces
                                import geompy
                                all_ids = geompy.SubShapeAllIDs( aGeomObject.GetMainShape(), geompy.ShapeType["FACE"])
                                cur_ids = geompy.GetObjectIDs( aGeomObject )
                                isValid = len(cur_ids) > 0 # not include empty list
                                if isValid:
                                    for face_id in cur_ids:
                                        if not face_id in all_ids:
                                            #invalid id
                                            isValid = False
                                            break

                                if isValid:
                                    #try to add new group to the listbox
	        	            if not local:
                                        local = aSmeshObject.GetName()
			            else:
                                        local = local + ' or ' + aGeomObject.GetName()

        if local in self.mdl.dicoLocal:
            t = PageText()
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            return

	if local:
            nature = 'wall'
            label, zone = self.insertHlist("", 0, nature, local)
            self.mdl.setBoundaries(label, str(zone), nature, local)
            self.mdl.updateDicos()

            self.mdl.updateBoundaryConditions()
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
 

    def selectHlist(self, entry=None):
        """
        Browsecmd:
        Every single Clik on the Button1 of the mouse send TWO signal to the GUI
        the first one when man push the button, and the second one when the
        button gets back.

        Command:
        Every double-Click and Return send only ONE signal to the GUI.
        """
        self.selectedEntry = self.h.hlist.info_selection()

        if not self.selectedEntry:
          self.llocal.config(state=NORMAL)
          self.elocal.config(state=NORMAL)
          self.elocal.delete(0,END)
          
          self.lzone.config(state=NORMAL)
          self.ezone.config(state=NORMAL)
          self.ezone.delete(0,END)

        elif self.currentEntry != self.selectedEntry:
            self.currentEntry = self.selectedEntry
            self.select.setItems(self.currentEntry)
            self.select.backgroundColor()
            label, zone, nature, local = self.select.widgetText()

            # Update all widgets
            #
            if nature == _MULTISEL:
                self.b1.menubutton.config(text=nature)
            else:
                self.b1.config(value=self.natureList[nature])

            self.elabel.delete(0,END)
            self.elabel.insert(0,label)
            self.elabel.icursor(END)
            if nature == _MULTISEL:
                self.elabel.config(state=DISABLED)
            else: 
                self.elabel.config(state=NORMAL)
 
            
            self.ezone.delete(0,END)
            self.ezone.insert(0,zone)
            self.ezone.icursor(END)
            
            self.elocal.delete(0,END)
            self.elocal.insert(0,local)
            self.elocal.icursor(END)
            
            if len(self.currentEntry) > 1:
                self.lzone.config(state=DISABLED)
                self.ezone.config(state=DISABLED)
                
                self.llocal.config(state=DISABLED)
                self.elocal.config(state=DISABLED)
            else:
                self.elabel.config(state=NORMAL)
                self.elabel.delete(0,END)
              
                self.llocal.config(state=NORMAL)
                self.elocal.config(state=NORMAL)
                self.elocal.delete(0,END)
              
                self.lzone.config(state=NORMAL)
                self.ezone.config(state=NORMAL)
                self.ezone.delete(0,END)
              
                self.elabel.delete(0,END)
                self.elabel.insert(0,label)
                self.elabel.icursor(END)
            
                self.ezone.delete(0,END)
                self.ezone.insert(0,zone)
                self.ezone.icursor(END)
                
                self.elocal.delete(0,END)
                self.elocal.insert(0,local)
                self.elocal.icursor(END)


    def selectHlistAll(self):
        """
        Fonction for the Popup Menu.
        Select everything in the Hlist.
        """
        try:
            first = 'item' + repr(1)
            last  = 'item' + repr(self.entriesNumber)
            self.h.hlist.selection_set(first, last)
            self.currentEntry = None
            self.selectHlist()
        except:
            pass


    def cancelSelection(self):
        """
        Fonction for the Popup Menu.
        Blank the selection.
        """
        self.h.hlist.selection_clear()
        self.elabel.config(state=NORMAL)
        self.lzone.config(state=NORMAL)
        self.ezone.config(state=NORMAL)
        self.llocal.config(state=NORMAL)
        self.elocal.config(state=NORMAL)

        self.b1.config(value=self.nature.get())
        self.currentEntry = None
        self.eraseEntries()


    def natureSelection(self, nature):
        """
        Fonction for the Popup Menu.
        Select item in the Hlist by nature.
        WARNING: the Tix function self.h.hlist.selection_includes(entry)
        doesn't work. One has to use self.h.hlist.selection_set(entry).
        """
        self.cancelSelection()

        entriesList = []
        for num in range(1, self.entriesNumber +1):
            entry = 'item' + repr(num)
            l, z, n, local = self.select.boundaryInfo(entry)
            if nature == n:
                entriesList.append(entry)

        if entriesList:
            for entry in entriesList:
                self.h.hlist.selection_set(entry)
            self.currentEntry = None
            self.selectHlist()


    def searchListEnv(self, event=None):
        """
        Research a Preprocessor listing to pick colors or groups of faces.
        """
        t = PageText()
        self.stbar.busy()

        file_name = preprocessorFile(self.master, self.case['resu_path'])

        if self.mdl.dicoLocal: 
            Dialog.ShowInfo(self.master,
                            title = t.INFO,
                            text  = t.INFO_LISTENV)
        else:
            if file_name:
                self.info = Informations(file_name, 'faces')
                refList, groupList = self.info.updateListes()
                for ref in refList:
                    lab = ''
                    zone = ''
                    lab, zone = self.insertHlist(lab, zone,'wall', ref)
                    self.mdl.setBoundaries(lab, zone, 'wall', ref)
                    self.mdl.updateDicos()

                for grp in groupList:
                    lab = ''
                    zone = ''
                    lab, zone = self.insertHlist(lab, zone, 'wall', grp)
                    self.mdl.setBoundaries(lab, zone, 'wall', grp)
                    self.mdl.updateDicos()

        self.stbar.idle()


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()

        self.lf1 = Tix.LabelFrame(self.myPage, bd=2, label=t.TITLE, relief=FLAT)
        self.lf1.label.config(font=fT)
        self.lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        hl = CreateHlistWidgets(self.lf1.frame, self.balloon, 'extended', self.selectHlist)
        self.h = hl.h
        self.style = hl.style
        self.natureList = hl.natureList
        self.natureColor = hl.natureColor

        # Popup Menu of Tree
        #
        pm = Tix.PopupMenu(self.h.hlist, title=t.POPUP, state=NORMAL)
        pm.bind_widget(self.h.hlist)
        pm.menu.add_separator()
        pm.menu.add_command(label=t.SELECT_ALL, command=self.selectHlistAll)
        pm.menu.add_command(label=t.UNSELECT, command=self.cancelSelection)
        pm.menu.m1 = Tix.Menu(pm.menu)
        pm.menu.add_cascade(label=t.SELECT_TYPE, menu=pm.menu.m1)
        for key in self.natureList.keys():
            pm.menu.m1.add_command(label=self.natureList[key],
                                   command=TkPage.Callback(self.natureSelection, key))


        # Create the mouse selection instance
        #
        self.select = MultiSelect(self.h.hlist, self.natureList, self.natureColor)

        # Separator
        #
        s = Tix.Frame(self.lf1.frame, height=2, bd=2, relief=SUNKEN)
        s.pack(side=TOP, fill=X, pady=5)


        f1 = Tix.Frame(self.lf1.frame, relief=FLAT)
        f1.pack(side=TOP, fill=X)
        
        Tix.Label(f1, text=t.LABEL).grid(row=0, column=0, padx=3, pady=5, sticky=E)
        self.elabel = Tix.Entry(f1, bd=2, width=15, textvariable=self.label)
        self.elabel.grid(row=0, column=1, padx=5, pady=5, sticky=W)
        self.elabel.bind('<Return>', self.createHlist)
        
        self.lzone = Tix.Label(f1, text=t.ZONE)
        self.lzone.grid(row=1, column=0, padx=3, pady=5, sticky=E)
        self.ezone = Tix.Entry(f1, bd=2, width=15, textvariable=self.zone)
        self.ezone.grid(row=1, column=1, padx=5, pady=5, sticky=W)
        self.ezone.bind('<Return>', self.createHlist)
        
        Tix.Label(f1, text=t.NATURE).grid(row=2, column=0, padx=3, pady=5, sticky=E)
        self.b1 = Tix.OptionMenu(f1, options='menubutton.width 10')
        self.b1.menubutton.config(bd=2, relief=RAISED)
        self.b1.config(variable=self.nature, command=self.getNature)
        for key in self.natureList.keys():
            self.b1.add_command(self.natureList[key], label=self.natureList[key])
        self.b1.grid(row=2, column=1, padx=5, pady=5, sticky=W)
        
        self.llocal = Tix.Label(f1, text=t.LOCAL)
        self.llocal.grid(row=3, column=0, padx=3, pady=5, sticky=E)
        self.elocal = Tix.Entry(f1, bd=2, width=45, textvariable=self.local)
        self.elocal.grid(row=3, column=1, padx=5, pady=5, sticky=W)
        self.elocal.bind('<Return>', self.createHlist)
    
        self.box1 = Tix.ButtonBox(f1, orientation=HORIZONTAL, relief=FLAT)
        self.box1.add('create', text=t.CREATE, command=self.createHlist)
        self.box1.add('modify', text=t.MODIFY, command=self.editHlist)
        self.box1.add('erase',  text=t.ERASE,  command=self.cancelHlist)
        if self.case['salome']:
            self.box1.add('import',  text=t.SALOME,  command=self.importHlist)
        self.box1.grid(row=4, column=0, columnspan=2, padx=30, pady=5)
            
        # Separator
        #
        s = Tix.Frame(self.lf1.frame, height=2, bd=2, relief=SUNKEN)
        s.pack(side=TOP, fill=X, pady=5)


        f2 = Tix.Frame(self.lf1.frame, relief=FLAT)
        f2.pack(side=TOP, fill=X)

        Tix.Label(f2, relief=FLAT, text=t.FIC_ENV).grid(row=0,column=0,padx=15,pady=15)

        self.img = Tix.PhotoImage(file=PIC_PATH+'search1.gif')
        env = Tix.Button(f2, image=self.img, command=self.searchListEnv)
        env.grid(row=0,column=1,padx=15,pady=15)


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        for label in self.mdl.dicoLabel2Nature.keys():
            nature = self.mdl.dicoLabel2Nature[label]
            zone = self.mdl.dicoLabel2Zone[label]
            local = self.mdl.getLocalisation(label, nature)
            self.insertHlist(label, zone, nature, local)
        self.mdl.updateDicos()
            
#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.AUTO_TITLE    = "AUTOMATIQUE"
            self.BAD_LABEL     = "Attention : cette tiquette ne peut tre utilise\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre tiquette"
            self.BAD_LOCAL     = "Attention : cette localisation ne peut tre utilise\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre localisation"
            self.NO_LOCAL      = "Attention : localisation obligatoire pour cration\n\n"\
                                 "Donnez une localisation"
            self.BAD_NATURE    = "Attention : ce type de conditions limites ne peut tre\n\n"\
                                 "utilis qu'une seule et unique fois.\n\n"\
                                 "Donnez un autre type de conditions limites"
            self.BAD_ZONE      = "Attention : ce numro de zone ne peut tre utilis\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre valeur"
            self.CREATE        = "Crer"
            self.ERASE         = "Effacer"
            self.FIC_ENV       = "Importer les groupes et les couleurs\n"\
                                 "d'un listing du Prprocesseur"
            self.GROUP         = "Groupe"
            self.INFO          = "Informations"
            self.INFO_LISTENV  = "Attention : des localisations de faces de bord "\
                                 " ont dj t dclares.\n\n "\
                                 "Il faut les supprimer pour importer des donnes"\
                                 " du listing."
            self.INFO_DELETE   = "Attention : la modification de la dfinition d'une"\
                                " condition limite entrane la perte des informations"\
                                " associes  la prcdente dfinition."
            self.INLET         = "entre"
            self.LABEL         = "tiquette"
            self.MANUAL_TITLE  = "MANUEL"
            self.MODIFY        = "Modifier"
            self.OUTLET        = "sortie"
            self.POPUP         = "Menu popup"
            self.LOCAL         = "Localisation"
            self.SALOME        = "Importer de SALOME"
            self.SELECT_ALL    = "Tout slectionner"
            self.SORT_OUT      = "Trier par"
            self.SELECT_TYPE   = "Slection par type"
            self.SYMMETRY      = "symtrie"
            self.TITLE         = "Dfinition des zones frontires"
            self.NATURE        = "Nature"
            self.UNSELECT      = "Dslectionner"
            self.VISU_TITLE    = "VISUALISATION"
            self.WALL          = "paroi"
            self.ZONE          = "Zone"
        else:
            self.AUTO_TITLE    = "AUTOMATIC"
            self.BAD_LABEL     = "Warning: this label can be used \n\n"\
                                  "only one and single time. \n\n"\
                                  "Give another label"
            self.BAD_LOCAL     = "Warning: this localization can be used \n\n"\
                                  "only one and single time. \n\n"\
                                  "Give another localization"
            self.NO_LOCAL      = "Warning: You must obligatorily give a localization "
            self.BAD_NATURE    = "Warning: this type of boundaries conditions can be used \n\n"\
                                 "only one and single time.\n\n"\
                                 "Give another type of boundaries conditions"
            self.BAD_ZONE      = "Warning: this number of zone can be used \n\n"\
                                 "only one and single time.\n\n"\
                                 "Give another value"
            self.CREATE        = "Create"
            self.ERASE         = "Delete"
            self.FIC_ENV       = "Import groups and references from Preprocessor listing"
            self.GROUP         = "Group"
            self.INFO          = "Informations"
            self.INFO_LISTENV  = "Warning: some localizations already have given.\n\n"\
                                 "You must destroy them to import data from listing"
            self.INFO_DELETE   = "Warning  the modification of a region boundary"\
                                " definition occurs the lost of data about the previous"\
                                " definition."
            self.INLET         = "inlet"
            self.LABEL         = "Label"
            self.MANUAL_TITLE  = "MANUAL"
            self.MODIFY        = "Modify"
            self.OUTLET        = "outlet"
            self.POPUP         = "Menu popup"
            self.LOCAL         = "Localisation"
            self.SALOME        = "Import from SALOME"
            self.SELECT_ALL    = "Select all"
            self.SELECT_TYPE   = "Selection by type"
            self.SORT_OUT      = "Sort out by"
            self.SYMMETRY      = "symmetry"
            self.TITLE         = "Definition of boundary regions"
            self.NATURE        = "Nature"
            self.UNSELECT      = "Unselect"
            self.VISU_TITLE    = "VISUALIZATION"
            self.WALL          = "wall"
            self.ZONE          = "Zone"

        # 2) Messages
        #
        if Tool.GuiParam.lang == 'fr':
            self.MSG_HLIST   = "Cliquer le bouton droit pour "       +\
                               "le menu contextuel."
            self.MSG_ZONE    = "zone doit tre une valeur entire positive"
        else:
            self.MSG_HLIST   = "Click right button for popup menu."
            self.MSG_ZONE    = "zone must be an positive integer value"


#-------------------------------------------------------------------------------
# Turbulence test case
#-------------------------------------------------------------------------------


class DefBCTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """
        This method is executed before all "check" methods.
        """
        from Base.XMLstuff import TkGuiModel
        self.case = TkGuiModel(None)


    def tearDown(self):
        """
        This method is executed after all "check" methods.
        """
        del self.case


    def checkDefBCModelInstantiation(self):
        """Check whether the DefBCModel class could be instantiated"""
        model = None
        model = DefBCModel(self.case)
        assert model != None, 'Could not instantiate DefBCModel'


    def checksetBoundaryDefandCond(self):
        """
        """
        model = DefBCModel(self.case)
        model.xmlSetBoundaryDefandCond('inlet', 'porte', '4', None)
        truc = model.node_bc.toString()
        
        doc = '<boundary_conditions>'\
                  '<boundary_definition>'\
                      '<inlet label="porte">'\
                          '<reference>4</reference>'\
                      '</inlet>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                       '<scalar choice="" label="temperature_celsius" type="thermal"/>'\
                       '<scalar choice="" label="scal1" type="user"/>'\
                  '</inlet>'\
              '</boundary_conditions>'

        assert doc == truc, 'Could not set the boundary reference number'

        model.xmlSetBoundaryDefandCond('inlet', 'porte', None, 'titi')
        truc = model.node_bc.toString()

        doc = '<boundary_conditions>'\
                  '<boundary_definition>'\
                      '<inlet label="porte">'\
                          '<reference>4</reference>'\
                          '<group>titi</group>'\
                      '</inlet>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                       '<scalar choice="" label="temperature_celsius" type="thermal"/>'\
                       '<scalar choice="" label="scal1" type="user"/>'\
                  '</inlet>'\
              '</boundary_conditions>'

        assert doc == truc, 'Could not set the boundary group string'



def suite():
    testSuite = unittest.makeSuite(DefBCTestCase, "check")
    return testSuite

    
def runTest():
    runner = unittest.TextTestRunner()
    runner.run(suite())


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
