# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the 'Additional user's scalars' page.

This module defines the following classes:
- MultiSelect
- DefineUserScalarsModel
- DefineUserScalarsView
- PageText
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import string
import Tix
import unittest
from Tkconstants import *


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Variables
from Base.XMLvariables import Model
from Base.XMLmodel import XMLmodel
from DefineBoundaryRegions import DefBCModel
from DefineVolumicZones import DefineVolumicZonesModel

#-------------------------------------------------------------------------------
# Multi-selection string
#-------------------------------------------------------------------------------


MULTISEL = "*****"

#-------------------------------------------------------------------------------
# Global constants
#-------------------------------------------------------------------------------

GRAND   =  1.0e+12
SPGRAND = 1e+12
SMGRAND = -1e+12


#-------------------------------------------------------------------------------
# Hlist multi-selection class
#-------------------------------------------------------------------------------


class MultiSelect:
    """
    This class is design in order to manage the storage of the selected items in
    the Hlist.
    """
    def __init__(self, case, Hlist, nodeScalar):
        """
        Constructor
        """
        self.case       = case
        self.Hlist      = Hlist
        self.nodeScalar = nodeScalar


    def _initList(self):
        """
        For each mouse selection we reinitialize all lists.
        """
        self.labelL = []
        self.initL = []
        self.variL = []
        self.miniL = []
        self.maxiL = []
        self.mdl = DefineUserScalarsModel(self.case)


    def scalarInfo(self, entry):
        """
        Return info from the argument entry.
        """
        # Here we must call the tk function directly without passing in
        # Tix.py, because of a bug in tix8.1 in the Hlist.item_cget function
        #
        t = PageText()
        h = self.Hlist
        label = h.tk.call(h, 'item', 'cget', entry, 0, '-text')
        init  = h.tk.call(h, 'item', 'cget', entry, 1, '-text')
        vari  = h.tk.call(h, 'item', 'cget', entry, 2, '-text')

        if vari == "--": vari = t.NO_VARIANCE

        return label, float(init), vari


    def setItems(self, entriesL):
        """
        Store values when a new selection is done.
        """
        self._initList()

        for entry in entriesL:
            label, init, vari = self.scalarInfo(entry)
            self.labelL.append(label)
            self.initL.append(init)
            self.variL.append(vari)

            list = self.mdl._getScalarList()
            for nlabel in list:
                if nlabel == label:
                    mini = self.mdl.getScalarMinValue(nlabel)
                    maxi = self.mdl.getScalarMaxValue(nlabel)
                    self.miniL.append(mini)
                    self.maxiL.append(maxi)


    def getItems(self):
        """
        Return all the lists.
        """
        return self.labelL, self.initL, self.variL, self.miniL, self.maxiL


    def widgetText(self):
        """
        Find the right texts for the widget layout.
        """
        label, init, vari = self.labelL[0], self.initL[0], self.variL[0]
        mini, maxi       = self.miniL[0], self.maxiL[0]

        for i in self.labelL:
            if label != i: 
                label = MULTISEL

        for i in self.initL:
            if init != i: 
                init = MULTISEL

        for i in self.variL:
            if vari != i: 
                vari = MULTISEL

        for i in self.miniL:
            if i == '' or i == None: i = 0
            if mini != i: 
                mini = MULTISEL

        for i in self.maxiL:
            if i == '' or i == None: i = 1.0e+12
            if maxi != i: 
                maxi = MULTISEL

        if init != MULTISEL: init = "%g" % (init)
        if mini != MULTISEL: mini = "%g" % (mini)
        if maxi != MULTISEL: maxi = "%g" % (maxi)

        return label, init, vari, mini, maxi


#-------------------------------------------------------------------------------
# Define User Scalars model class
#-------------------------------------------------------------------------------


class DefineUserScalarsModel(Variables, Model):
    """
    Useful methods for operation of the page. 
    __method : private methods for the Model class. 
    _method  : private methods for the View Class
    """
    def __init__(self, case):
        """
        Constructor
        """
        self.case = case

        # TODO: the InitZoneView class !
##        zone_label = "1"
##        self.zone = zone_label

        self.node_th_sca    = self.case.xmlGetNode('thermal_scalar')
        self.node_user_sca  = self.case.xmlGetNode('additional_scalars')
        self.node_bc        = self.case.xmlGetNode('boundary_conditions')

        self.dicoCoeff = {}
        self.dicoVariance = {}
        for scalar in self.getUserScalarList():
            self.dicoCoeff[scalar] = self.getScalarDiffusivityLabel(scalar)
            self.getScalarVariance(scalar)


    def defaultValues(self):
        """Return the default values"""
        default = {}
        default['initial_value'] = 0.0
        default['min_value'] = -1e+12
        default['max_value'] = 1e+12
        default['diffusion_coefficient'] = 1.83e-05
        default['temperature_celsius'] = 20.0
        default['temperature_kelvin'] = 293.15
        default['enthalpy'] = 297413.
        default['zone'] = '1'
        
        return default


    def _getScalarList(self):
        """Return the User scalar label list (thermal scalar included)"""
        list = []
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            list.append(node['label'])
        return list


    def getUserScalarList(self):
        """Return the User scalar label list (without thermal scalar)"""
        list = []
        for node in self.node_user_sca.xmlGetNodeList('scalar', type='user'):
            list.append(node['label'])
        return list


    def setScalarBoundaries(self):
        """Input boundaries conditions for a scalar node""" 
        for node in self.node_bc.xmlGetChildNodeList('inlet'):
            DefBCModel(self.case).setCondScalar(node, 'dirichlet', '0.0')
        for node in self.node_bc.xmlGetChildNodeList('outlet'):
            DefBCModel(self.case).setCondScalar(node, 'neumann', '0.0')


    def __deleteScalarBoundaries(self, label):
        """
        Delete scalar nodes for scalar named label
        """
        for nature in ('inlet','outlet','wall'):
            for node in self.node_bc.xmlGetChildNodeList(nature):
                for n in node.xmlGetChildNodeList('scalar'):
                    if n['label'] == label: n.xmlRemoveNode()


    def addNewUserScalar(self, zone, scalar_label=None):
        """Input a new additional_scalar with label scalar_label""" 
        num = str(1)
        if not scalar_label:
            if self.dicoCoeff:
                i = 1
                while (("Dscal"+str(i)) in self.dicoCoeff.values()):
                    i = i + 1
                num = str(i)
            else: 
                num = str(1)
            scalar_label = "scalar" + num
        else:
            l = len(self.dicoCoeff)
            if l != 0 and scalar_label not in self.dicoCoeff.keys():
                num = str(l+1)

        self.dicoCoeff[scalar_label] = "Dscal" + num
        self.__setNewUserScalar(zone, scalar_label)
        self.updateScalarDiffusivityName()

        return scalar_label


    def __setNewUserScalar(self, zone, scalar_label):
        """Input a new additional_scalar with label scalar_label""" 
        list = self._getScalarList()
        # TODO : boundary_conditions, diffusivity
        if scalar_label not in list:
            s1 = self.node_user_sca.xmlInitNode('scalar', type="user", label=scalar_label)
            def_ini_value   = self.defaultValues()['initial_value']
            def_min_val     = self.defaultValues()['min_value']
            def_max_val     = self.defaultValues()['max_value']
            self.setScalarInitialValue(zone, scalar_label, def_ini_value)
            self.setScalarMinValue(scalar_label, def_min_val)
            self.setScalarMaxValue(scalar_label, def_max_val)
            self.setScalarDiffusivity(scalar_label)
            self.setScalarBoundaries()


    def setScalarLabel(self, old_label, new_label):
        """Modify old_label of scalar with new_label and put new label if variancy exists""" 
        label = new_label[:LABEL_LENGTH_MAX]
        if label not in self.getUserScalarList():
            for node in self.node_user_sca.xmlGetNodeList('scalar'):
                if node['label'] == old_label: 
                    node['label'] = label
                    if old_label in self.dicoCoeff.keys():
                        coeff = self.dicoCoeff[old_label]
                        del self.dicoCoeff[old_label]
                        self.dicoCoeff[label] = coeff
                if node.xmlGetString('variance') == old_label:
                    node.xmlSetData('variance', label)
                    for i in range(len(self.dicoVariance)):
                        if self.dicoVariance.values()[i] == old_label:
                            self.dicoVariance.values()[i] = label


    def getThermalScalarLabel(self):
        """
        Get label for thermal scalar
        """
        label = ""
        node = self.node_user_sca.xmlGetNode('scalar', type='thermal')
        if node: label = node['label']
        
        return label

##
##    def deleteUserScalar(self, scalar_label):
##        """Delete an additional_scalar node with label scalar_label""" 
##        node = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
##        node.xmlRemoveNode()
##        # TODO : boundary_conditions, variances associes
##        self.__deleteScalarBoundaries(scalar_label)


    def setScalarInitialValue(self, zone, scalar_label, initial_value):
        """
        Put initial value for an additional_scalar with label scalar_label
        and zone zone
        """
        # we verify args are good
        self.isFloat(initial_value)
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        if not n:
            raise ValueError, "this scalar does not exist"
        nz = n.xmlInitChildNode('initial_value', zone=zone)
        if nz:
            nz.xmlSetTextNode(initial_value)


    def getScalarInitialValue(self, scalar_label, zone):
        """
        Get initial value from an additional_scalar with label scalar_label
        and zone zone
        """
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        n_init = n.xmlGetChildNode('initial_value', zone=zone)
        if n_init: 
            val = n.xmlGetDouble('initial_value', zone=zone)
        else:
            if n['type'] == 'thermal':
                val = self.defaultValues()[n['name']]
            else:
                val = self.defaultValues()['initial_value']
            self.setScalarInitialValue(zone, scalar_label, val)

        return val


    def setScalarMinValue(self, scalar_label, min_value):
        """Put minimal value for an additional_scalar with label scalar_label"""
        # we verify min_value is a float value
        self.isFloat(min_value)
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        if min_value == '' or min_value ==  None:
            n.xmlSetData('min_value', self.defaultValues()['min_value'])
        else:
            n.xmlSetData('min_value', min_value)


    def getScalarMinValue(self, scalar_label):
        """Get minimal value from an additional_scalar with label scalar_label"""
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        n_min = n.xmlGetChildNode('min_value')
        if n_min: 
            min_val = n.xmlGetDouble('min_value')
        else:
            min_val = self.defaultValues()['min_value']
            if self.getScalarVariance(scalar_label) == "":
                self.setScalarMinValue(scalar_label, min_val)
            
        return min_val


    def setScalarMaxValue(self, scalar_label, max_value):
        """Put maximal value for an additional_scalar with label scalar_label"""
        # we verify max_value is a float value
        self.isFloat(max_value)
        
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        if max_value == '' or max_value == None:
            n.xmlSetData('max_value', self.defaultValues()['max_value'])
        else:
            n.xmlSetData('max_value', max_value)


    def getScalarMaxValue(self, scalar_label):
        """Get maximal value from an additional_scalar with label scalar_label"""
        n = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        n_max = n.xmlGetChildNode('max_value')
        if n_max: 
            max_val = n.xmlGetDouble('max_value')
        else:
            max_val = self.defaultValues()['max_value']
            if self.getScalarVariance(scalar_label) == "":
                self.setScalarMaxValue(scalar_label, max_val)
        return max_val


    def setScalarVariance(self, scalar_label, variance_label):
        """Put variance of an additional_scalar with label scalar_label"""
        n = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
        n.xmlSetData('variance',variance_label)
        self.__deleteNode(scalar_label, 'property')
        self.setScalarMinValue(scalar_label, 0.0)
        self.setScalarMaxValue(scalar_label, self.defaultValues()['max_value'])
        self.dicoVariance[scalar_label] = variance_label


    def getScalarVariance(self, scalar_label):
        """Get variance of an additional_scalar with label scalar_label"""
        var = ""
        n = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
        if n:
            n_var = n.xmlGetChildNode('variance')
            if n_var: 
                var = n.xmlGetString('variance')
                if var not in self.dicoVariance.values():
                    self.dicoVariance[scalar_label] = var

        return var


    def getScalarVarianceList(self):
        """
        Return list of scalars which are also a variance
        """
        list = []
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            if node.xmlGetString('variance'):
                list.append(node.xmlGetString('variance'))

        return list


    def getVarianceLabelFromScalarLabel(self, label):
        """
        Get the label of scalar with variancy'label label
        """
        lab = ""
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            if node.xmlGetString('variance') == label:
                lab = node['label']
        return lab


    def setScalarDiffusivity(self, scalar_label):
        """
        Put default initial values of property "diffusivity" for an additional_scalar 
        with label scalar_label
        """
        #TODO : on ne peut pas modifier la diffusivity d'une variance
        if scalar_label not in self.getScalarVarianceList():
            n = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
            self.setScalarDiffusivityLabel(scalar_label)
            if not self.getScalarDiffusivityChoice(scalar_label):
                self.setScalarDiffusivityChoice(scalar_label, 'constant')
            if not self.getScalarDiffusivityInitialValue(scalar_label):
                def_ini_val = self.defaultValues()['diffusion_coefficient']
                self.setScalarDiffusivityInitialValue(scalar_label,def_ini_val)


    def updateScalarDiffusivityName(self):
        """
        Put suffixe at name fo diffusivity's property 'diffusion_coefficient'
        for an additional_scalar
        """
        list = []
        n = 0
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            n = n + 1
            dif_name = 'diffusion_coefficient_' + str(n)
            nprop = node.xmlGetChildNode('property')
            if nprop:
                nprop['name'] = dif_name
##            else:
##                if self.getScalarVariance(node['label']) == '':
##                    node.xmlInitChildNode('property', name=dif_name)
##                    n = n + 1



    def setScalarDiffusivityLabel(self, scalar_label):
        """
        Put label of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
        n = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
##        self.updateScalarDiffusivityName()
        n_dif = n.xmlGetChildNode('property')
        if n_dif:
            n_dif['label'] = self.dicoCoeff[scalar_label]
        else:
            n.xmlInitChildNode('property', label=self.dicoCoeff[scalar_label])


    def getScalarDiffusivityLabel(self, scalar_label):
        """
        Get label of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
        lab_diff = ''
        n = self.node_user_sca.xmlGetNode('scalar', type='user', label=scalar_label)
        n_diff = n.xmlGetChildNode('property')
        if n_diff:
            lab_diff = n_diff['label']
            
        return lab_diff


    def setScalarDiffusivityInitialValue(self, scalar_label, initial_value):
        """
        Set initial value of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
        # we verify initial_value is a float value
        self.isFloat(initial_value)
        
        for n in self.node_user_sca.xmlGetNodeList('scalar', type='user'):
            if n['label'] == scalar_label:
                n_diff = n.xmlInitChildNode('property')
                n_diff.xmlSetData('initial_value', initial_value)


    def getScalarDiffusivityInitialValue(self, scalar_label):
        """
        Get initial value of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
        for n in self.node_user_sca.xmlGetNodeList('scalar', type='user'):
            if n['label'] == scalar_label:
                n_diff = n.xmlGetChildNode('property')
                if n_diff:
                    diffu = n_diff.xmlGetDouble('initial_value')
        return diffu


    def _setStatusPrintAndRecordStatus(self, node, choice):
        """
        Set status of listing_printing and postprocessing_recording
        if choice is variable
        """
        list = ['constant', 'variable']
        self.isInList(choice, list)
        nod = node.xmlGetChildNode('property')

        if nod['choice'] == "constant":
            nod.xmlInitNode('listing_printing')['status'] = "off"
            nod.xmlInitNode('postprocessing_recording')['status'] = "off"
        elif nod['choice'] == "variable":
            if nod.xmlGetChildNode('listing_printing'): 
                nod.xmlRemoveChild('listing_printing')
            if nod.xmlGetChildNode('postprocessing_recording'):
                nod.xmlRemoveChild('postprocessing_recording')


    def setScalarDiffusivityChoice(self, scalar_label, choice):
        """
        Set choice of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
        for n in self.node_user_sca.xmlGetNodeList('scalar', type='user'):
            if n['label'] == scalar_label:
                n_diff = n.xmlGetChildNode('property')
                if n_diff :
                    n_diff['choice'] = choice
                else:
                    n.xmlInitChildNode('property', choice=choice)
                self._setStatusPrintAndRecordStatus(n, choice)


    def getScalarDiffusivityChoice(self, scalar_label):
        """
        Get choice of diffusivity's property for an additional_scalar 
        with label scalar_label
        """
	choice = 'constant'
        for n in self.node_user_sca.xmlGetNodeList('scalar', type='user'):
            if n['label'] == scalar_label:
                n_diff = n.xmlGetChildNode('property')
                if n_diff:
                    choice = n_diff['choice']
        return choice


    def UsedNumberbyDefaultList(self):
        """Return the default label for scalar's diffusivity"""
        item_used = []
        for scal in self.dicoCoeff.keys():
            if scal[:6] == 'scalar':
                item_used.append(scal[-1:])
        return item_used


    def getNumScalarLabelandDiffusivityLabelDefault(self, scalar_label=None):
        """Return the default label for scalar's diffusivity"""
        if self.dicoCoeff:
            i = 1
            while (("Dscal"+str(i)) in self.dicoCoeff.values()) \
                or (("scalar"+str(i)) in self.dicoCoeff.keys()): 
                i = i + 1
            num = str(i)
        else: 
            num = str(1)
        diffLabel = "Dscal" + num

        if scalar_label:
            self.dicoCoeff[scalar_label] = diffLabel
        else:
            self.dicoCoeff["scalar" + num] = diffLabel

        return num, diffLabel


    def setScalarValues(self, label, zone, init, mini, maxi, vari):
        """Put values to scalar with label label"""
        # we verify values are good
        self.isFloat(init)
        self.isFloat(mini)
        self.isFloat(maxi)
        
        t = PageText()

        type = 'user'

        if label not in self.getUserScalarList():
            if label not in self._getScalarList():
                self.addNewUserScalar(zone, label)
            else: 
                type = 'thermal'
                self.setScalarMinValue(label, mini)
                self.setScalarMaxValue(label, maxi)
 
        self.setScalarInitialValue(zone, label, init)
        if type == 'user':
            if vari != t.NO_VARIANCE:
                self.setScalarVariance(label, vari)
            else:
                self.__deleteNode(label, 'variance')
                self.setScalarMinValue(label, mini)
                self.setScalarMaxValue(label, maxi)
                self.setScalarDiffusivity(label)
        self.updateScalarDiffusivityName()


    def __deleteNode(self, label, tag):
        """
        Delete node tagged for scalar with label label
        """
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            if node['label'] == label:
                node.xmlRemoveChild(tag)


    def deleteScalar(self, label):
        """
        Delete node scalar with label label
        """
        for node in self.node_user_sca.xmlGetNodeList('scalar'):
            if node['label'] == label:
                node.xmlRemoveNode()
                self.__deleteScalarBoundaries(label)
        self.updateScalarDiffusivityName()


    def getScalarListtoDelete(self, label, list):
        """
        Get list of scalar to delete beyond scalar has variancy or no
        this list may be empty
        """
        list.append(label)       

        for i in range(len(self.dicoVariance)):
            if label == self.dicoVariance.values()[i]:
                scal = self.dicoVariance.keys()[i]
                if scal in self.dicoVariance.values():
                    self.getScalarListtoDelete(scal, list)
                else:
                    list.append(scal)
        
        return list


    def setNewLabelintoBoundCondScalar(self, old_label, new_label):
        """
        Replace the old label by new ones into boundary conditions 
        for scalar in the XML doc 
        """
        for nature in ('inlet','outlet','wall'):
            for node in self.node_bc.xmlGetChildNodeList(nature):
                for n in node.xmlGetChildNodeList('scalar', type='user'):
                    if n['label'] == old_label: n['label'] = new_label 

    def _getTypeScalar(self, scalar_label):
        """
        Return type of scalar for choice of color (for view)
        """
        type = 'user'
        node = self.node_user_sca.xmlGetNode('scalar', label=scalar_label)
        if node['type'] == 'thermal':
            type = 'thermal'
            
        return type


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------


class DefineUserScalarsView(TkPage.Page):
    """
    Class to open Define User Scalars Page.
    """

    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
#        self.case[name]['depends'] = ['scalar/prosca',
#                                      'bounda/scalar']
        self.case[name]['depends'] = ['scalar/prosca']


    def _pageControl(self):
        """
        Default value settings.
        """

        # Class attribute 'self.currentEntry' represents the selected entries
        # in the Hlist with information allready displayed in the widgets.
        # In the other side, 'self.selectedEntry' contents the last selected entries
        # from the Hlist to be displayed (but not yet). If 'self.currentEntry' and
        # 'self.selectedEntry' are different so the 'self.selectedEntry' becomes
        # the 'self.currentEntry'.
        #
        self.selectedEntry = None
        self.currentEntry  = None
        self.entriesNumber = 0
        self.label2Item     = {}

        self.mdl = DefineUserScalarsModel(self.case)
        self.volzone = DefineVolumicZonesModel(self.case)


    def _make_style(self):
        """
        Define beautiful style in order to display columns in the Hlist.
        """
        self.style={}

        self.style['header'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.lf1,
                                                anchor=CENTER,
                                                padx=5, pady=2,
                                                font=fB)
        self.style['style'+'b'] = Tix.DisplayStyle(Tix.TEXT,
                                                   refwindow=self.lf1,
                                                   padx=5,
                                                   selectforeground='#fefefe',
                                                   font=fB)
        self.style['styleV'+'b'] = Tix.DisplayStyle(Tix.TEXT,
                                                    refwindow=self.lf1,
                                                    padx=5,
                                                    fg='brown',
                                                    font=fB)
        self.style['styleT'+'b'] = Tix.DisplayStyle(Tix.TEXT,
                                                    refwindow=self.lf1,
                                                    padx=5,
                                                    fg='red',
                                                    font=fB)
        self.style['style'] = Tix.DisplayStyle(Tix.TEXT,
                                               refwindow=self.lf1,
                                               padx=5, anchor=CENTER,
                                               selectforeground='#fefefe',
                                               font=fN)
        self.style['styleV'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.lf1,
                                                padx=5, anchor=CENTER,
                                                fg='brown',
                                                font=fN)

        self.style['styleT'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.lf1,
                                                padx=5, anchor=CENTER,
                                                fg='red',
                                                font=fN)


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        Note: 'self.init', 'self.mini' and 'self.maxi' are StringVar() and
        not a DoubleVar() because of the 'MULTISEL' contents possibility.
        """
        self.label = Tix.StringVar()
        self.init  = Tix.StringVar()
        self.vari  = Tix.StringVar()
        self.mini  = Tix.StringVar()
        self.maxi  = Tix.StringVar()
        self.zone_label = Tix.StringVar()


    def getNumberZone(self, event=None):
        """
        INPUT label for choice of zone
        """
        self.stbar.busy()
        self.zone = self.zone_label.get()
        self.initializeScalar(self.zone)
        self.stbar.idle()


    def getVari(self, event=None):
        """
        Return the variance value from the ComboBox.
        """
        t = PageText()
        self.e4.config(state=NORMAL)        
        self.e5.config(state=NORMAL)
        self.l4.config(state=NORMAL)
        self.l5.config(state=NORMAL)

        if self.vari.get() == "":
            self.vari.set(t.NO_VARIANCE)

        elif self.vari.get() != t.NO_VARIANCE:
            self.mini.set('0.0')
            self.maxi.set(SPGRAND)
            self.e4.config(state=DISABLED)
            self.e5.config(state=DISABLED)
            self.l4.config(state=DISABLED)
            self.l5.config(state=DISABLED)

        return self.vari.get()


    def isFloat(self, entry, var):
        """
        Verify if the 'entry' contents a float. If not, signal an error
        and return None.
        """
        try:
            double = float(var.get())
        except:
            self.check2.error(entry)
##            double = None
            double = ''

        return double


    def string2float(self, entry, var):
        """
        If the StringVar() 'var' associated to the 'entry', is not a 'MULTISEL',
        convert it to a float.
        """
        self.check2.begin(entry)

        # Several selections in Hlist
        #
        if self.currentEntry and len(self.currentEntry) > 1:
            if var.get() != MULTISEL:
                double = self.isFloat(entry, var)
            else:
                double = MULTISEL

        # Only one selection in Hlist
        #
        else:
            double = self.isFloat(entry, var)

        return double


    def getVerifyLabel(self, label):
        """
        Verify length of label, cut if it's more longer than 32 characters
        """ 
        t =PageText()
        new = label
        if len(label) > 32:
            new = label[:LABEL_LENGTH_MAX]
            if new in self.mdl._getScalarList():
                default = {}
                default['label'] = new
                default['list'] = self.mdl._getScalarList()
                dialog = Dialog.VerifyExistenceLabel(self.myPage, title=t.VERIFY_LABEL, default=default)
                select = dialog.result.copy()
                new = select['label']
        return new


    def getInit(self, event=None):
        """
        Return the initialization value, which can be
        1) a float, if all selections have the same initialization value,
        2) a 'MULTISEL' string if all selections have not the same initialization value,
        3) 'None' if there is an error.
        """
        if not self.init.get(): self.init.set('0.0')
        return self.string2float(self.e2, self.init)


    def getMini(self, event=None):
        """
        Return the min value, which can be
        1) a float, if all selections have the same min value,
        2) a 'MULTISEL' string if all selections have not the same min value,
        3) 'None' if there is an error.
        """
##        if not self.mini.get(): self.mini.set(SMGRAND)
        if self.mini.get() == '': self.mini.set(SMGRAND)
        return self.string2float(self.e4, self.mini)


    def getMaxi(self, event=None):
        """
        Return the min value, which can be
        1) a float, if all selections have the same max value,
        2) a 'MULTISEL' string if all selections have not the same max value,
        3) 'None' if there is an error.
        """
##        if not self.maxi.get(): self.maxi.set(SPGRAND)
        if self.maxi.get() == '': self.maxi.set(SPGRAND)
        return self.string2float(self.e5, self.maxi)


    def insertHlist(self, label, init, vari):
        """
        Create the item Hlist associated with the new boundary definition
        and update the listBox
        """
        self.stbar.busy()
        t = PageText()

        self.entriesNumber = self.entriesNumber + 1

        scal_num = 'item' + str(self.entriesNumber)
        self.label2Item[label] = scal_num
        
        type = self.mdl._getTypeScalar(label)

        if vari == t.NO_VARIANCE: vari = "--"

        # If the scalar is a variance, it is displayed with a specific color.
        #
        if vari != "--":
            self.h.hlist.add(scal_num, itemtype=Tix.TEXT, text=label,
                             style=self.style['styleV'+'b'])
            self.h.hlist.item_create(scal_num, 1, itemtype=Tix.TEXT,
                                     text="%g"%(init),
                                     style=self.style['styleV'])
            self.h.hlist.item_create(scal_num, 2, itemtype=Tix.TEXT,
                                     text=vari,
                                     style=self.style['styleV'])
        elif type == 'thermal':
            self.h.hlist.add(scal_num, itemtype=Tix.TEXT, text=label,
                             style=self.style['styleT'+'b'])
            self.h.hlist.item_create(scal_num, 1, itemtype=Tix.TEXT,
                                     text="%g"%(init),
                                     style=self.style['styleT'])
            self.h.hlist.item_create(scal_num, 2, itemtype=Tix.TEXT,
                                     text=vari,
                                     style=self.style['styleT'])
        else:
            self.h.hlist.add(scal_num, itemtype=Tix.TEXT, text=label,
                             style=self.style['style'+'b'])
            self.h.hlist.item_create(scal_num, 1, itemtype=Tix.TEXT,
                                     text="%g"%(init),
                                     style=self.style['style'])
            self.h.hlist.item_create(scal_num, 2, itemtype=Tix.TEXT,
                                     text=vari,
                                     style=self.style['style'])

        self.e3.insert(END, label)
        self.stbar.idle()


    def replaceHlist(self, item, old, label, init, vari):
        """
        Update the 'item' into the Hlist.
        """
        self.stbar.busy()
        t = PageText()

        self.label2Item[label] = item
        type = self.mdl._getTypeScalar(old)


        if vari == t.NO_VARIANCE: vari = "--"

        # If the scalar is a variance, it is displayed with a specific color.
        #
        if vari != "--":
            self.h.hlist.item_configure(item, 0, itemtype=Tix.TEXT, text=label,
                                        style=self.style['styleV'+'b'])
            self.h.hlist.item_configure(item, 1, itemtype=Tix.TEXT,
                                        text="%g"%(init),
                                        style=self.style['styleV'])
            self.h.hlist.item_configure(item, 2, itemtype=Tix.TEXT,
                                        text=vari,
                                        style=self.style['styleV'])
        elif type == 'thermal':
            self.h.hlist.item_configure(item, 0, itemtype=Tix.TEXT, text=label,
                                        style=self.style['styleT'+'b'])
            self.h.hlist.item_configure(item, 1, itemtype=Tix.TEXT,
                                        text="%g"%(init),
                                        style=self.style['styleT'])
            self.h.hlist.item_configure(item, 2, itemtype=Tix.TEXT,
                                        text=vari,
                                        style=self.style['styleT'])
        else:
            self.h.hlist.item_configure(item, 0, itemtype=Tix.TEXT, text=label,
                                        style=self.style['style'+'b'])
            self.h.hlist.item_configure(item, 1, itemtype=Tix.TEXT,
                                        text="%g"%(init),
                                        style=self.style['style'])
            self.h.hlist.item_configure(item, 2, itemtype=Tix.TEXT,
                                        text=vari,
                                        style=self.style['style'])
        self.stbar.idle()


    def eraseEntries(self, event=None):
        """
        Delete all caracters in the entries.
        """
        t = PageText()
        self.e1.config(state=NORMAL)
        self.e1.delete(0, END)
        self.e2.delete(0, END)
        self.vari.set(t.NO_VARIANCE)
        self.e4.delete(0, END)
        self.e5.delete(0, END)


    def checkInitMinMax(self, name, init, mini, maxi):
        """
        Verify the coherence between init, mini and maxi
        """
        OK = 1

        if mini > maxi:
            t = PageText()
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.MSG_CLIPPING+name)

        elif init < mini or init > maxi:
            t = PageText()
            Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_INIT+name)
            OK = 0

        return OK


    def createHlist(self,event=None):
        """
        Upload contents of the definition widget
        when the "Create" button is pushed.
        """
        self.stbar.busy()
        t = PageText()

        # Get back data from entries
        #
        lab = self.label.get()
        lab = self.getVerifyLabel(lab)
        label = self.mdl.addNewUserScalar(self.zone, lab)
        vari = self.getVari()
        init = self.getInit()
##        if init != None: mini = self.getMini()
##        if init != None and mini != None: maxi = self.getMaxi()
        if init != '': mini = self.getMini()
        if init != '' and mini != '': maxi = self.getMaxi()
        
        # Verify if everything is OK
        #
        if vari not in self.mdl.getScalarVarianceList():

            if label not in self.e3.slistbox.listbox.get(0,END) and \
                    label != MULTISEL and init != '' and \
                    mini != '' and maxi != '':
##                   label != MULTISEL and init != None and \
##                   mini != None and maxi != None:

                if self.checkInitMinMax(label, init, mini, maxi):

                # Update the Hlist and the XML doc with the new data
                #
                    self.insertHlist(label, init, vari)
                    self.mdl.setScalarValues(label, self.zone, init, mini, maxi, vari)
        else:
            self.stbar.set(t.MSG_VARIANCE)
##        self.mdl.updateScalarDiffusivityName()

        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def setEditedValueHlist(self, entry, old_label, old_init, 
                            old_vari,  old_mini, old_maxi):
        """
        Checking consistency of values and interpreting
        complexe entry for reference if necessary.
        """
        t = PageText()

        # Get back data from entries
        #
        new_label = self.getVerifyLabel(self.label.get())
        new_vari = self.getVari()
        new_init = self.getInit()
##        if new_init != None: new_mini = self.getMini()
##        if new_init != None and new_mini != None: new_maxi = self.getMaxi()
        if new_init != '': new_mini = self.getMini()
        if new_init != '' and new_mini != '': new_maxi = self.getMaxi()

        # Verify if everything is OK
        #
        if not self.mdl.dicoVariance.has_key(new_vari):

            if (new_label == old_label or new_label not in self.e3.slistbox.listbox.get(0,END) ) and \
               new_vari != old_label and new_init != '' and \
               new_mini != '' and new_maxi != '':
##               new_vari != old_label and new_init != None and \
##               new_mini != None and new_maxi != None:

                if new_init == MULTISEL: new_init = old_init
                if new_mini == MULTISEL: new_mini = old_mini
                if new_maxi == MULTISEL: new_maxi = old_maxi
                if new_label == MULTISEL: new_label = old_label
                if new_vari == MULTISEL: new_vari = old_vari

                if self.checkInitMinMax(old_label, new_init, new_mini, new_maxi): 

                # Update the Hlist and the XML doc with the new data
                #
                    del self.label2Item[old_label]
                    self.replaceHlist(entry, old_label, new_label, new_init, new_vari)
                    self.mdl.setScalarValues(old_label, self.zone, new_init, new_mini, new_maxi, new_vari)

                # What to do if the label has changed?
                #
                    if new_label != MULTISEL and new_label != old_label:

                    # Input the new label of the scalar in 'self.node_user_sca'
                    #
                        self.mdl.setScalarLabel(old_label, new_label)

                    # If the scalar with the new label has a variance
                    #
                        label = self.mdl.getVarianceLabelFromScalarLabel(new_label)
                        if label:
                            item = self.label2Item[label]
                            self.h.hlist.item_configure(item, 2, itemtype=Tix.TEXT,
                                                    text=new_label,
                                                    style=self.style['styleV'])
                    # Update the listbox
                    #
                        list = self.e3.slistbox.listbox.get(0,END)
                        for index in range(len(list)):
                            if list[index] == old_label:
                                self.e3.slistbox.listbox.delete(index)
                                self.e3.insert(index, new_label)

                    # Input the new label of the scalar in 'self.node_bc'
                    #
                        self.mdl.setNewLabelintoBoundCondScalar(old_label, new_label)

        else:
            self.stbar.set(t.MSG_VARIANCE)

        self.stbar.idle()


    def editHlist(self, event=None):
        """
        Modify previous selected entries. A new node is created with
        the new specifications and the previous ChildNodes are cloned
        in order to taking into account the user's boundary conditions
        if they exist.
        """
        if not self.currentEntry: return

        self.stbar.busy()

        label, init, vari, mini, maxi = self.select.getItems()
        # Search scalar to modify
        #
        for i in range(len(self.currentEntry)):
            entry = self.currentEntry[i]
            for nlabel in self.mdl._getScalarList():
                # Modify the scalar's properties one by one
                if nlabel == label[i]:
                    self.setEditedValueHlist(entry,
                                             label[i], init[i],
                                             vari[i], mini[i], maxi[i])

##        self.mdl.updateScalarDiffusivityName()

        self.selectHlist()
        self.stbar.idle()


    def cancelScalar(self, label):
        """
        Just delete the current selected entries from the Hlist and
        of course from the XML file.
        """
        # we get list of scalars to delete

        list = []
        list_label = self.mdl.getScalarListtoDelete(label, list)
        
        boxlist = self.e3.slistbox.listbox.get(0,END)
        
        for name in list_label:
            self.mdl.deleteScalar(name)
            if self.mdl.dicoVariance.has_key(name):
                del self.mdl.dicoVariance[name]
            for index in range(len(boxlist)):
                if boxlist[index] == name:
                    self.e3.slistbox.listbox.delete(index)
                    self.h.hlist.delete_entry(self.label2Item[name])
                    self.entriesNumber = self.entriesNumber + 1
                    del self.label2Item[name]

        for name in list_label:
            del self.mdl.dicoCoeff[name]


    def cancelHlist(self, event=None):
        """
        Just delete the current selected entries from the Hlist and
        of course from the XML file.
        """
        if not self.currentEntry: return

        self.stbar.busy()
        for entry in self.currentEntry:
            try:
                label, init, vari = self.select.scalarInfo(entry)
                self.cancelScalar(label)
            except :pass

##        self.mdl.updateScalarDiffusivityName()
        self.eraseEntries()
        self.currentEntry = None
##        self.selectHlist()
        self.stbar.idle()


    def selectHlist(self, entry=None):
        """
        Browsecmd:
        Every single Clik on the Button1 of the mouse send TWO signal to the GUI
        the first one when man push the button, and the second one when the
        button gets back.

        Command:
        Every double-Click and Return send only ONE signal to the GUI.
        """
        self.selectedEntry = self.h.hlist.info_selection()

        if not self.selectedEntry:
            self.l1.config(state=NORMAL)
            self.l3.config(state=NORMAL)
            self.e1.config(state=NORMAL)
            self.e3.config(state=NORMAL)

            self.e4.config(state=NORMAL)
            self.e5.config(state=NORMAL)
            self.l4.config(state=NORMAL)
            self.l5.config(state=NORMAL)

        elif self.currentEntry != self.selectedEntry:

            self.currentEntry = self.selectedEntry
            self.select.setItems(self.currentEntry)

            label, init, vari, mini, maxi = self.select.widgetText()

            self.e1.delete(0,END)
            self.e1.insert(0,label)
            self.e1.icursor(END)

            self.e2.delete(0,END)
            self.e2.insert(0,init)
            self.e2.icursor(END)

            self.vari.set(vari)

            self.e4.delete(0,END)
            self.e4.insert(0,mini)
            self.e4.icursor(END)

            self.e5.delete(0,END)
            self.e5.insert(0,maxi)
            self.e5.icursor(END)

            if len(self.currentEntry) > 1:
                self.l1.config(state=DISABLED)
                self.l3.config(state=DISABLED)
                self.e1.config(state=DISABLED)
                self.e3.config(state=DISABLED)
            else:
                self.l1.config(state=NORMAL)
                self.l3.config(state=NORMAL)
                self.e1.config(state=NORMAL)
                self.box1.config(state=NORMAL)
                if self.mdl._getTypeScalar(label) == 'thermal':
                    self.l3.config(state=DISABLED)
                    self.e3.config(state=DISABLED)
                    self.box1.erase.config(state=DISABLED)
                else:
                    self.e3.config(state=NORMAL)
                self.label.set(label)
                

    def reloadHlist(self):
        """
        Destroy the contents of the Hlist, and reload the scalar data
        from the XMLdoc.
        """
        t = PageText()
        self.entriesNumber = 0
        self.label2Item = {}

        self.e3.slistbox.listbox.delete(0,END)
        self.e3.insert(0, t.NO_VARIANCE)

        self.h.hlist.delete_all()

        for nlabel in self.mdl._getScalarList():
            init = self.mdl.getScalarInitialValue(nlabel, self.zone)
            vari = self.mdl.getScalarVariance(nlabel)
            if vari in ("", "no variance", "no_variance"):
                vari = t.NO_VARIANCE
            self.insertHlist(nlabel, init, vari)


    def selectHlistAll(self):
        """
        Fonction for the Popup Menu.
        Select everything in the Hlist.
        """
        self.reloadHlist()
        try:
            first = 'item' + repr(1)
            last  = 'item' + repr(self.entriesNumber)
            self.h.hlist.selection_set(first, last)
            self.currentEntry = None
            self.selectHlist()
        except:
            pass


    def cancelSelection(self):
        """
        Fonction for the Popup Menu.
        Blank the selection.
        """
        self.h.hlist.selection_clear()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()


    def initializeScalar(self, zone):
        """
        INPUT label for choice of zone
        """
        t = PageText()
        self.reloadHlist()
        self.cancelSelection()
        self.vari.set(t.NO_VARIANCE)

        # on rajoute le scalaire thermique dans la liste des variances
        label = self.mdl.getThermalScalarLabel()
        if label: self.e3.insert(0, label)


    def _createWidgets(self):
        """
        Page layout
        """

        t = PageText()
        
        #########################################################
        #Zone button choice
        #########################################################

        fz = Tix.Frame(self.myPage, bd=2, relief=GROOVE)
        fz.pack(side=TOP, fill=X, padx=14, pady=10)
        
        f0 = Tix.Frame(fz, relief=FLAT)
        f0.pack(side=TOP, fill=X, padx=10, pady=10)

        ch_zone = Tix.Label(f0, text=t.ZONE_CHOICE)
        ch_zone.config(font=fB)
        ch_zone.grid(row=0, column=0, padx=5, pady=10, sticky=W)

        self.z = Tix.OptionMenu(f0, options='menubutton.width 15')
        self.z.menubutton.config(bd=2, relief=RAISED)

        for lab in self.volzone.getListLabel():
            self.z.add_command(self.volzone.dicoLabel2Zone[lab], label=lab)
        self.z.config(variable=self.zone_label, command=self.getNumberZone)
        self.z.grid(row=0, column=1, pady=10, padx=5)
        
        #########################################################
        #Scalar definition by zone
        #########################################################
        
        self.lf1 = Tix.LabelFrame(self.myPage, bd=2, label=t.TITLE, relief=FLAT)
        self.lf1.label.config(font=fT)
        self.lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        # Put a simple hierarchy into the Hlist (two levels).
        #
        self.h = Tix.ScrolledHList(self.lf1.frame, options=
                                   'hlist.columns 4 hlist.header 1')
        self.h.config(scrollbar="auto +x +y")
        self.h.hlist.config(selectmode='extended',
                            browsecmd=self.selectHlist,
                            bg=wm['hlistbackground'])
        self.h.hsb.config(width=10, bd=1)
        self.h.vsb.config(width=10, bd=1)
        self.h.pack(padx=10, pady=10, side=TOP)

        self.balloon.bind_widget(self.h.hlist, statusmsg=t.MSG_HLIST)

        # Create the headers
        #
        self._make_style()

        self.h.hlist.header_create(0, itemtype=Tix.TEXT, text=t.NAME,
                                   style=self.style['header'])
        self.h.hlist.header_create(1, itemtype=Tix.TEXT, text=t.INITIAL,
                                   style=self.style['header'])
        self.h.hlist.header_create(2, itemtype=Tix.TEXT, text=t.VARIANCE,
                                   style=self.style['header'])

        # Notice that we use 4 columns in the Hlist widget. This way when the user
        # expands the windows wide, the right side of the header doesn't look
        # chopped off. The following line ensures that the 4 column header is
        # not shown unless the Hlist window is wider than its contents.
        #
        self.h.hlist.column_width(col=3, width=0)

        # Let configure the appearance of the Hlist subwidget 
        #
        self.h.hlist.config(separator='.', width=39, height=11, drawbranch=0, indent=10)

        # width of the columns
        #
        self.h.hlist.column_width(col=0, chars=13)
        self.h.hlist.column_width(col=1, chars=13)
        self.h.hlist.column_width(col=2, chars=13)

        # Create the mouse selection instance
        #
        self.select = MultiSelect(self.case, self.h.hlist, self.mdl.node_user_sca)

        # Popup Menu of Tree
        #
        pm = Tix.PopupMenu(self.h.hlist, title=t.POPUP, state=NORMAL)
        pm.bind_widget(self.h.hlist)
        pm.menu.add_separator()
        pm.menu.add_command(label=t.SELECT_ALL, command=self.selectHlistAll)
        pm.menu.add_command(label=t.UNSELECT, command=self.cancelSelection)
        pm.menu.m1 = Tix.Menu(pm.menu)


        # Separator
        #
        s = Tix.Frame(self.lf1.frame, height=2, bd=2, relief=SUNKEN)
        s.pack(side=TOP, fill=X)

        f1 = Tix.Frame(self.lf1.frame, relief=FLAT)
        f1.pack(side=TOP, fill=X)

        self.l1 = Tix.Label(f1, text=t.NAME)
        self.l2 = Tix.Label(f1, text=t.INITIAL)
        self.l3 = Tix.Label(f1, text=t.VARIANCE)
        self.l4 = Tix.Label(f1, text=t.MIN)
        self.l5 = Tix.Label(f1, text=t.MAX)

        self.l1.grid(row=0, column=0, padx=3, pady=2, sticky=E)
        self.l2.grid(row=1, column=0, padx=3, pady=2, sticky=E)
        self.l3.grid(row=2, column=0, padx=3, pady=2, sticky=E)
        self.l4.grid(row=3, column=0, padx=3, pady=2, sticky=E)
        self.l5.grid(row=4, column=0, padx=3, pady=2, sticky=E)
        self.balloon.bind_widget(self.l4, balloonmsg=t.KEYWORD+'SCAMIN')
        self.balloon.bind_widget(self.l5, balloonmsg=t.KEYWORD+'SCAMAX')

        self.e3 = Tix.ComboBox(f1, editable=0, bd=2, listwidth=50, 
                               selectmode="", variable=self.vari)
        self.e3.entry.config(width=15, bg=wm['textbackground'])
        self.e3.slistbox.config(height=100, scrollbar="auto")
        self.e3.slistbox.vsb.config(width=10, bd=1)
        self.e3.slistbox.hsb.config(width=10, bd=1)

        self.e1 = Tix.Entry(f1, bd=2, width=15, textvariable=self.label)
        self.e2 = Tix.Entry(f1, bd=2, width=15, textvariable=self.init)
        self.e4 = Tix.Entry(f1, bd=2, width=10, textvariable=self.mini)
        self.e5 = Tix.Entry(f1, bd=2, width=10, textvariable=self.maxi)

        self.e1.grid(row=0, column=1, padx=5, pady=2,  sticky=W)
        self.e2.grid(row=1, column=1, padx=5, pady=2,  sticky=W)
        self.e3.grid(row=2, column=1,         pady=2,  sticky=W)
        self.e4.grid(row=3, column=1, padx=5, pady=2,  sticky=W)
        self.e5.grid(row=4, column=1, padx=5, pady=10, sticky=W)
        self.balloon.bind_widget(self.e4, balloonmsg=t.KEYWORD+'SCAMIN')
        self.balloon.bind_widget(self.e5, balloonmsg=t.KEYWORD+'SCAMAX')

        self.box1 = Tix.ButtonBox(f1, orientation=VERTICAL, relief=FLAT)
        self.box1.add('create', text=t.CREATE, command=self.createHlist)
        self.box1.add('modify', text=t.MODIFY, command=self.editHlist)
        self.box1.add('erase',  text=t.ERASE,  command=self.cancelHlist)
        self.box1.grid(row=0, column=2, rowspan=4, padx=15, pady=0, sticky=W)

        self.e3.config(bg=wm['textbackground'], command=self.getVari)


    def _initializeWidgets(self):
        """
        Get XML previous value.
        """
##        t = PageText()
##        self.reloadHlist()
##        self.cancelSelection()
##        self.vari.set(t.NO_VARIANCE)
##
##        # on rajoute le scalaire thermique dans la liste des variances
##        label = self.mdl.getThermalScalarLabel()
##        if label: self.e3.insert(0, label)
        self.zone = self.volzone.dicoLabel2Zone['all_cells']
        self.z.config(value = self.zone)
        self.initializeScalar(self.zone)

#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.CREATE           = "Crer"
            self.DEFINITION       = "DFINITION\nINITIALISATION"
            self.ERASE            = "Effacer"
            self.INITIAL          = "Valeur\ninitiale"
            self.MAX              = "Valeur\nmaximale"
            self.MIN              = "Valeur\nminimale"
            self.MODIFY           = "Modifier"
            self.NAME             = "Nom"
            self.NO_VARIANCE      = "pas de variance"
            self.POPUP            = "Menu popup"
            self.SELECT_ALL       = "Tout slectionner"
            self.TITLE            = "Dfinition des scalaires et Initialisation par zone"
            self.UNSELECT         = "Dslectionner"
            self.VARIANCE         = "Variance\ndu scalaire"
            self.INFO             = "Information"
            self.WARNING          = "Attention"
            self.KEYWORD          = "Mot cl Code_Saturne : "
            self.VERIFY_LABEL     = "Vrification du label"
            self.ZONE_CHOICE      = "Choix de la zone"

        else:
            self.CREATE           = "Create"
            self.DEFINITION       = "DEFINITION\nINITIALIZATION"
            self.ERASE            = "Erase"
            self.INITIAL          = "Initial\nvalue"
            self.MAX              = "Maximal\nvalue"
            self.MIN              = "Minimal\nvalue"
            self.MODIFY           = "Modify"
            self.NAME             = "Name"
            self.NO_VARIANCE      = "no variance"
            self.POPUP            = "Menu popup"
            self.SELECT_ALL       = "Select all"
            self.TITLE            = "User Scalar Definition and Initialization by zone"
            self.UNSELECT         = "Unselect"
            self.VARIANCE         = "Variance\nof scalar"
            self.INFO             = "Information"
            self.WARNING          = "Warning"
            self.KEYWORD          = "Code_Saturne key word: "
            self.VERIFY_LABEL     = "Checking of label"
            self.ZONE_CHOICE      = "Zone selection"


        # 2) Messages
        #
        if Tool.GuiParam.lang == 'fr':
            self.MSG_HLIST    = "Cliquer le bouton droit pour " \
                                "le menu contextuel."
            self.MSG_CLIPPING = "La valeur minimale est suprieure  la valeur "\
                                "maximale. Il n'y aura donc pas de clipping "\
                                "pour le scalaire :\n\n"
            self.MSG_INIT     = "La valeur initial doit tre comprise entre les "\
                                "valeurs minimale et maximale du scalaire :\n\n"
            self.MSG_LABEL    = "Le nom du scalaire est coup  8 caractres"
            self.MSG_VARIANCE = "Un scalaire ne peut pas tre la variance d'une variance"
        else:

            self.MSG_HLIST    = "Click right button for popup menu."
            self.MSG_CLIPPING = "The minimal value is greater than the maximal "\
                                "value. Therefore there will be no clipping for the "\
                                "scalar named:\n\n"
            self.MSG_INIT     = "The initial value must be set between the "\
                                "minimal and the maximal value of the scalar "\
                                "named:\n\n"
            self.MSG_LABEL    = "Only 8 characters are kept for Scalar name"
            self.MSG_VARIANCE = "Scalar can't be variance of another variance"


#-------------------------------------------------------------------------------
# DefineUsersScalars test case
#-------------------------------------------------------------------------------


class UserScalarTestCase(unittest.TestCase):
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        self.doc = XMLDocument()


    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case


    def checkDefineUserScalarsModelInstantiation(self):
        """Check whether the DefineUserScalarsModel class could be instantiated."""
        model = None
        model = DefineUserScalarsModel(self.case)

        assert model != None, 'Could not instantiate DefineUserScalarsModel'


    def checkAddNewUserScalar(self):
        """Check whether the DefineUserScalarsModel class could be set new scalar."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        truc = model.node_user_sca.toString()
        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                '<initial_value zone="1">0.0</initial_value>'\
                '<min_value>-1e+12</min_value>'\
                '<max_value>1e+12</max_value>'\
                '<property choice="constant" label="Dscal1" name="diffusion_coefficient_1">'\
                    '<listing_printing status="off"/>'\
                    '<postprocessing_recording status="off"/>'\
                    '<initial_value>1.83e-05</initial_value>'\
                '</property>'\
                '</scalar>'\
              '</additional_scalars>'

        assert doc == truc, 'Could not add a new scalar'

    def checkGetUserScalarList(self):
        """Check whether the DefineUserScalarsModel class could be get user list."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.addNewUserScalar(zone, 'tutu')
        list = model.getUserScalarList()
        assert list == ['toto', 'titi', 'tutu'], 'Could not get user scalar list'

##    def checkDeleteUserScalar(self):
##        """Check whether the DefineUserScalarsModel class could be delete one scalar from user list."""
##        model = DefineUserScalarsModel(self.case)
##        zone = '1'
##        model.addNewUserScalar(zone, 'toto')
##        model.addNewUserScalar(zone, 'titi')
##        model.addNewUserScalar(zone, 'tutu')
##        model.deleteUserScalar('titi')
##        list = model.getUserScalarList()
##        assert list == ['toto', 'tutu'], 'Could not delete user scalar list'

    def checkSetScalarlabel(self):
        """Check whether the DefineUserScalarsModel class could modify label."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone,'titi')
        model.setScalarLabel('titi', 'MACHIN')
        truc = model.node_user_sca.toString()
        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal1" name="diffusion_coefficient_1">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
                '<scalar label="MACHIN" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal2" name="diffusion_coefficient_2">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
              '</additional_scalars>'\
              
        assert doc == truc, 'Could not modify label of scalar'

    def checkSetScalarInitialValue(self):
        """Check whether the DefineUserScalarsModel class could be set initial value."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.setScalarInitialValue(zone, 'toto', 0.05)
        truc = model.node_user_sca.toString()
        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                    '<initial_value zone="1">0.05</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal1" name="diffusion_coefficient_1">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                  '</scalar>'\
                '</additional_scalars>'
        assert doc == truc, 'Could not set initial value to user scalar'

    def checkGetScalarInitialValue(self):
        """Check whether the DefineUserScalarsModel class could be get initial value."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.setScalarInitialValue(zone, 'toto', 0.05)
        value = model.getScalarInitialValue('toto', zone)
        assert value == 0.05,'Could not get initial value to user scalar'

    def checkSetScalarMinMaxValue(self):
        """Check whether the DefineUserScalarsModel class could be set min and max value."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarInitialValue(zone, 'toto', 0.05)
        model.setScalarMinValue('toto',0.01)
        model.setScalarMaxValue('titi',100.)
        truc = model.node_user_sca.toString()
        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                    '<initial_value zone="1">0.05</initial_value>'\
                    '<min_value>0.01</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal1" name="diffusion_coefficient_1">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
                '<scalar label="titi" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>100</max_value>'\
                    '<property choice="constant" label="Dscal2" name="diffusion_coefficient_2">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
              '</additional_scalars>'

        assert doc == truc,'Could not set min ansd max value to user scalar'

    def checkGetScalarMinMaxValue(self):
        """Check whether the DefineUserScalarsModel class could be get min and max value."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarInitialValue(zone, 'toto', 0.05)
        model.setScalarMinValue('toto',0.01)
        model.setScalarMaxValue('titi',100.)
        min = model.getScalarMinValue('toto')
        max = model.getScalarMaxValue('titi')
        
        assert min == 0.01,'Could not get min value to user scalar'
        assert max == 100.,'Could not get max value to user scalar'

    def checkSetScalarVariance(self):
        """Check whether the DefineUserScalarsModel class could be set variance."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarVariance('toto', 'titi')
        truc = model.node_user_sca.toString()

        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>0</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<variance>titi</variance>'\
                '</scalar>'\
                '<scalar label="titi" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal2" name="diffusion_coefficient_2">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
              '</additional_scalars>'

        assert doc == truc,'Could not set variance to user scalar'

    def checkGetScalarVariance(self):
        """Check whether the DefineUserScalarsModel class could be get variance."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarVariance('toto', 'titi')
        vari = model.getScalarVariance('toto')
        assert vari == 'titi','Could not get variance to user scalar'

    def checkGetScalarVarianceList(self):
        """Check whether the DefineUserScalarsModel class could be get variance."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.addNewUserScalar(zone, 'tutu')
        model.setScalarVariance('toto', 'titi')
        model.setScalarVariance('titi', 'tutu')
        list = model.getScalarVarianceList()
        assert list == ['titi', 'tutu'],'Could not get list of variance for scalars'


    def checkSetandGetScalarDiffusivityLabelandInit(self):
        """Check whether the DefineUserScalarsModel class could be get label of diffusivity."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarDiffusivityInitialValue('toto', 0.005)
        label = model.getScalarDiffusivityLabel('toto')
        val = model.getScalarDiffusivityInitialValue('toto')
        assert label == 'Dscal1',"Could not set or get label of user scalar's property"
        assert val == 0.005,"Could not set or get initial value of user scalar's property"


    def checkSetandGetScalarDiffusivityChoice(self):
        """Check whether the DefineUserScalarsModel class could be set diffusivity's choice."""
        model = DefineUserScalarsModel(self.case)
        zone = '1'
        model.addNewUserScalar(zone, 'toto')
        model.addNewUserScalar(zone, 'titi')
        model.setScalarDiffusivityChoice('toto', 'variable')
        truc = model.node_user_sca.toString()
        doc = '<additional_scalars>'\
                '<scalar label="toto" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="variable" label="Dscal1" name="diffusion_coefficient_1">'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
                '<scalar label="titi" type="user">'\
                    '<initial_value zone="1">0.0</initial_value>'\
                    '<min_value>-1e+12</min_value>'\
                    '<max_value>1e+12</max_value>'\
                    '<property choice="constant" label="Dscal2" name="diffusion_coefficient_2">'\
                        '<listing_printing status="off"/>'\
                        '<postprocessing_recording status="off"/>'\
                        '<initial_value>1.83e-05</initial_value>'\
                    '</property>'\
                '</scalar>'\
              '</additional_scalars>'
              
        assert doc == truc,"Could not set choice of user scalar's property"
              
        ch = model.getScalarDiffusivityChoice('toto')
        assert ch == "variable","Could not get choice of user scalar's property"


def suite():
    """unittest function"""
    testSuite = unittest.makeSuite(UserScalarTestCase, "check")
    return testSuite


def runTest():
    """unittest function"""
    print "UserScalarTestTestCase to be completed..."
    runner = unittest.TextTestRunner()
    runner.run(suite())


#-------------------------------------------------------------------------------
# End DefineUsersScalars
#-------------------------------------------------------------------------------
