# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the thermal scalar management.

This module contains the following classes and function:
- DefBCModel
- MultiSelect
- DefBCView
- PageText
- DefBCTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import sys, string
import Tix, unittest
from Tkconstants import *
import tkFileDialog


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Pages.RadiativeBoundaries import RadiativeBoundariesModel
from Pages.PreProcessingInformations import Informations, preprocessorFile


#-------------------------------------------------------------------------------
# Multi-selection string
#-------------------------------------------------------------------------------


_MULTISEL = "************"

#-------------------------------------------------------------------------------
# Thermal scalar model class
#-------------------------------------------------------------------------------


class DefineVolumicZonesModel:
    """
    """
    def __init__(self, case):
        """
        Constuctor.
        """
        self.case = case
        self.node_domain   = self.case.xmlGetNode('solution_domain')
        self.node_volzone  = self.node_domain.xmlInitNode('volumic_conditions')

        self.updateDicos()


    def getListLabel(self):
        """
        Get list of labels for volumics zones
        """
        list = []
        for node in self.node_volzone.xmlGetNodeList('zone'):
            list.append(node['label'])
        return list


    def getListStrZone(self):
        """
        Get list of volumics zones
        """
        list_zones= []
        for node in self.node_volzone.xmlGetNodeList('zone'):
            list_zones.append(str(node['name']))
        return list_zones

        
    def updateDicos(self):
        """
        Get list of labels for boundaries conditions
        """
        self.dicoLocal = []
        self.dicoLabel2Zone = {}
        self.zoneNumber = 0
        for node in self.node_volzone.xmlGetNodeList('zone'):
            self.zoneNumber = self.zoneNumber + 1
            self.dicoLabel2Zone[node['label']] = node['name']
##            local = self.getLocalisation(node['label'])
            local = node.xmlGetTextNode()
            self.dicoLocal.append(local)


    def setNewVolumicZone(self, label, zone, local):
        """
        Put new volumic zone in XML file 
        """
        node = self.node_volzone.xmlInitChildNode('zone', label=label, name=zone)
        if node:
            if local not in self.dicoLocal:
                node.xmlSetTextNode(local)


    def getZone(self, label):
        """
        Get zone number in name of zone balise in XML file
        """
        node = self.node_volzone.xmlGetChildNode('zone', label=label)
        if node:
            zone = node['name']
        return zone


    def replaceVolumicZone(self, old_label, old_zone, label, zone, local):
        """
        Replace values for old label in XML file
        """
        node = self.node_volzone.xmlGetChildNode('zone', label=old_label, name=old_zone)
        if node:
            node.xmlRemoveNode()
            self.updateDicos()
            self.setNewVolumicZone(label, zone, local)

        self.updateDicos()


    def cancelVolumicZone(self, label, zone):
        """
        Cancel balise volumic zone labelled label in XML file
        """
        self.node_volzone.xmlRemoveChild('zone', label=label, name=zone)
        self.updateDicos()


    def getLocalisation(self, label):
        """
        Get localisation from zone balise in XML file
        """
        node = self.node_volzone.xmlGetChildNode('zone', label=label)
        if node:
            if node['name'] == self.dicoLabel2Zone[label]:
                local = node.xmlGetTextNode()
        return str(local)
       

       
#-------------------------------------------------------------------------------
# Hlist multi-selection class
#-------------------------------------------------------------------------------


class MultiSelect:
    """
    This class is design in order to manage the storage of the selected items in
    the Hlist.
    """
    def __init__(self, Hlist):
        """
        Constructor
        """
        self.Hlist = Hlist


    def _initList(self):
        """
        For each mouse selection we do reinitialize all lists.
        """
        self.labelL = []
        self.zoneL  = []
        self.localL   = []


    def zoneInfo(self, entry):
        """
        Return info from the argument entry.
        """
        # Here we must call the tk function directly without passing in
        # Tix.py, because of a bug in tix8.1 in the Hlist.item_cget function
        #
        h = self.Hlist
        label = h.tk.call(h, 'item', 'cget', entry, 0, '-text')
        zone  = h.tk.call(h, 'item', 'cget', entry, 1, '-text')
        local = h.tk.call(h, 'item', 'cget', entry, 2, '-text')
        
        if local == "--": local =""

        return label, zone, local


    def setItems(self, entriesL):
        """
        Store values when a new selection is done.
        """
        self._initList()

        for entry in entriesL:
            label, zone, local = self.zoneInfo(entry)
            self.labelL.append(label)
            self.zoneL.append(zone)
            self.localL.append(local)


    def widgetText(self):
        """
        Find the right texts for the widget layout.
        """
        label, zone, local = self.labelL[0], self.zoneL[0], self.localL[0]

        for i in self.labelL:
            if label != i: label = _MULTISEL
        for i in self.zoneL:
            if zone != i: zone  = _MULTISEL
        for i in self.localL:
            if local != i: local  = _MULTISEL

        return label, zone, local

#-------------------------------------------------------------------------------
# Main View Class
#-------------------------------------------------------------------------------


class DefineVolumicZonesView(TkPage.Page):
    """
    Class to open Turbulence Page.
    """
    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.label = Tix.StringVar()
##        self.zone  = Tix.IntVar()
        self.zone  = Tix.StringVar()
        self.local = Tix.StringVar()


    def _pageControl(self):
        """
        Instantiate the definition boundary modelling class.
        """
        self.mdl = DefineVolumicZonesModel(self.case)

        # Definition of boundaries types
        #
        t = PageText()

##        # Number of zone
##        #
##        self.zoneNumber = 0

        #  Initialize variables concerning the display of the Hlist
        #
        self.selectedEntry = None
        self.currentEntry = None
        self.entriesNumber = 0


    def getLabel(self, event=None):
        """
        Return the name of the voluminal zone. It 's not allowed to have
        blank or not ASCII caracters in this name.
        """
        label = self.label.get()
        if len(label) > LABEL_LENGTH_MAX:
            label = label[:LABEL_LENGTH_MAX]
            if label in self.mdl.getListLabel():
                default = {}
                default['label'] = label
                default['list'] = self.mdl.getListLabel()
                dialog = Dialog.VerifyExistenceLabel(self.myPage, default=default)
                select = dialog.result.copy()
                label = select['label']
        return  string.join(string.split(label), '_')


    def testInt(self, entry, var):
        """
        Verify if the 'entry' contents an integer. If not, signal an error
        and return None.
        """
        try:
            i = int(var.get())
            if i <= 0: 
                self.check2.error(entry)
                i = -999
        except:
            self.check2.error(entry)
            i = -999

        return i


    def string2int(self, entry, var):
        """
        If the StringVar() 'var' associated to the 'entry', is not a '_MULTISEL',
        convert it to an integer.
        """
        self.check2.begin(entry)

        # Several selections in Hlist

        if self.currentEntry and len(self.currentEntry) > 1:
            if var.get() != _MULTISEL:
                i = self.testInt(entry, var)
            else:
                i = _MULTISEL
        # Only one selection in Hlist
        else:
            i = self.testInt(entry, var)
        return i


    def getZone(self, event=None):
        """
        Return an integer for number of voluminal zone.
        """
        if self.zone.get():
            zone = self.string2int(self.ezone, self.zone)
        else:
            zone = None
        return zone


    def getLocalisation(self, event=None):
        """
        Return a string for the localisation of the zone.
        """
##        return string.join(string.split(self.local.get()), '_')
        return self.local.get()


    def eraseEntries(self, event=None):
        """
        This button is for delete all caracters in the three entries.
        """
        self.elabel.delete(0, END)
        self.ezone.delete(0, END)
        self.elocal.delete(0, END)


    def insertHlist(self, label, zone, local):
        """
        Create the item Hlist associated with the new boundary definition.
        """
        self.stbar.busy()
        self.entriesNumber = self.entriesNumber +1

        name = 'item' + repr(self.entriesNumber)
        n = repr(self.entriesNumber)
        if not zone:
            zone = str(n)
            if zone in self.mdl.dicoLabel2Zone.values():
                for i in range(self.mdl.zoneNumber):
                    i = i+1
                    if repr(i) not in self.mdl.dicoLabel2Zone.values():
                        n = repr(i)
            zone = str(n)
            name = 'item' + n
            
        if not label: 
            label = 'zone' + "_" + n

        self.h.hlist.add(name, itemtype=Tix.TEXT, text=label,
                         style=self.style['style'])

        self.h.hlist.item_create(name, 1, itemtype=Tix.TEXT,
                                 text=zone,
                                 style=self.style['style'+'c'])

        self.h.hlist.item_create(name, 2, itemtype=Tix.TEXT,
                                 text=local,
                                 style=self.style['style'])
        self.stbar.idle()
        return label, zone


    def createHlist(self, event=None):
        """
        Upload the contents of the definition widget
        when the 'create' button is pushed.
        Checking consistency of values and interpreting
        complexe entry for reference if necessary.
        """
        t = PageText()
        self.stbar.busy()

        label = self.getLabel()
        if label == _MULTISEL:
            self.stbar.idle()
            return
        if label in self.mdl.dicoLabel2Zone.keys():
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LABEL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            return

        zone = self.getZone()
        if zone == -999: 
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.MSG_ZONE)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return
        if zone == _MULTISEL:
            self.stbar.idle()
            return

        if str(zone) in self.mdl.dicoLabel2Zone.values():
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_ZONE)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            return

        local = self.getLocalisation()
        if not local:
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.NO_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return
        if local in self.mdl.dicoLocal:
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return


        # Reading string and add to Hlist
        #
        try:
            label, zone = self.insertHlist(label, zone, local)
            self.mdl.setNewVolumicZone(label, zone, local)
            self.mdl.updateDicos()
        except:
            pass

        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def replaceHlist(self, entry, label, zone, local):
        """
        Add the item Hlist associated with the new boundary definition.
        """
        self.stbar.busy()

        # Update the Hlist
        #
        self.h.hlist.item_configure(entry, 0, text=label, style=self.style['style'])
        self.h.hlist.item_configure(entry, 1, text=zone, style=self.style['style'+'c'])
        self.h.hlist.item_configure(entry, 2, text=local, style=self.style['style'])

        self.stbar.idle()


    def setEditedValueHlist(self, entry, old_label, old_zone, old_local):
        """
        Checking consistency of values and interpreting
        complexe entry for reference if necessary.
        """
        t = PageText()

        new_label = self.getLabel()
        if new_label == _MULTISEL or not new_label:
            new_label = old_label

        new_zone = self.getZone()
        if new_zone == _MULTISEL or not new_zone:
            new_zone = old_zone
            if new_zone == -999: 
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.MSG_ZONE)
                return
        

        if len(self.currentEntry) == 1:
            new_local = self.getLocalisation()
        else:
            new_local = old_local
            
        if not new_local:
#TO DO : popup ??
            return
            
        if new_label != old_label:
            if new_label in self.mdl.getListLabel() and len(self.currentEntry) == 1:
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LABEL)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return

        if str(new_zone) != old_zone:
            if str(new_zone) in self.mdl.dicoLabel2Zone.values():
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_ZONE)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return

        if new_local != old_local:
            if new_local in self.mdl.dicoLocal:
                Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
                self.eraseEntries()
                self.currentEntry = None
                self.selectHlist()
                return

        self.mdl.replaceVolumicZone(old_label, old_zone, new_label, str(new_zone), new_local)
        
        self.replaceHlist(entry, new_label, str(new_zone), new_local)


    def editHlist(self, event=None):
        """
        Modify previous selected entries. A new node is created with
        the new specifications and the previous ChildNodes are cloned
        in order to taking into account the user's boundary conditions
        if they exist.
        """
        if not self.currentEntry: return
        self.stbar.busy()
        t = PageText()
        for entry in self.currentEntry:
            label, zone, local = self.select.zoneInfo(entry)
            self.setEditedValueHlist(entry, label, zone, local)

        if len(self.currentEntry) >1:
            label0, zone0, local0 = self.select.widgetText()
            entry0 = self.currentEntry[0]
            label = self.getLabel()
            if label != _MULTISEL :
                new_local = string.join(self.select.localL, ' or ')
                new_zone = min(self.select.zoneL)
                
                for entry in self.currentEntry:
                    self.mdl.cancelVolumicZone(label, self.mdl.dicoLabel2Zone[label])
                    if entry != entry0:
                        self.h.hlist.delete_entry(entry)
                self.mdl.setNewVolumicZone(label, new_zone, new_local)
                self.replaceHlist(entry0, label, new_zone, new_local)
                    

        self.mdl.updateDicos()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def cancelHlist(self, event=None):
        """
        Just delete the current selected entries from the Hlist and
        of course from the XML file.
        """
        if not self.currentEntry: return
        self.stbar.busy()
        for entry in self.currentEntry:
            if entry == 'item1': return
            label, zone,local = self.select.zoneInfo(entry)
            self.mdl.cancelVolumicZone(label, zone)

            self.entriesNumber = self.entriesNumber -1
##            self.zoneNumber = self.zoneNumber -1
            self.h.hlist.delete_entry(entry)

        self.mdl.updateDicos()
        self.eraseEntries()
        self.currentEntry = None
        self.selectHlist()
        self.stbar.idle()


    def importHlist(self, event=None):
        """
        Import from SALOME.
        """
        from CFDSTUDYGUI_DataModel import _getStudy,_getEngine
        from CFDSTUDYGUI_Commons import sg, sgPyQt
        from salome import lcc

        aStudy = _getStudy()
        aSMESH_SO = aStudy.FindComponent("SMESH")

        if aSMESH_SO == None:
            return

        #loading IORs
        builder = aStudy.NewBuilder()
        if aSMESH_SO != None:
            import smesh
            aSMESHEngine = lcc.FindOrLoadComponent( "FactoryServer", "SMESH" )
            builder.LoadWith( aSMESH_SO , aSMESHEngine )

        localization = ""
        if sg.SelectedCount() > 0:
            for i in range (sg.SelectedCount()):
                entry = sg.getSelected( i )
                if entry != '':
                    sobj = aStudy.FindObjectID( entry )
                    if sobj !=  None:
                        anObjectDS = sobj.GetObject()
                        #check for smesh group
                        if anObjectDS !=  None:
                            aSmeshObject = anObjectDS._narrow( smesh.SMESH_Group )
                            if aSmeshObject != None and aSmeshObject.GetType() == smesh.VOLUME:
			        if not localization:
                                    localization = aSmeshObject.GetName()
			        else:
                                    localization = localization + ' or ' + aSmeshObject.GetName()

        if localization in self.mdl.dicoLocal:
            t = PageText()
            Dialog.ShowInfo(self.master, title=t.INFO, text=t.BAD_LOCAL)
            self.eraseEntries()
            self.currentEntry = None
            self.selectHlist()
            self.stbar.idle()
            return

	if localization:
            label, zone = self.insertHlist("", "", localization)
            self.mdl.setNewVolumicZone(label, zone, localization)
            self.mdl.updateDicos()


    def selectHlist(self, entry=None):
        """
        Browsecmd:
        Every single Clik on the Button1 of the mouse send TWO signal to the GUI
        the first one when man push the button, and the second one when the
        button gets back.

        Command:
        Every double-Click and Return send only ONE signal to the GUI.
        """
        self.selectedEntry = self.h.hlist.info_selection()
        self.box1.config(state=NORMAL)
        if self.currentEntry == 'item1':
            self.box1.erase.config(state=DISABLED)
            return

        if not self.selectedEntry:
            self.llocal.config(state=NORMAL)
            self.elocal.config(state=NORMAL)
            self.elocal.delete(0,END)
            
            self.lzone.config(state=NORMAL)
            self.ezone.config(state=NORMAL)
            self.ezone.delete(0,END)

        elif self.currentEntry != self.selectedEntry:

            self.currentEntry = self.selectedEntry
            self.select.setItems(self.currentEntry)
##            self.select.backgroundColor()
            label, zone, local = self.select.widgetText()

            # Update all widgets
            #

            self.elabel.delete(0,END)
            self.elabel.insert(0,label)
            self.elabel.icursor(END)
            self.elabel.config(state=NORMAL)

            self.ezone.delete(0,END)
            self.ezone.insert(0,zone)
            self.ezone.icursor(END)
            
            self.elocal.delete(0,END)
            self.elocal.insert(0,local)
            self.elocal.icursor(END)

            if len(self.currentEntry) > 1:
                self.lzone.config(state=DISABLED)
                self.ezone.config(state=DISABLED)
                
                self.llocal.config(state=DISABLED)
                self.elocal.config(state=DISABLED)

            else:
                self.elabel.config(state=NORMAL)
                self.elabel.delete(0,END)
              
                self.llocal.config(state=NORMAL)
                self.elocal.config(state=NORMAL)
                self.elocal.delete(0,END)
              
                self.lzone.config(state=NORMAL)
                self.ezone.config(state=NORMAL)
                self.ezone.delete(0,END)
                
                self.elabel.delete(0,END)
                self.elabel.insert(0,label)
                self.elabel.icursor(END)
                
                self.ezone.delete(0,END)
                self.ezone.insert(0,zone)
                self.ezone.icursor(END)

                self.elocal.delete(0,END)
                self.elocal.insert(0,local)
                self.elocal.icursor(END)


    def _make_style(self):
        """
        Define beautyful style in order to display columns in the Hlist.
        """
        self.style={}

        self.style['header'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.w1,
                                                anchor=CENTER,
                                                padx=5, pady=2,
                                                font=fB)

        self.style['style'+'c'] = Tix.DisplayStyle(Tix.TEXT,
                                                   refwindow=self.w1,
                                                   anchor=CENTER,
                                                   padx=5,
                                                   selectforeground='#fefefe',
                                                  font=fN)
        self.style['style'] = Tix.DisplayStyle(Tix.TEXT,
                                               refwindow=self.w1,
                                               padx=5,
                                               selectforeground='#fefefe',
                                               font=fN)


    def selectHlistAll(self):
        """
        Fonction for the Popup Menu.
        Select everything in the Hlist.
        """
        try:
            first = 'item' + repr(1)
            last  = 'item' + repr(self.entriesNumber)
            self.h.hlist.selection_set(first, last)
            self.currentEntry = None
            self.selectHlist()
        except:
            pass


    def cancelSelection(self):
        """
        Fonction for the Popup Menu.
        Blank the selection.
        """
        self.h.hlist.selection_clear()
        self.lzone.config(state=NORMAL)
        self.ezone.config(state=NORMAL)
        
        self.llocal.config(state=NORMAL)
        self.elocal.config(state=NORMAL)

        self.currentEntry = None
        self.eraseEntries()


    def searchListEnv(self, event=None):
        """
        Research a Preprocessor listing to pick colors or groups of cells.
        """
        t = PageText()
        self.stbar.busy()

        file_name = preprocessorFile(self.master, self.case['resu_path'])

        if len(self.mdl.dicoLocal) > 1: 
            Dialog.ShowInfo(self.master,
                            title = t.INFO,
                            text  = t.INFO_LISTENV)
        else:
            if file_name:
                self.info = Informations(file_name, 'cells')
                refList, groupList = self.info.updateListes()
    
                for ref in refList:
                    lab = ''
                    zone = ''
                    lab, zone = self.insertHlist(lab, zone, ref)
                    self.mdl.setNewVolumicZone(lab, zone, ref)
                    self.mdl.updateDicos()
                for grp in groupList:
                    lab = ''
                    zone = ''
                    lab, zone = self.insertHlist(lab, zone, grp)
                    self.mdl.setNewVolumicZone(lab, zone, grp)
                    self.mdl.updateDicos()

        self.stbar.idle()


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()

        lf1 = Tix.LabelFrame(self.myPage, bd=2, label=t.TITLE, relief=FLAT)
        lf1.label.config(font=fT)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        self.w1 = Tix.Frame(lf1.frame, relief=FLAT)
        self.w1.pack(side=TOP, fill=X)


        # Put a simple hierachy into the Hlist (two levels).
        #
        self.h = Tix.ScrolledHList(self.w1, options='hlist.columns 3 hlist.header 1 ' )
        self.h.config(scrollbar="auto +y +x")

        # selectmode='extended' -> multi select with <Shift> and <Ctrl>
        # selectmode='multiple' -> multi select only with <Shift>
        #
        self.h.hlist.config(selectmode='extended',
                            browsecmd=self.selectHlist,
                            bg=wm['hlistbackground'])
        self.h.hsb.config(width=10, bd=1)
        self.h.vsb.config(width=10, bd=1)
        self.h.pack(expand=1, fill=BOTH, padx=10, pady=10, side=TOP)

        self.balloon.bind_widget(self.h.hlist, statusmsg=t.MSG_HLIST)

        # First some styles for the headers
        #
        self._make_style()

        # Create the headers
        #
        self.h.hlist.header_create(0, itemtype=Tix.TEXT, text=t.LABEL,
                                   style=self.style['header'])
        self.h.hlist.header_create(1, itemtype=Tix.TEXT, text=t.ZONE,
                                   style=self.style['header'])
        self.h.hlist.header_create(2, itemtype=Tix.TEXT, text=t.LOCAL,
                                   style=self.style['header'])
                                   
        # Notice that we use 5 columns in the Hlist widget. This way when the user
        # expands the windows wide, the right side of the header doesn't look
        # chopped off. The following line ensures that the 5 column header is
        # not shown unless the Hlist window is wider than its contents.
        #
        self.h.hlist.column_width(2,0)

        # Let configure the appearance of the Hlist subwidget 
        #
        self.h.hlist.config(separator='.', width=25, height=11, drawbranch=0, indent=10)

        # Let configure the width of the columns
        #
        self.h.hlist.column_width(0, chars=16)
        self.h.hlist.column_width(1, chars=6)
        self.h.hlist.column_width(2, chars=30)

        # Create the mouse selection instance
        #
        self.select = MultiSelect(self.h.hlist)

        # Popup Menu of Tree
        #
        pm = Tix.PopupMenu(self.h.hlist, title=t.POPUP, state=NORMAL)
        pm.bind_widget(self.h.hlist)
        pm.menu.add_separator()
        pm.menu.add_command(label=t.SELECT_ALL, command=self.selectHlistAll)
        pm.menu.add_command(label=t.UNSELECT, command=self.cancelSelection)
        pm.menu.m1 = Tix.Menu(pm.menu)

        # Separator
        #
        s = Tix.Frame(lf1.frame, height=2, bd=2, relief=SUNKEN)
        s.pack(side=TOP, fill=X)

        f1 = Tix.Frame(lf1.frame, relief=FLAT)
        f1.pack(side=TOP, fill=X)

        Tix.Label(f1, text=t.LABEL).grid(row=0, column=0, padx=3, pady=5, sticky=E)
        self.elabel = Tix.Entry(f1, bd=2, width=15, textvariable=self.label)
        self.elabel.grid(row=0, column=1, padx=5, pady=5, sticky=W)
        self.elabel.bind('<Return>', self.createHlist)

        self.lzone= Tix.Label(f1, text=t.ZONE)
        self.lzone.grid(row=1, column=0, padx=3, pady=5, sticky=E)
        self.ezone = Tix.Entry(f1, bd=2, width=10, textvariable=self.zone)
        self.ezone.grid(row=1, column=1, padx=5, pady=5, sticky=W)
        self.ezone.bind('<Return>', self.createHlist)
        
        self.llocal = Tix.Label(f1, text=t.LOCAL)
        self.llocal.grid(row=2, column=0, padx=3, pady=5, sticky=E)
        self.elocal = Tix.Entry(f1, bd=2, width=45, textvariable=self.local)
        self.elocal.grid(row=2, column=1, padx=5, pady=5, sticky=W)
        self.elocal.bind('<Return>', self.createHlist)
    
        self.box1 = Tix.ButtonBox(f1, orientation=HORIZONTAL, relief=FLAT)
        self.box1.add('create', text=t.CREATE, command=self.createHlist)
        self.box1.add('modify', text=t.MODIFY, command=self.editHlist)
        self.box1.add('erase',  text=t.ERASE,  command=self.cancelHlist)
        if self.case['salome']:
            self.box1.add('import',  text=t.SALOME,  command=self.importHlist)
        self.box1.grid(row=3, column=1, columnspan=4, padx=15, pady=15, sticky=W)

        # Separator
        #
        s = Tix.Frame(lf1.frame, height=2, bd=2, relief=SUNKEN)
        s.pack(side=TOP, fill=X)
        
        f2 = Tix.Frame(lf1.frame, relief=FLAT)
        f2.pack(side=TOP, fill=X)

        Tix.Label(f2, relief=FLAT, text=t.FIC_ENV).grid(row=0,column=0,padx=15,pady=15)

        self.img = Tix.PhotoImage(file=PIC_PATH+'search1.gif')
        env = Tix.Button(f2, image=self.img, command=self.searchListEnv)
        env.grid(row=0,column=1,padx=15,pady=15)


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """

        if self.mdl.getListLabel():
            for label in self.mdl.getListLabel():
                zone = self.mdl.getZone(label)
                local = self.mdl.getLocalisation(label)
                self.insertHlist(label, zone, local)
        else:
##            self.mdl.setNewVolumicZone('all_cells','1','all')
            self.insertHlist('all_cells','1','all[]')

        self.mdl.updateDicos()


#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.AUTO_TITLE    = "AUTOMATIQUE"
            self.BAD_LABEL     = "Attention : cette tiquette ne peut tre utilise\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre tiquette"
            self.BAD_LOCAL     = "Attention : cette localisation ne peut tre utilise\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre localisation"
            self.BAD_ZONE      = "Attention : ce numro de zone ne peut tre utilis\n\n"\
                                 "qu'une seule et unique fois.\n\n"\
                                 "Donnez une autre valeur"
            self.CREATE        = "Crer"
            self.ERASE         = "Effacer"
            self.FACES_BORD    = "Attention : les rfrences de faces de bord suivantes" \
                                 " ont dj t dclares :\n\n"
            self.FIC_ENV       = "Importer les groupes et les couleurs\n"\
                                 "d'un listing du Prprocesseur"
            self.GROUP         = "Groupe"
            self.INFO          = "Informations"
            self.INFO_LISTENV  = "Attention : des localisations de faces de bord "\
                                 " ont dj t dclares.\n\n "\
                                 "Il faut les supprimer pour importer des donnes"\
                                 " du listing."
            self.INFO_DELETE   = "Attention : la modification de la dfinition d'une"\
                                " condition limite entrane la perte des informations"\
                                " associes  la prcdente dfinition."
            self.INLET         = "entre"
            self.LABEL         = "tiquette"
            self.MANUAL_TITLE  = "MANUEL"
            self.MODIFY        = "Modifier"
            self.NO_LOCAL      = "Attention : localisation obligatoire pour cration\n\n"\
                                 "Donnez une localisation"
            self.OUTLET        = "sortie"
            self.POPUP         = "Menu popup"
            self.REF           = "Rfrence"
            self.SELECT_ALL    = "Tout slectionner"
            self.SORT_OUT      = "Trier par"
            self.SELECT_TYPE   = "Slection par type"
            self.SYMMETRY      = "symtrie"
            self.TITLE         = "Dfinition des zones volumiques"
            self.LOCAL         = "Localisation"
            self.ZONE          = "Zone"
##            self.UNDEFINED     = "indfini"
            self.UNSELECT      = "Dslectionner"
            self.VISU_TITLE    = "VISUALISATION"
            self.WALL          = "paroi"
            self.SALOME        = "Importer de SALOME"
        else:
            self.AUTO_TITLE    = "AUTOMATIC"
            self.BAD_LABEL     = "Warning : this label can be used \n\n"\
                                  "only one and single time. \n\n"\
                                  "Give another label"
            self.BAD_LOCAL     = "Warning : this localization can be used \n\n"\
                                  "only one and single time. \n\n"\
                                  "Give another localization"
            self.BAD_ZONE      = "Warning : this number of zone can be used \n\n"\
                                 "only one and single time.\n\n"\
                                 "Give another value"
            self.CREATE        = "Create"
            self.ERASE         = "Delete"
            self.FACES_BORD    = "Warning : the following faces de bord referencies " \
                                 "have been already created :\n\n"
            self.FIC_ENV       = "Import groups and references from Preprocessor listing"
            self.GROUP         = "Group"
            self.INFO          = "Informations"
            self.INFO_LISTENV  = "Warning: some localizations already have given.\n\n"\
                                 "You must destroy them to import data from listing"
            self.INFO_DELETE   = "Warning : the modification of a region boundary"\
                                " definition occurs the lost of data about the previous"\
                                " definition."
            self.INLET         = "inlet"
            self.LABEL         = "Label"
            self.MANUAL_TITLE  = "MANUAL"
            self.MODIFY        = "Modify"
            self.NO_LOCAL      = "Warning : You must obligatorily give a localization "

            self.OUTLET        = "outlet"
            self.POPUP         = "Menu popup"
            self.REF           = "Reference"
            self.SELECT_ALL    = "Select all"
            self.SELECT_TYPE   = "Selection by type"
            self.SORT_OUT      = "Sort out by"
            self.SYMMETRY      = "symmetry"
            self.TITLE         = "Definition of volumic regions"
            self.LOCAL         = "Localisation"
            self.ZONE          = "Zone"
##            self.UNDEFINED     = "undefined"
            self.UNSELECT      = "Unselect"
            self.VISU_TITLE    = "VISUALIZATION"
            self.WALL          = "wall"
            self.SALOME        = "Import from SALOME"

        # 2) Messages
        #
        if Tool.GuiParam.lang == 'fr':
            self.MSG_HLIST   = "Cliquer le bouton droit pour "       +\
                               "le menu contextuel."
            self.MSG_ZONE    = "zone doit tre une valeur entire positive"
        else:
            self.MSG_HLIST   = "Click right button for popup menu."
            self.MSG_ZONE    = "zone must be an positive integer value"


#-------------------------------------------------------------------------------
# Turbulence test case
#-------------------------------------------------------------------------------


class DefBCTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """
        This method is executed before all "check" methods.
        """
        from Base.XMLstuff import TkGuiModel
        self.case = TkGuiModel(None)


    def tearDown(self):
        """
        This method is executed after all "check" methods.
        """
        del self.case


    def checkDefBCModelInstantiation(self):
        """Check whether the DefBCModel class could be instantiated"""
        model = None
        model = DefBCModel(self.case)
        assert model != None, 'Could not instantiate DefBCModel'


    def checksetBoundaryDefandCond(self):
        """
        """
        model = DefBCModel(self.case)
        model.xmlSetBoundaryDefandCond('inlet', 'porte', '4', None)
        truc = model.node_bc.toString()
        
        doc = '<boundary_conditions>'\
                  '<boundary_definition>'\
                      '<inlet label="porte">'\
                          '<reference>4</reference>'\
                      '</inlet>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                       '<scalar choice="" label="temperature_celsius" type="thermal"/>'\
                       '<scalar choice="" label="scal1" type="user"/>'\
                  '</inlet>'\
              '</boundary_conditions>'

        assert doc == truc, 'Could not set the boundary reference number'

        model.xmlSetBoundaryDefandCond('inlet', 'porte', None, 'titi')
        truc = model.node_bc.toString()

        doc = '<boundary_conditions>'\
                  '<boundary_definition>'\
                      '<inlet label="porte">'\
                          '<reference>4</reference>'\
                          '<group>titi</group>'\
                      '</inlet>'\
                  '</boundary_definition>'\
                  '<inlet label="porte">'\
                       '<velocity_pressure choice=""/>'\
                       '<turbulence choice=""/>'\
                       '<scalar choice="" label="temperature_celsius" type="thermal"/>'\
                       '<scalar choice="" label="scal1" type="user"/>'\
                  '</inlet>'\
              '</boundary_conditions>'

        assert doc == truc, 'Could not set the boundary group string'



def suite():
    testSuite = unittest.makeSuite(DefBCTestCase, "check")
    return testSuite

    
def runTest():
    runner = unittest.TextTestRunner()
    runner.run(suite())


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
