# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the storage system type.

This module contains the following classes and function:
- MatisseTypeModel
- MatisseTypeView
- PageText
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import Tix
from Tkconstants import *
import sys, unittest


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.TkPage as TkPage


#-------------------------------------------------------------------------------
# Turbulence model class
#-------------------------------------------------------------------------------


class MatisseTypeModel:
    def __init__(self, case):
        self.case        = case
        self.node_matisse= self.case.root().xmlInitChildNode('matisse')
        self.node_compute= self.node_matisse.xmlInitChildNode('compute')
        self.node_geom   = self.node_compute.xmlInitChildNode('geometry')
        self.node_phys   = self.node_compute.xmlInitChildNode('physical_model')
        
        self.node_type   = self.node_geom.xmlInitChildNode('typent', 'label')
        self.node_alveo  = self.node_phys.xmlInitChildNode('ialveo', 'status')

        self.matisseTypes = ('vault', 'emm','djw')


    def defaultMatisseTypeValues(self):
        """
        Return in a dictionnary which contains default values
        """
        default = {}
        default['typent'] = 'vault'
        return default


    def setMatisseType(self, matisse_type):
        """
        Input storage matisse type
        """
        if matisse_type not in self.matisseTypes :
	    print "MatissTypeModel: error with the storage concept list"
            sys.exit(1)

        if matisse_type == 'djw' :
            self.node_type['label'] = 'vault'
            self.node_alveo['status'] = 'on'
        else :
            self.node_alveo['status'] = 'off'
            self.node_type['label'] = matisse_type


    def getMatisseType(self):
        """
        Return the storage matisse type
        """
        if self.node_alveo['status'] == 'on' :
            if self.node_type['label'] != 'vault':
                print "MatissTypeModel: error with the storage concept input"
                sys.exit(1)
            matisse_type = 'djw'
        else :
            matisse_type = self.node_type['label']
            
        if matisse_type not in self.matisseTypes :
            matisse_type = self.defaultMatisseTypeValues()['typent']
            self.setMatisseType(matisse_type)
        return matisse_type



#-------------------------------------------------------------------------------
# Main view class
#-------------------------------------------------------------------------------


class MatisseTypeView(TkPage.Page):
    """
    Class to open Matisse Page.
    """
    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.type = Tix.StringVar()


    def _pageControl(self):
        """
        Instantiate the matisse type modelling class.
        """
        self.model = MatisseTypeModel(self.case)


    def getMatisseType(self, event=None):
        """
        Input Storage matisse type.
        """
        self.stbar.busy()

        t = self.type.get()
        self.model.setMatisseType(t)

        self.stbar.idle()

 
    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()

        lf1 = Tix.LabelFrame(self.myPage, bd=2,
                            label=t.MATISSE_TYPE_TITLE, relief=FLAT)
        lf1.label.config(font=fT)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        # Select the storage type
        #
        self.b1 = Tix.OptionMenu(lf1.frame, options='menubutton.width 45')
        self.b1.menubutton.config(bd=2, relief=RAISED)

        self.b1.grid(row=0, column=0, padx=10, pady=20)

        self.b1.add_command('vault',  label=t.VAULT)
        self.b1.add_command('emm',    label=t.EMM)
        self.b1.add_command('djw',    label=t.DJW)
        self.b1.config(variable=self.type,
                       command=self.getMatisseType)


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        t = self.model.getMatisseType()
        self.b1.config(value=t)


#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        if Tool.GuiParam.lang == 'fr':
            self.MATISSE_TYPE_TITLE = "Type d'entrept"
            self.VAULT              = "Vote"
            self.EMM                = "Cathdrale"
            self.DJW                = "Vote avec puits chemiss"
        else:
            self.MATISSE_TYPE_TITLE = "Storage concept"
            self.VAULT              = "Vault"
            self.EMM                = "Cask storage"
            self.DJW                = "Double jacketed wells"


#-------------------------------------------------------------------------------
# MatisseTypeTestCase test case
#-------------------------------------------------------------------------------


class MatisseTypeTestCase(unittest.TestCase):
    def setUp(self):
        """
        This method is executed before all "check" methods.
        """
        from Base.XMLengine import Case
        self.case = Case()


    def tearDown(self):
        """
        This method is executed after all "check" methods.
        """
        del self.case


    def checkMatisseTypeInstantiation(self):
        """
        Check whether the TurbulenceModel class could be instantiated
        """
        model = None
        model = MatisseTypeModel(self.case)
        assert model != None, 'Could not instantiate MatisseTypeModel'


def suite():
    testSuite = unittest.makeSuite(TurbulenceTestCase, "check")
    return testSuite


def runTest():
    print "MatisseTypeTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())


#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
