# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the values of reference.

This module contains the following classes and function:
- MobilMeshModel
- MobilMeshView
- PageText
- MobilMeshTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import Tix
from Tkconstants import *
import sys, unittest


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Variables
from Pages.OutputControlModel import OutputControlModel


#-------------------------------------------------------------------------------
# Reference values model class
#-------------------------------------------------------------------------------

class MobilMeshModel:
    """
    Manage the input/output markups in the xml doc about mobil mesh
    """
    def __init__(self, case):
        """
        Constructor.
        """
        self.case = case

        self.node_models = self.case.xmlGetNode('thermophysical_models')
        self.node_ale    = self.node_models.xmlInitChildNode('ale_method', 'status')
        
        self.out = OutputControlModel(self.case)


    def defaultInitialValues(self):
        """
        Return in a dictionnary which contains default values.
        """
        default = {}
        default['nalinf']  = 0
        default['nalimx']  = 0
        default['epalim']  = 1e-05
        default['iortvm']  = 'isotrop'

        return default

    def setVariablesandProperties(self):
        """
        Set variables and properties if ALE method is activated.
        """
        self.node_ale.xmlInitChildNode('variable', name='mesh_velocity_U', label='mesh_u')
        self.node_ale.xmlInitChildNode('variable', name='mesh_velocity_V', label='mesh_v')
        self.node_ale.xmlInitChildNode('variable', name='mesh_velocity_W', label='mesh_w')
        self.node_ale.xmlInitChildNode('property', name='mesh_viscosity_1', label='mesh_vi1')
        self.updateNodeViscosity()
        
        
    def updateNodeViscosity(self):
        """
        Update properties beyond mesh visosity is isotrope or not.
        """
        if self.getViscosity() == 'orthotrop':
            self.node_ale.xmlInitChildNode('property', name='mesh_viscosity_2', label='mesh_vi2')
            self.node_ale.xmlInitChildNode('property', name='mesh_viscosity_3', label='mesh_vi3')
        else:
            node1 = self.node_ale.xmlGetChildNode('property', name='mesh_viscosity_2')
            node2 = self.node_ale.xmlGetChildNode('property', name='mesh_viscosity_3')
            if node1:
                node1.xmlRemoveNode()
            if node2:
                node2.xmlRemoveNode()


##    def removeVariablesandProperties(self):
##        """
##        Set variables and properties if ALE method is activated.
##        """
##        self.node_ale.xmlRemoveChild('variable')
##        self.node_ale.xmlRemoveChild('property')


    def setMethod(self, answer):
        """
        Set method of activation of ALE into xml file.
        """
        typ = ''
        typ = self.out.getTypePostMeshes()
        self.node_ale['status'] = answer
        if answer == 'on':
            if typ not in ('10', '11', '12'):
                typ = '10'
            self.setVariablesandProperties()
        else:
            if typ not in ('0', '1', '2'):
                typ = '0'
##            self.removeVariablesandProperties()
        self.out.setTypePostMeshes(typ)


    def getMethod(self):
        """
        Get method of activation of ALE from xml file.
        """
        status = 'off'
        if self.node_ale['status'] == '':
            status = 'off'
            self.setMethod(status)
##            self.node_ale['status'] = 'off'
        else:
            status = self.node_ale['status']
        return status


    def setSubIterations(self, value):
        """
        Set value of fluid initialization sub iterations into xml file.
        """ 
        self.node_ale.xmlSetData('fluid_initialization_sub_iterations', value)


    def getSubIterations(self):
        """
        Get value of fluid initialization sub iterations from xml file.
        """ 
        nalinf = self.node_ale.xmlGetInt('fluid_initialization_sub_iterations')
        if not nalinf:
            nalinf = self.defaultInitialValues()['nalinf']
            self.setSubIterations(nalinf)
        return nalinf


    def setMaxIterations(self, value):
        """
        Set value of maximum of iteration if implicitation into xml file.
        """ 
        self.node_ale.xmlSetData('max_iterations_implicitation', value)


    def getMaxIterations(self):
        """
        Get value of maximum of iteration if implicitation from xml file.
        """ 
        nalimx = self.node_ale.xmlGetInt('max_iterations_implicitation')
        if not nalimx:
            nalimx = self.defaultInitialValues()['nalimx']
            self.setMaxIterations(nalimx )
        return nalimx


    def setPrecision(self, value):
        """
        Set value of precision of implicitation into xml file.
        """ 
        self.node_ale.xmlSetData('implicitation_precision', value)


    def getPrecision(self):
        """
        Get value of precision of implicitation from xml file.
        """ 
        epalim = self.node_ale.xmlGetDouble('implicitation_precision')
        if not epalim:
            epalim = self.defaultInitialValues()['epalim']
            self.setPrecision(epalim)
        return epalim


    def setViscosity(self, value):
        """
        Set value of mesh viscosity into xml file.
        """ 
        node = self.node_ale.xmlInitChildNode('mesh_viscosity')
        node['type'] = value
        self.updateNodeViscosity()


    def getViscosity(self):
        """
        Get value of mesh viscosity from xml file.
        """
        iortvm = self.defaultInitialValues()['iortvm']
        node = self.node_ale.xmlGetChildNode('mesh_viscosity')
        if node :
            iortvm = node['type']
        else:
            self.setViscosity(iortvm)

        return iortvm


#-------------------------------------------------------------------------------
# Main View class
#-------------------------------------------------------------------------------


class MobilMeshView(TkPage.Page):
    """
    Class to open Page.
    """
 
    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['models/initia',
                                      'bounda/thermo']


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.ale            = Tix.StringVar()
        self.viscosity_type = Tix.StringVar()
        self.nalinf         = Tix.StringVar()
        self.nalimx         = Tix.StringVar()
        self.epalim         = Tix.DoubleVar()


    def _pageControl(self):
        """
        Instantiate the turbulence modelling class.
        """
        self.mdl = MobilMeshModel(self.case)


    def getMethod(self, event=None):
        """
        Input activation or not of ALE method.
        """
        self.stbar.busy()
        
        self.s1.pack_forget()
        self.w2.pack_forget()
        
        if self.ale.get() == 'on':
            self.s1.pack(side=TOP, fill=X)
            self.w2.pack(side=TOP, fill = X, padx=10, pady=10)
            self.nalinf.set(self.mdl.getSubIterations())
            #si suite nalinf = 0 : a ajouter
            self.nalimx.set(self.mdl.getMaxIterations())
            self.epalim.set(self.mdl.getPrecision())
            self.visc.config(value=self.mdl.getViscosity())
        else:
            self.s1.pack_forget()
            self.w2.pack_forget()
        self.mdl.setMethod(self.ale.get())
        
        self.stbar.idle()


    def getNalinf(self, event=None):
        """
        Input viscosity type of mesh : isotrop or orthotrop.
        """
        nalinf = self.nalinf.get()
        self.mdl.setSubIterations(nalinf)
        return nalinf


    def getNalimx(self, event=None):
        """
        Input viscosity type of mesh : isotrop or orthotrop.
        """
        nalimx = self.nalimx.get()
        self.mdl.setMaxIterations(nalimx)
        return nalimx


    def getEpalim(self, event=None):
        """
        Input viscosity type of mesh : isotrop or orthotrop.
        """
        epalim = self.epalim.get()
        self.mdl.setPrecision(epalim)
        return epalim


    def getViscosityType(self, event=None):
        """
        Input viscosity type of mesh : isotrop or orthotrop.
        """
        visco = self.viscosity_type.get()
        self.mdl.setViscosity(visco)
        return visco


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()
        lf1 = Tix.LabelFrame(self.myPage, bd=2,
                             label=t.ALE_TITLE, relief=FLAT)
        lf1.label.config(font=fT)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        w1 = Tix.Frame(lf1.frame,relief=FLAT)
        w1.pack(side=TOP)

        l = Tix.Label(w1, text=t.ALE_METHOD)
        self.balloon.bind_widget(l, balloonmsg=t.KEYWORD+"IALE")
        l.grid(row=0, column=0, padx=5, pady=10, sticky=W)
        
        self.r = [0]*2
        for (i, tx1, tx2) in [(0, t.YES, "on"),
                              (1, t.NO,  "off")]:
            self.r[i] = Tix.Radiobutton(w1, text=tx1, value=tx2,
                                        variable=self.ale,
                                        command=self.getMethod)
            self.r[i].grid(row=0, column=i+1)
            self.balloon.bind_widget(self.r[i], balloonmsg=t.KEYWORD+"IALE")
            
        # Separator
        #            
        self.s1 = Tix.Frame(lf1.frame, height=2, bd=2, relief=SUNKEN)
        self.s1.pack(side=TOP, fill=X)
        
        # Parameters of ALE
        #   
        self.w2 = Tix.Frame(lf1.frame,relief=FLAT)
        self.w2.pack(side=TOP, fill = X, padx=10, pady=10)
        
        self.lab = [0]*3
        self.ent = [0]*3
        for (i, txt, word, var, cmd) in [(0, t.NALINF, 'NALINF', self.nalinf, self.getNalinf),
                                         (1, t.NALIMX, 'NALIMX', self.nalimx, self.getNalimx),
                                         (2, t.EPALIM, 'EPALIM', self.epalim, self.getEpalim)]:
            self.lab[i] = Tix.Label(self.w2, text=txt)
            self.lab[i].grid(row=i, column=0, padx=10, pady=10, sticky=W)
            self.balloon.bind_widget(self.lab[i],balloonmsg=t.KEYWORD+word)
            
            self.ent[i] = Tix.Entry(self.w2, bd=2, width=10, textvariable=var)
            self.ent[i].grid(row=i, column=1, padx=5, pady=10, sticky=E)
            self.ent[i].event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.ent[i].bind("<<Event>>",cmd)
            self.balloon.bind_widget(self.ent[i],balloonmsg=t.KEYWORD+word)

        lvisc = Tix.Label(self.w2, text=t.VISCOSITY_TYPE)
        self.balloon.bind_widget(lvisc, balloonmsg=t.KEYWORD+"IORTVM")
        lvisc.grid(row=3, column=0, padx=10, pady=10, sticky=W)

        self.visc = Tix.OptionMenu(self.w2, options='menubutton.width 10')
        self.visc.menubutton.config(bd=2, relief=RAISED)
        self.balloon.bind_widget(self.visc, balloonmsg=t.KEYWORD+"IORTVM")
        self.visc.grid(row=3, column=1, padx=5, pady=10, sticky=E)
        self.visc.add_command('isotrop', label=t.ISOTROP)
        self.visc.add_command('orthotrop', label=t.ORTHOTROP)
        self.visc.config(variable=self.viscosity_type, command=self.getViscosityType)

    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        if self.mdl.getMethod() == 'on':
            self.r[0].select()
        else:
            self.r[1].select()
        self.getMethod()


#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.ALE_TITLE       = "Gestion du maillage mobile"
            self.ALE_METHOD      = "Activation de la mthode ALE"
            self.YES             = "oui"
            self.NO              = "non"
            self.NALINF          = "Nombre d'itrations d'initialisation\n"\
                                   "du fluide"
            self.NALIMX          = "Nombre maximal de sous-itrations pour le\n"\
                                   "couplage interne implicite des structures"
            self.EPALIM          = "Prcision relative pour le couplage\n"\
                                   "interne implicite des structures"
            self.VISCOSITY_TYPE  = "Type de viscosit de maillage"
            self.ISOTROP         = "isotrope"
            self.ORTHOTROP       = "orthotrope"
            self.KEYWORD         = "Mot cl Code_Saturne : "
        else:

            self.ALE_TITLE       = "Management of mobil mesh"
            self.ALE_METHOD      = "Activation of ALE method"
            self.YES             = "yes"
            self.NO              = "no"
            self.NALINF          = "Number of iterations for fluid initialization"
            self.NALIMX          = "Maximum number of sub-iterations for implicit\n"\
                                   "coupling with internal structures"
            self.EPALIM          = "Relative precision for implicit coupling\n"\
                                   "with internal structures"
            self.VISCOSITY_TYPE  = "Type of mesh viscosity"
            self.ISOTROP         = "isotropic"
            self.ORTHOTROP       = "orthotropic"
            self.KEYWORD         = "Code_Saturne key word: "


#-------------------------------------------------------------------------------
# ReferenceValuesModel test case
#-------------------------------------------------------------------------------


class MobilMeshTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        self.doc = XMLDocument()

    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case
        del self.doc

    def xmlNodeFromString(self, string):
        """Private method to return a xml node from string"""
        return self.doc.parseString(string).root()

    def checkTurbulenceInstantiation(self):
        """Check whether the TurbulenceModel class could be instantiated"""
        model = None
        model = MobilMeshModel(self.case)
        assert model != None, 'Could not instantiate TurbulenceModel'


def suite():
    testSuite = unittest.makeSuite(MobilMeshTestCase, "check")
    return testSuite

def runTest():
    print "MobilMeshTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
