# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------

"""
This module manages the differents possible outputs :
- listing printing
- post-processing and relationship with the FVM library
- monitoring points

This module defines the following classes:
- OutputControlModel
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import string, sys
from types import FloatType
from xml.dom.minidom import parse
import Tix
from Tkconstants import *

#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------

import Base.XMLmodel as XMLmodel


#-------------------------------------------------------------------------------
# Model class
#-------------------------------------------------------------------------------


class OutputControlModel:
    """
    Class for Variables and Scalar model initialization.
    """
    def __init__(self, case):
        """
        Constructor
        """
        self.case = case
        node_control  = self.case.xmlGetNode('analysis_control')
        self.node_out = node_control.xmlInitNode('output')
        self.dicoName = {}


    def defaultInitialValues(self):
        """
        Return in a dictionnary which contains default values.
        """
        default = {}
        default['listing_printing_frequency'] = 1
        default['postprocessing_frequency'] = -1
        default['probe_recording_frequency'] = 1
        default['postprocessing_options'] = "binary"
        default['postprocessing_format'] = "EnSight"
        default['fluid_domain'] = "on"
        default['domain_boundary'] = "off"
        default['syrthes_boundary'] = "on"
        default['postprocessing_mesh'] = 0

        return default


    def getAllVarScalProper(self):
        """
        We put in a list all variables scalars and properties nodes
        """
        varList = []
        node_models = self.case.xmlGetNode('thermophysical_models')
        node1 = node_models.xmlGetNode('gas_combustion',  'model')
        node2 = node_models.xmlGetNode('pulverized_coal', 'model')
        node3 = node_models.xmlGetNode('joule_effect',    'model')
        node4 = node_models.xmlGetNode('thermal_scalar',  'model')
        node5 = self.case.xmlGetNode('additional_scalars')

        node6 = node_models.xmlGetNode('velocity_pressure')
        node7 = node_models.xmlGetNode('turbulence')

        nodepp = self.case.xmlGetNode('physical_properties')
        node8 = nodepp.xmlInitNode('fluid_properties')

        nodets = self.case.xmlGetNode('analysis_control')
        node9 = nodets.xmlInitNode('time_parameters')
        node10 = nodets.xmlGetNode('time_averages')

        nodList = []
        for node in (node1, node2, node3, node4, node5, 
                     node6, node7, node8, node9, node10):
            if node:
                for var in ('scalar','variable','property','time_average'):
                    nodList = node.xmlGetNodeList(var)
                    for nodvar in nodList: varList.append(nodvar)

        return varList


    def setListingFrequency(self, freq):
        self.node_out.xmlSetData('listing_printing_frequency', freq)


    def getListingFrequency(self):
        f = self.node_out.xmlGetInt('listing_printing_frequency')
        if not f:
            f = self.defaultInitialValues()['listing_printing_frequency']
            self.setListingFrequency(f)
        return f


    def setPostprocessingFrequency(self, freq):
        self.node_out.xmlSetData('postprocessing_frequency', freq)


    def getPostprocessingFrequency(self):
        f = self.node_out.xmlGetInt('postprocessing_frequency')
        if not f:
            f = self.defaultInitialValues()['postprocessing_frequency']
            self.setPostprocessingFrequency(f)
        return f


    def getFluidDomainPostProStatus(self):
        nod = self.node_out.xmlGetNode('fluid_domain', 'status')
        if not nod: 
            status = self.defaultInitialValues()['fluid_domain']
            self.setFluidDomainPostProStatus(status)
        else:
            status = nod['status']
        return status


    def setFluidDomainPostProStatus(self, status):
        node = self.node_out.xmlInitNode('fluid_domain', 'status')
        node['status'] = status


    def getDomainBoundaryPostProStatus(self):
        nod = self.node_out.xmlGetNode('domain_boundary', 'status')
        if not nod:
            status = self.defaultInitialValues()['domain_boundary']
            self.setDomainBoundaryPostProStatus(status)
        else:
            status = nod['status']
        return status


    def setDomainBoundaryPostProStatus(self, status):
        node = self.node_out.xmlInitNode('domain_boundary', 'status')
        node['status'] = status


    def getSyrthesBoundaryPostProStatus(self):
        nod = self.node_out.xmlGetNode('syrthes_boundary', 'status')
        if not nod: 
            status = self.defaultInitialValues()['syrthes_boundary']
            self.setSyrthesBoundaryPostProStatus(status)
        else:
            status = nod['status']
        return status


    def setSyrthesBoundaryPostProStatus(self, status):
        node = self.node_out.xmlInitNode('syrthes_boundary', 'status')
        node['status'] = status


    def getTypePostMeshes(self):
        node = self.node_out.xmlGetNode('postprocessing_mesh_options')
        if not node:
            choice = self.defaultInitialValues()['postprocessing_mesh']
            self.setTypePostMeshes(choice)
        else:
            choice = node['choice']
        return choice


    def setTypePostMeshes(self, choice):
        node = self.node_out.xmlInitNode('postprocessing_mesh_options', 'choice')
        node['choice'] = choice


    def getPostProFormat(self):
        node = self.node_out.xmlGetNode('postprocessing_format', 'choice')
        if not node:
            choice = self.defaultInitialValues()['postprocessing_format']
            self.setPostProFormat(choice)
        else:
            choice = node['choice']
        return choice


    def setPostProFormat(self, choice):
        node = self.node_out.xmlInitNode('postprocessing_format', 'choice')
        node['choice'] = choice


    def getPostProOptionsFormat(self):
        node = self.node_out.xmlGetNode('postprocessing_options', 'choice')
        if not node:
            line = self.defaultInitialValues()['postprocessing_options']
            self.setPostProOptionsFormat(line)
        else:
            line = node['choice']
        return line 


    def setPostProOptionsFormat(self, line):
        n = self.node_out.xmlInitNode('postprocessing_options', 'choice')
        n['choice'] = line


    def setMonitoringPointFrequency(self, freq):
        self.node_out.xmlSetData('probe_recording_frequency', freq)


    def getMonitoringPointFrequency(self):
        f = self.node_out.xmlGetInt('probe_recording_frequency')
        if not f:
            f = self.defaultInitialValues()['probe_recording_frequency']
            self.setMonitoringPointFrequency(f)
        return f


    def addMonitoringPoint(self, x=0.0, y=0.0, z=0.0):
        """
        Input a new monitoring point definition.
        """
        status="on"
        num = 1
        while (str(num)) in self.dicoName.keys(): num = num+1
        num = str(num)
        probe_num = 'item' + num
        self.dicoName[num] = probe_num
        
        node = self.node_out.xmlInitNode('probe', name=num, status=status)
        node.xmlSetData('probe_x', x)
        node.xmlSetData('probe_y', y)
        node.xmlSetData('probe_z', z)

        # we put probes on all variables, scalars and properties
        #
##        for node in self.getAllVarScalProper():
##            if node['name'] != 'weight_matrix_X' \
##              or node['name'] != 'weight_matrix_Y' or node['name'] != 'weight_matrix_Z':
##                node.xmlInitNode('probe_recording', name=num)

        return int(num)


    def replaceMonitoringPointCoordinates(self, name=None, x=0.0, y=0.0, z=0.0):
        """
        Change the localization of a probe
        """
        if not name:
            raise ValueError, "replaceMonitoringPointCoordinates : pas de nom au point de monitoring"

        node = self.node_out.xmlGetNode('probe', name=name)
        node.xmlSetData('probe_x', x)
        node.xmlSetData('probe_y', y)
        node.xmlSetData('probe_z', z)

    def __deleteMonitoringPoint(self, n):
        node = self.node_out.xmlGetNode('probe', name=n)
        if node:
            node.xmlRemoveNode()
            self.case.xmlRemoveChild('probe_recording', name=n)
            del self.dicoName[n]


    def deleteMonitoringPointsList(self, nameList):
        """
        Destroy the definition of a monitoring points list
        """
        if not nameList: return
        for n in nameList:
            self.__deleteMonitoringPoint(n)

        # compactage du numero des points de monitorings
        listprob, dicocoord = [], {}
        for node in self.node_out.xmlGetNodeList('probe'):
            listprob.append(node['name'])
            dicocoord[node['name']] = (node.xmlGetDouble('probe_x'),
                                       node.xmlGetDouble('probe_y'),
                                       node.xmlGetDouble('probe_z'))
        listprob.sort()
        for n in range(len(listprob)):
            # si l'indice de rangement ne correspond pas  un point de monitoring ...
            if str(n+1) not in listprob:
                # alors on fait un dcalage sur tous les points de monitoring suivants
                for num in range(n, len(listprob)):
                    for node in self.node_out.xmlGetNodeList('probe'):
                        if node['name'] == listprob[n]:
                            node['name'] = str(n+1)
                            prob = listprob[n]
                            node.xmlSetData('probe_x',dicocoord[prob][0])
                            node.xmlSetData('probe_y',dicocoord[prob][1])
                            node.xmlSetData('probe_z',dicocoord[prob][2])
                     
                    for node in self.case.xmlGetNodeList('probe_recording'):
                        if node['name'] == listprob[n]:
                            node['name'] = str(n+1)
                    listprob[n] = n+1

        #on rinitialise le dico pour le remplir avec les valeurs compactes
        self.dicoName = {}
        for n in range(len(listprob)):
            self.dicoName[str(n+1)] = 'item'+str(listprob[n])


    def getMonitoringPointInfo(self):
        """
        """
        X = []
        Y = []
        Z = []
        N = []
        for probe in self.node_out.xmlGetNodeList('probe'):
            X.append(probe.xmlGetDouble('probe_x'))
            Y.append(probe.xmlGetDouble('probe_y'))
            Z.append(probe.xmlGetDouble('probe_z'))
            num = probe['name']
            N.append(num)
            probe_num = 'item' + num
            self.dicoName[num] = probe_num
        if len(N) == 0: 
            for n in self.case.xmlGetNodeList('probes'):
                n.xmlRemoveNode()
        return N, X, Y, Z

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
