# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the values of reference.

This module contains the following classes and function:
- ReferenceValuesModel
- ReferenceValuesView
- PageText
- ReferenceValuesTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import Tix
from Tkconstants import *
import sys, unittest


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Variables
from Pages.CoalCombustion import CoalCombustionModel
from Pages.GasCombustion import GasCombustionModel
from Pages.ElectricalModels import ElectricalModel

#-------------------------------------------------------------------------------
# Reference values model class
#-------------------------------------------------------------------------------

class ReferenceValuesModel:
    """
    Manage the input/output markups in the xml doc about Pressure
    """
    def __init__(self, case):
        """
        Constructor.
        """
        self.case = case

        self.node_models = self.case.xmlGetNode('thermophysical_models')
        self.node_veloce = self.node_models.xmlGetNode('velocity_pressure')
        self.node_coal = self.node_models.xmlGetNode('pulverized_coal', 'model')
        self.node_gas   = self.node_models.xmlGetNode('gas_combustion',  'model')
        self.node_joule = self.node_models.xmlGetNode('joule_effect',  'model')

    def defaultValues(self):
        """
        Return reference values by default
        """
        default = {}
        default['reference_pressure'] = 1.013e+5
        default['reference_temperature'] = 1273.15
        # mass molar for dry air
        default['reference_mass_molar'] = 28.966e-3
        
        return default


    def setPressure(self, value):
        """
        Set value of reference pressure into xml file.
        """ 
        node = self.node_veloce.xmlGetNode('variable',name ='pressure')
        node.xmlSetData('reference_pressure', value)


    def getPressure(self):
        """
        Return the value of reference pressure.
        """ 
        node = self.node_veloce.xmlGetNode('variable',name ='pressure')
        value = node.xmlGetDouble('reference_pressure')
        if value == None: 
            value = self.defaultValues()['reference_pressure']
            self.setPressure(value)

        return value


    def setTemperature(self, value):
        """
        Set reference temperature.
        """
        model, node = self.getParticularPhysical()
        node.xmlSetData('reference_temperature', value)


    def getTemperature(self):
        """
        Get reference temperature.
        """
        model, node = self.getParticularPhysical()
        value = node.xmlGetDouble('reference_temperature')
        if not value :
            value = self.defaultValues()['reference_temperature']
            self.setTemperature(value)
        return value


    def setMassemol(self, value):
        """
        Set reference mass molar.
        """
        model, node = self.getParticularPhysical()
        node.xmlSetData('reference_mass_molar', value)


    def getMassemol(self):
        """
        Get reference mass molar.
        """
        model, node = self.getParticularPhysical()
        value = node.xmlGetDouble('reference_mass_molar')
        if not value :
            value = self.defaultValues()['reference_mass_molar']
            self.setMassemol(value)
        return value



    def getParticularPhysical(self):
        """
        Get model for set temperature for relative model
        """
        model = 'off'
        node = None
        coalModel = CoalCombustionModel(self.case).getCoalCombustionModel()
        gasModel = GasCombustionModel(self.case).getGasCombustionModel()
        jouleModel = ElectricalModel(self.case).getElectricalModel()
        if coalModel != 'off': 
            model = coalModel
            node = self.node_coal
        if gasModel != 'off': 
            model = gasModel
            node = self.node_gas
        if jouleModel != 'off': 
            model = jouleModel
            node = self.node_joule
        return model, node
        
#-------------------------------------------------------------------------------
# Main View class
#-------------------------------------------------------------------------------


class ReferenceValuesView(TkPage.Page):
    """
    Class to open Reference Pressure Page.
    """

 
    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['models/initia',
                                      'bounda/thermo']


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.pressure    = Tix.DoubleVar()
        self.temperature = Tix.DoubleVar()
        self.massemol    = Tix.DoubleVar()


    def _pageControl(self):
        """
        Instantiate the turbulence modelling class.
        """
        self.mdl = ReferenceValuesModel(self.case)


    def getPressure(self, event=None):
        """
        Input PRESS.
        """
        self.stbar.busy()
        
        if self.check2.isPositive(self.p, self.pressure):
            self.stbar.clear()
            pressure = self.pressure.get()
            self.mdl.setPressure(pressure)

        self.stbar.idle()


    def getTemperature(self, event=None):
        """
        Input TEMPERATURE.
        """
        self.stbar.busy()

        temperature = self.temperature.get()
        self.mdl.setTemperature(temperature)

        self.stbar.idle()


    def getMassemol(self, event=None):
        """
        Input Mass molar.
        """
        self.stbar.busy()

        masse = self.massemol.get()
        if self.check2.isSPositive(self.m, self.massemol):
            self.mdl.setMassemol(masse)

        self.stbar.idle()


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()
        lf1 = Tix.LabelFrame(self.myPage, bd=2,
                             label=t.PRESSURE_TITLE, relief=FLAT)
        lf1.label.config(font=fT)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)

        lp = Tix.Label(lf1.frame, text=t.REF_PRESS)
        lp.grid(row=0, column=0, padx=5, pady=10, sticky=W)
        self.balloon.bind_widget(lp,balloonmsg=t.KEYWORD+'P0')
        self.p = Tix.Entry(lf1.frame, bd=2, width=10, textvariable=self.pressure)
        self.p.grid(row=0, column=1, padx=5, pady=10, sticky=W)
        self.p.event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
        self.p.bind("<<Event>>",self.getPressure)
        labp = Tix.Label(lf1.frame, text="Pa")
        labp.grid(row=0, column=2, padx=5, pady=10, sticky=E)
        self.balloon.bind_widget(self.p,balloonmsg=t.KEYWORD+'P0')

        model, node = self.mdl.getParticularPhysical()
        if model != "off":
            lf2 = Tix.LabelFrame(self.myPage, bd=2,
                             label=t.TEMPERATURE_TITLE, relief=FLAT)
            lf2.label.config(font=fT)
            lf2.pack(side=TOP, fill=X, padx=10, pady=10)
            lt = Tix.Label(lf2.frame, text=t.REF_TEMP)
            lt.grid(row=0, column=0, padx=5, pady=10, sticky=W)
            self.balloon.bind_widget(lt,balloonmsg=t.KEYWORD+'T0')
            self.t = Tix.Entry(lf2.frame, bd=2, width=10, textvariable=self.temperature)
            self.t.grid(row=0, column=1, padx=5, pady=10, sticky=W)
            self.t.event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.t.bind("<<Event>>",self.getTemperature)
            labt = Tix.Label(lf2.frame, text="K")
            labt.grid(row=0, column=2, padx=5, pady=10, sticky=W)
            Tix.Label(lf2.frame, text=t.PRECIS).grid(row=1, column=0, columnspan=2, padx=5, pady=5, sticky=E)
            self.balloon.bind_widget(self.t,balloonmsg=t.KEYWORD+'T0')
            if node != 'self.node_joule':
                lf3 = Tix.LabelFrame(self.myPage, bd=2,
                                    label=t.MASSE_MOL_TITLE, relief=FLAT)
                lf3.label.config(font=fT)
                lf3.pack(side=TOP, fill=X, padx=10, pady=10)
                lm = Tix.Label(lf3.frame, text=t.REF_MASSE)
                lm.grid(row=0, column=0, padx=5, pady=10, sticky=W)
                self.balloon.bind_widget(lm,balloonmsg=t.KEYWORD+'WMOLME')
                self.m = Tix.Entry(lf3.frame, bd=2, width=10, textvariable=self.massemol)
                self.m.grid(row=0, column=1, padx=5, pady=10, sticky=W)
                self.m.event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
                self.m.bind("<<Event>>",self.getMassemol)
                labm = Tix.Label(lf3.frame, text="kg/mol")
                labm.grid(row=0, column=2, padx=5, pady=10, sticky=E)
                Tix.Label(lf3.frame, text=t.MPRECIS).grid(row=1, column=0, columnspan=2, padx=5, pady=5, sticky=E)
                self.balloon.bind_widget(self.m,balloonmsg=t.KEYWORD+'WMOLME')


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        self.pressure.set(self.mdl.getPressure())
        model, node = self.mdl.getParticularPhysical()
        if model != "off":
            self.temperature.set(self.mdl.getTemperature())
            if node != 'self.node_joule':
                self.massemol.set(self.mdl.getMassemol())

#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.MASSE_MOL_TITLE     = "Masse molaire molculaire de rfrence"
            self.PRESSURE_TITLE      = "Pression de rfrence"
            self.TEMPERATURE_TITLE   = "Temprature de rfrence"
            self.VALUES_TITLE        = "Valeurs de rfrence"
            self.MPRECIS             = "(utile pour l'initialisation des scalaires "\
                                       "utilisateurs)"
            self.PRECIS              = "(utile pour l'initialisation de l'enthalpie "\
                                       "et de la masse volumique)"
            self.REF_MASSE           = "Masse molaire de rfrence                     "
            self.REF_PRESS           = "Valeur de rfrence pour la pression totale"
            self.REF_TEMP            = "Temprature de rfrence                     "
            self.KEYWORD             = "Mot cl Code_Saturne : "
        else:
            self.MASSE_MOL_TITLE     = "Mass molar molecular reference"
            self.PRESSURE_TITLE      = "Reference pressure"
            self.TEMPERATURE_TITLE   = "Reference temperature"
            self.VALUES_TITLE        = "Reference values"
            self.MPRECIS             = "(used for initialization of users's scalars)"
            self.PRECIS              = "(used for initialization of enthalpy and density)"
            self.REF_MASSE           = "Mass molar reference"
            self.REF_PRESS           = "Reference value for total pressure"
            self.REF_TEMP            = "Reference temperature                "
            self.KEYWORD             = "Code_Saturne key word: "

#-------------------------------------------------------------------------------
# ReferenceValuesModel test case
#-------------------------------------------------------------------------------


class ReferencePressureTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        self.doc = XMLDocument()

    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case
        del self.doc

    def xmlNodeFromString(self, string):
        """Private method to return a xml node from string"""
        return self.doc.parseString(string).root()

    def checkTurbulenceInstantiation(self):
        """Check whether the TurbulenceModel class could be instantiated"""
        model = None
        model = TurbulenceModel(self.case)
        assert model != None, 'Could not instantiate TurbulenceModel'


def suite():
    testSuite = unittest.makeSuite(TurbulenceTestCase, "check")
    return testSuite

def runTest():
    print "TurbulenceTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
