# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the values of reference.

This module contains the following classes and function:
- SteadyManagementModel
- SteadyManagementView
- PageText
- SteadyManagementTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import Tix
from Tkconstants import *
import sys, unittest


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Variables
from Pages.CoalCombustion import CoalCombustionModel
from Pages.GasCombustion import GasCombustionModel
from Pages.ElectricalModels import ElectricalModel

#-------------------------------------------------------------------------------
# Reference values model class
#-------------------------------------------------------------------------------

class SteadyManagementModel:
    """
    Manage the input/output markups in the xml doc about Pressure
    """
    def __init__(self, case):
        """
        Constructor.
        """
        self.case = case

        self.node_anal   = self.case.xmlInitNode('analysis_control')
        self.node_steady = self.node_anal.xmlGetNode('steady_management')

    def defaultValues(self):
        """
        Return in a dictionnary which contains default values
        """
        default = {}
        default['iterations']             = 10
        default['relaxation_coefficient'] = 0.9
        default['zero_iteration']         = "off"

        return default

##        
##    def setSteadyFlowManagement(self, steady):
##        """
##        Set steady flow management balise into xml file.
##        """
##        node =  self.node_anal.xmlInitNode('steady_management')
##        node['status'] = steady
##        node.xmlInitChildNode('zero_iteration', status="off")
##        node.xmlSetData('relaxation_coefficient', 0.9)
##        node.xmlSetData('iterations', 10)


    def setRelaxCoefficient(self, value):
        """
        Set value of coefficient of relaxation into xml file.
        """ 
        self.node_steady.xmlSetData('relaxation_coefficient', value)


    def setNbIter(self, value):
        """
        Set value of iterations number into xml file.
        """ 
        self.node_steady.xmlSetData('iterations', value)


    def setZeroIteration(self, status):
        """
        Set status of option of zero iteration into xml file.
        """ 
        node = self.node_steady.xmlGetChildNode('zero_iteration', 'status')
        if node:
            node['status'] = status


    def getRelaxCoefficient(self):
        """
        Get value of coefficient of relaxation from xml file.
        """ 
        coef = self.node_steady.xmlGetDouble('relaxation_coefficient')
        if not coef:
            coef = self.defaultValues()['relaxation_coefficient']
            self.setRelaxCoefficient(coef)

        return coef


    def getNbIter(self):
        """
        Get value of coefficient of relaxation from xml file.
        """ 
        value = self.node_steady.xmlGetInt('iterations')
        if not value:
            value = self.defaultValues()['iterations']
            self.setNbIter(value)

        return value


    def getZeroIteration(self):
        """
        Get status of option of zero iteration from xml file.
        """ 
        node = self.node_steady.xmlGetChildNode('zero_iteration')
        if node:
            status = node['status']
        else:
            status = self.defaultValues()['zero_iteration']
            self.setZeroIteration(status)
        
        return status


#-------------------------------------------------------------------------------
# Main View class
#-------------------------------------------------------------------------------


class SteadyManagementView(TkPage.Page):
    """
    Class to open steady management Page.
    """
 
    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['models/initia',
                                      'bounda/thermo']


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.zero_iter   = Tix.StringVar()
        self.relax_coef  = Tix.DoubleVar()
        self.nb_iter     = Tix.IntVar()


    def _pageControl(self):
        """
        Instantiate the turbulence modelling class.
        """
        self.mdl = SteadyManagementModel(self.case)


    def getRelaxCoef(self, event=None):
        """
        Input relaxation coefficient.
        """
        self.stbar.busy()
        
        val1 = 0
        val2 = 1
        if self.check2.isBetween(self.rel, self.relax_coef, val1, val2):
            self.mdl.setRelaxCoefficient(self.relax_coef.get())
            
        self.stbar.idle()


    def getNbIter(self, event=None):
        """
        Input itarations number.
        """
        self.stbar.busy()

        if self.check2.isSPositive(self.nbiter, self.nb_iter):
            self.mdl.setNbIter(self.nb_iter.get())
            
        self.stbar.idle()



    def getZeroIteration(self, event=None):
        """
        Input zero iteration number.
        """
        self.stbar.busy()

        iter = self.zero_iter.get()
        self.mdl.setZeroIteration(iter)

        self.stbar.idle()


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()
        lf1 = Tix.LabelFrame(self.myPage, bd=2,
                             label=t.STEADY_TITLE, relief=FLAT)
        lf1.label.config(font=fT)
##        lf1.pack(side=TOP, fill=X, padx=10, pady=10)
        lf1.pack(side=TOP, fill=X, padx=10, pady=10)
        
        w1 = Tix.Frame(lf1.frame, relief=FLAT)
        w1.pack(side=TOP, fill=X, padx=10, pady=10)

        lr = Tix.Label(w1, text=t.RELAX_COEF)
        lr.grid(row=0, column=0, padx=10, pady=10, sticky=W)
        self.balloon.bind_widget(lr,balloonmsg=t.KEYWORD+'RELXST')
        self.rel = Tix.Entry(w1, bd=2, width=10, textvariable=self.relax_coef)
        self.rel.grid(row=0, column=1, padx=5, pady=10)
        self.rel.event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
        self.rel.bind("<<Event>>",self.getRelaxCoef)
        self.balloon.bind_widget(self.rel,balloonmsg=t.KEYWORD+'RELXST')

        # Separator
        #
        Tix.Frame(lf1.frame, height=2, bd=2, relief=SUNKEN).pack(side=TOP, fill=X)

        w2 = Tix.Frame(lf1.frame, relief=FLAT)
        w2.pack(side=TOP, fill=X, padx=10, pady=10)
        
        lnb = Tix.Label(w2, text=t.NB_ITER)
        self.balloon.bind_widget(lnb, balloonmsg=t.KEYWORD+"NTMABS")
        lnb.grid(row=0, column=0, padx=10, pady=10, sticky=W)
        
        self.nbiter = Tix.Entry(w2, bd=2, width=10, textvariable=self.nb_iter)
        self.nbiter.grid(row=0, column=1, padx=5, pady=10)
        self.nbiter.event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
        self.nbiter.bind("<<Event>>",self.getNbIter)
        self.balloon.bind_widget(self.nbiter,balloonmsg=t.KEYWORD+'NTMABS')

        # Separator
        #
        Tix.Frame(lf1.frame, height=2, bd=2, relief=SUNKEN).pack(side=TOP, fill=X)

        w3 = Tix.Frame(lf1.frame, relief=FLAT)
        w3.pack(side=TOP, fill=X, padx=10, pady=10)
        
        lz = Tix.Label(w3, text=t.ZERO_ITER)
        self.balloon.bind_widget(lz, balloonmsg=t.KEYWORD+"INPDT0")
        lz.grid(row=0, column=0, padx=10, pady=10, sticky=W)

        self.zero = Tix.Checkbutton(w3, onvalue='on', offvalue='off',
                                  variable=self.zero_iter,
                                  command=self.getZeroIteration)
        self.zero.grid(row=0, column=1, sticky=E)



    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        self.relax_coef.set(self.mdl.getRelaxCoefficient())
        
        self.nb_iter.set(self.mdl.getNbIter())
        
        if self.mdl.getZeroIteration() == 'on':
            self.zero.select()
        else:
            self.zero.deselect()


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.STEADY_TITLE    = "Gestion de l'coulement stationnaire"
            self.ZERO_ITER       = "Option zro itration"
            self.RELAX_COEF      = "Coefficient de relaxation"
            self.KEYWORD         = "Mot cl Code_Saturne : "
            self.NB_ITER         = "Nombre d'itrations (suites comprises) : "
        else:

            self.STEADY_TITLE    = "Steady management"
            self.ZERO_ITER       = "Option zero iteration"
            self.RELAX_COEF      = "Relaxation coefficient"
            self.KEYWORD         = "Code_Saturne key word: "
            self.NB_ITER         = "Iterations number (restart included): "

#-------------------------------------------------------------------------------
# ReferenceValuesModel test case
#-------------------------------------------------------------------------------


class SteadyManagementTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        from Base.XMLmodel import XMLmodel
        from Pages.AnalysisFeatures import AnalysisFeaturesModel
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        XMLmodel(self.case)
        AnalysisFeaturesModel(self.case).setSteadyFlowManagement('off')
        self.doc = XMLDocument()

    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case
        del self.doc

    def xmlNodeFromString(self, string):
        """Private method to return a xml node from string"""
        n = self.doc.parseString(string)
        self.doc.xmlCleanAllBlank(n)
        return self.doc.root()

    def checkSteadyManagementInstantiation(self):
        """Check whether the SteadyManagementModel class could be instantiated"""
        model = None
        model = SteadyManagementModel(self.case)
        assert model != None, 'Could not instantiate SteadyManagementModel'

    def checkSetandGetRelaxCoefficient(self):
        """Check whether the SteadyManagementModel class could be set or get relax coefficient """ 
        mdl = SteadyManagementModel(self.case)
        mdl.setRelaxCoefficient(0.45)
        doc = """<steady_management status="off">
                   <zero_iteration status="off"/>
                   <iterations>10</iterations>
                   <relaxation_coefficient>0.45</relaxation_coefficient>
                 </steady_management>"""
        assert mdl.node_steady == self.xmlNodeFromString(doc),\
                     'Could not set a relax coefficient'
        coef = mdl.getRelaxCoefficient()
        assert coef == 0.45, 'Could not get a relax coefficient'

    def checkSetandGetZeroIteration(self):
        """Check whether the SteadyManagementModel class could be set or get zero iteration status """ 
        mdl = SteadyManagementModel(self.case)
        mdl.setZeroIteration('on')
        doc = """<steady_management status="off">
                   <zero_iteration status="on"/>
                   <iterations>10</iterations>
                   <relaxation_coefficient>0.9</relaxation_coefficient>
                 </steady_management>"""
        assert mdl.node_steady == self.xmlNodeFromString(doc),\
                    'Could not set a status for zero iteration'
        stat = mdl.getZeroIteration()
        assert stat == 'on', 'Could not get a status for zero iteration'
    
def suite():
    testSuite = unittest.makeSuite(SteadyManagementTestCase, "check")
    return testSuite

def runTest():
    print "SteadyManagementTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
