# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the 'Meshes and Preprocessor' page.

This module defines the following classes:
- TimeAveragesModel
- TimeAveragesView
- PageText
- TimeAveragesTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import os, sys, string, types, unittest
import Tix
from Tkconstants import *
import tkFileDialog


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
import Base.XMLmodel as XMLmodel
from Base.XMLvariables import Variables
from Pages.OutputVariables import OutputVariablesModel
from Pages.StartRestart import StartRestartModel


#-------------------------------------------------------------------------------
# Model class
#-------------------------------------------------------------------------------


class TimeAveragesModel(Variables):
    """
    Class to open TimeAverages Page.
    """
    def __init__(self, case):
        """
        Simple constructor.
        """
        self.case = case
        self.node_anal      = self.case.xmlInitNode('analysis_control')
        self.node_mean      = self.node_anal.xmlInitNode('time_averages')
        self.node_model     = self.case.xmlInitNodeList('thermophysical_models')[0]
        self.node_model_vp  = self.node_model.xmlInitNode('velocity_pressure')
        self.node_var_vp    = self.node_model_vp.xmlGetNodeList('variable')
        self.node_pro_vp    = self.node_model_vp.xmlGetNodeList('property')

        self.LabelDico = []


    def defaultValues(self):
        """
        Return a dictionnary with default values
        """
        defvalue = {}
        defvalue['start'] = 1
        if StartRestartModel(self.case).mGetRestart() == 'off':
            defvalue['restart']    = ''
        else:
            defvalue['restart']    = -1

        return defvalue

        
    def updateDicoLabelName(self):
        """
        Get dictionnary to connect name and label from list of variables and properties 
        """
        self.dicoLabel2Name = {}
        model = XMLmodel.XMLmodel(self.case)
        output = OutputVariablesModel(self.case)
        for nodeList in [self.node_var_vp,
                         self.node_pro_vp,
                         model.getTurbNodeList(),
                         output.getFluidProperty(), 
                         output.getAdditionalScalarProperty(), 
                         output.getTimeProperty(),  
                         output.getPuCoalScalProper()]:
##                         output.getWeightMatrixProperty()]:
            for node in nodeList:
                name = node['name']
                label = node['label']
                if not label:
                    raise ValueError, "Variable node or scalar node has no label"
                if not name:
                    name = label
                if name != 'local_time_step':
                    self.dicoLabel2Name[label] = name
        for nodeList in [output.getThermalScalar(),
                        output.getAdditionalScalar()]:
            for node in nodeList:
                self.dicoLabel2Name[node['label']] = node['label']


    def getRestart(self, nb):
        """
        Get restart to know if balise exists.
        """
##        restart = nb
        restart = ''
        node = self.node_mean.xmlGetNode('time_average', id=nb)
        if node:
            restart = node.xmlGetInt('restart_from_time_average')
        return restart


    def setMoment(self, nb, label, start, restart, list):
        """
        Set list of variables or properties used for calculation of average
        for moment number nb, and time's start and restart values.
        """
        node = self.node_mean.xmlInitNode('time_average', id=nb)
        node['label'] = label
        for var in list:
            node.xmlAddChild('var_prop', name=self.dicoLabel2Name[var])
        node.xmlSetData('time_step_start', start)
        if StartRestartModel(self.case).mGetRestart() != 'off' and restart != '':
            node.xmlSetData('restart_from_time_average', restart)
        else:
            if node.xmlGetInt('restart_from_time_average'):
                node.xmlRemoveChild('restart_from_time_average')
##        self.setOutputControl(node)


    def getMomentLabel(self, imom):
        """
        Get label of moments with id imom.
        """
        node = self.node_mean.xmlInitNode('time_average', id=imom)

        return node['label']


    def getMomentList(self):
        """
        Get list of moments and return list of imom number.
        """
        list = []
        for node in self.node_mean.xmlGetNodeList('time_average'):
            list.append(node['id'])

        return list


    def replaceMoment(self, nb, label, start, restart, list):
        """
        Replace list of variables or properties used for calculation of mean
        for moment number nb, or label, or time's start or restart values.
        """
        node = self.node_mean.xmlGetNode('time_average', id=nb)
        if node:
            node.xmlRemoveChild('var_prop')
            node.xmlRemoveChild('time_step_start')
##            if StartRestartModel(self.case).mGetRestart() != 'off':
            node.xmlRemoveChild('restart_from_time_average')
        for var in list:
            node.xmlAddChild('var_prop', name=self.dicoLabel2Name[var])
        node['label'] = label
        node.xmlSetData('time_step_start', start)
        if StartRestartModel(self.case).mGetRestart() != 'off' and restart != '':
            node.xmlSetData('restart_from_time_average', restart)
        else:
            if node.xmlGetInt('restart_from_time_average'):
                node.xmlRemoveChild('restart_from_time_average')


    def getMomentImom(self, imom):
        """
        Get moment with imom and return time_step_start, restart_from_time_average, 
        and list of variables or properties ....
        """
        list = []
##        restart = -1
        restart = ''
        node = self.node_mean.xmlGetNode('time_average', id=imom)
        start = node.xmlGetInt('time_step_start')
        if StartRestartModel(self.case).mGetRestart() != 'off':
            restart = node.xmlGetInt('restart_from_time_average')
##            if not restart:
##                restart = imom
##        else:
##            restart = ''
        for var in node.xmlGetChildNodeList('var_prop'):
            for label in self.dicoLabel2Name.keys():
                if self.dicoLabel2Name[label] == var['name']:
                    list.append(label)
        return node['label'], start, restart, list
        


    def deleteMoment(self, nb):
        """
        Set list of variables or properties used for calculation of mean
        for moment number nb, and time's start and restart values.
        """
        node = self.node_mean.xmlGetNode('time_average', id=nb)
        if node:
            node.xmlRemoveNode()
            node.xmlRemoveChild('var_prop')
            node.xmlRemoveChild('time_step_start')
            if node.xmlGetInt('restart_from_time_average'):
                node.xmlRemoveChild('restart_from_time_average')
##            if StartRestartModel(self.case).mGetRestart() != 'off':
##            node.xmlRemoveChild('restart_from_time_average')


    def _updateImom(self, nb, label):
        """
        Set list of variables or properties used for calculation of mean
        for moment number nb, and time's start and restart values.
        """
        node = self.node_mean.xmlGetNode('time_average', id=nb)
        nb = str(int(nb) - 1)
        node['id'] = nb
        self.LabelDico.remove(node['label'])
        node['label'] = label
        if label not in self.LabelDico:
            self.LabelDico.append(label)


#-------------------------------------------------------------------------------
# Main class
#-------------------------------------------------------------------------------


class TimeAveragesView(TkPage.Page):
    """
    Class to open TimeAverages Page.
    """
    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['models/initia',
                                      'bounda/scalar']


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.label    = Tix.StringVar()
        self.start    = Tix.StringVar()
        self.restart  = Tix.StringVar()
        self.moment   = []


    def _pageControl(self):
        """
        XML node declaration and supplementary default value settings.
        """
        self.mdl = TimeAveragesModel(self.case)

        #  Initialize variables concerning the display of the Hlist
        #
        self.selectedEntry = None
        self.currentEntry = None
        self.entriesNumber = 0


    def testInt1(self, entry, var):
        """
        Verify if the 'entry' contents an integer. If not, signal an error
        and return None.
        """

        self.check2.begin(entry)
        try:
            i = int(var.get())
            if i <= 0: 
                self.check2.error(entry)
                i = -999

        except:
            self.check2.error(entry)
            i = -999
            
        return i


    def testInt2(self, entry, var):
        """
        Verify if the 'entry' contents an integer. If not, signal an error
        and return None.
        """

        self.check2.begin(entry)
        try:
            i = int(var.get())
            if i <= 0 and i != -1: 
                self.check2.error(entry)
                i = -999

        except:
            self.check2.error(entry)
            i = -999
            
        return i


    def getLabel(self, event=None):
        """
        Return label of moment.
        """
        label = self.label.get()
        if len(label) > 32:
            label = label[:LABEL_LENGTH_MAX]
            if label in self.mdl.LabelDico:
                default = {}
                default['label'] = label
                default['list'] = self.mdl.LabelDico
                dialog = Dialog.VerifyExistenceLabel(self.myPage, default=default)
                select = dialog.result.copy()
                label = select['label']
        return label


    def getStart(self, event=None):
        """
        Return an integer for ntdmom, value of start of calculation.
        """
        t = PageText()
        if self.start.get():
            start = self.testInt1(self.ste, self.start)
            if start == -999:
                Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_NTDMOM)
                start = 1
        else:
            start = 1
        return start


    def getRestart(self, event=None):
        """
        Return an integer for imoold, value of restart of calculation.
        """
        t = PageText()
        if self.restart.get():
            restart = self.testInt2(self.rste, self.restart)
            if restart == -999 or  restart> 50:
                Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_IMOOLD)
                restart = ''
        else:
            restart = ''
            
        return restart
        


    def momentInfo(self, entries):
        """
        Return info from the argument entry.
        """
        # Here we must call the tk function directly without passing in
        # Tix.py, because of a bug in tix8.1 in the Hlist.item_cget function
        #
        h = self.list.hlist
        for entry in entries:
            imom   = h.tk.call(h, 'item', 'cget', entry, 0, '-text')
            label  = h.tk.call(h, 'item', 'cget', entry, 1, '-text')
            ntdmom = h.tk.call(h, 'item', 'cget', entry, 2, '-text')
            imoold = h.tk.call(h, 'item', 'cget', entry, 3, '-text')
            idfmom = h.tk.call(h, 'item', 'cget', entry, 4, '-text')

        return imom, label, ntdmom, imoold, idfmom 


    def insertMoment(self, imom, label, ntdmom, imoold, list):
        """
        Insert values in Hlist.
        """
        idfmom = string.join(list,'*')
        idfmom_view = "<" + idfmom +">"

        index = 'item' + str(imom)
        self.list.hlist.add(index, itemtype=Tix.TEXT, text=imom,
                         style=self.style['normal'])

        self.list.hlist.item_create(index, 1, itemtype=Tix.TEXT,
                                 text=label,
                                 style=self.style['normal'])

        self.list.hlist.item_create(index, 2, itemtype=Tix.TEXT,
                                 text=ntdmom,
                                 style=self.style['normal'])

        self.list.hlist.item_create(index, 3, itemtype=Tix.TEXT,
                                 text=imoold,
                                 style=self.style['normal'])

        self.list.hlist.item_create(index, 4, itemtype=Tix.TEXT,
                                 text=idfmom_view,
                                 style=self.style['left_normal'])


    def addMean(self, event=None):
        """
        Set in hlist IMOM, NTDMOM, IMOOLD, IDFMOM
        """
        t = PageText()
        self.stbar.busy()

        ok = 1

        ntdmom = self.getStart()
        imoold = ''
        if StartRestartModel(self.case).mGetRestart() != 'off':
            imoold = self.getRestart()
##            if imoold == '':
##                imoold = repr(self.entriesNumber +1)
##            imoold_xml = imoold
            if imoold == -999:
                ok = 0
##        else:
##            imoold_xml = repr(self.entriesNumber +1)
        imoold_xml = imoold
        
        idfmom = string.join(self.moment,'*')
        if idfmom == '':
            Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_IDFMOM)
            ok = 0

        if ok == 0: 
            return
        else: 
            label = self.getLabel()
            if label in self.mdl.LabelDico:
                self.stbar.set(t.MSG_LABEL)
                label = ''
            else:
                self.mdl.LabelDico.append(label)

            self.entriesNumber = self.entriesNumber +1
            if label == '':
                label = 'Moy' + repr(self.entriesNumber)
                self.mdl.LabelDico.append(label)
            self.insertMoment(self.entriesNumber, label, ntdmom, imoold, self.moment)
            moment = string.split(idfmom,'*')
            self.mdl.setMoment(repr(self.entriesNumber), label, ntdmom, imoold_xml, moment)
            self.stbar.idle()
        self.eraseEntries()


    def editMean(self, event=None):
        """
        Edit moment to modify the selected momentfrom the list.
        """
        self.stbar.busy()
        t = PageText()
        self.selectMoment()
##        imom, label, ntdmom, imoold, idfmom = self.momentInfo(self.selectedEntry)
        try:
            imom, label, ntdmom, imoold, idfmom = self.momentInfo(self.selectedEntry)
        except:
            Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_MODIFY)
            return
        self.setEditedMoment(imom, label, ntdmom, imoold, idfmom)
        self.cancelSelection()
        self.eraseEntries()
        self.stbar.idle()


    def deleteMean(self, event=None):
        """
        Delete the selected momentfrom the list (one by one).
        """

        self.stbar.busy()
        t = PageText()

        self.selectMoment()

        self.entriesNumber = self.entriesNumber -1
        try:
            imom, label, ntdmom, imoold, idfmom = self.momentInfo(self.selectedEntry)
        except:
            Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_IDFMOM)
            return
        self.list.hlist.delete_entry(self.selectedEntry)
        self.mdl.deleteMoment(imom)
        self.mdl.LabelDico.remove(label)


        for nb in self.mdl.getMomentList():
            if int(nb) > int(imom):
                lab = self.mdl.getMomentLabel(nb)
                if lab == 'Moy' + str(int(nb)):
                    lab = 'Moy' + str(int(nb) - 1)
                self.mdl._updateImom(nb, lab)
                new_nb = str(int(nb) - 1)
                self.replaceIndex(nb, new_nb, lab)

        self.eraseEntries()
        self.stbar.force_idle()


    def setEditedMoment(self, imom, old_label, old_start, old_restart, old_moment):
        """
        Modify the selected momentfrom the list.
        """
        self.stbar.busy()
        t = PageText()
        iok = 1
        new_label = self.getLabel()
        if new_label in self.mdl.LabelDico:
            self.stbar.set(t.MSG_NEW_LABEL)
            new_label = ''
        else:
            self.mdl.LabelDico.append(new_label)

        if new_label == '':
            new_label = old_label
        else:
            self.mdl.LabelDico.remove(old_label)
        new_start = self.getStart()
        new_restart = ''
        if StartRestartModel(self.case).mGetRestart() != 'off':
            new_restart = self.getRestart()
##            if new_restart == '':
##                new_restart = imom
            new_restart_xml = new_restart
            if new_restart == -999:
                ok = 0
        else:
            new_restart_xml = self.mdl.getRestart(imom)
            
        if len(self.moment) == 0 or self.moment[0] == '': 
            Dialog.ShowInfo(self.master, title=t.WARNING, text=t.MSG_IDFMOM)
            iok = 0

        if iok == 0:
            return
        else:
            new_moment = self.moment
            self.replaceMean(imom, new_label, new_start, new_restart, new_moment)
            self.mdl.replaceMoment(imom, new_label, new_start, new_restart_xml, new_moment)
        
        self.stbar.idle()


    def replaceIndex(self, old_index, new_index, new_label):
        """
        Replace index when one moment is deleted.
        """
        entry = 'item' + old_index
        self.list.hlist.delete_entry(entry)
        label, start, restart, list = self.mdl.getMomentImom(new_index)
        self.insertMoment(new_index, label, start, restart, list)


    def replaceMean(self, imom, label, start, restart, moment):
        """
        Replace values on the selected moment.
        """
        self.stbar.busy()

        entry = 'item' + imom
        moment = string.join(moment,'*')
        moment_view = "<" + moment +">"
        self.list.hlist.item_configure(entry, 0, text=imom, style=self.style['normal'])
        self.list.hlist.item_configure(entry, 1, text=label, style=self.style['normal'])
        self.list.hlist.item_configure(entry, 2, text=start, style=self.style['normal'])
        self.list.hlist.item_configure(entry, 3, text=restart, style=self.style['normal'])
        self.list.hlist.item_configure(entry, 4, text=moment_view, style=self.style['left_normal'])

        self.stbar.idle()


    def importVarProp(self, event=None):
        """
        Set variable or property from list1 to list2.
        """
        self.stbar.busy()
        select,index = self.selectList(self.varp)
        if select:
            self.mom.listbox.insert(END, select)
            self.moment.append(select)
        self.stbar.idle()

        
        
    def delVarProp(self, event=None):
        """
        Delete variable or property from list2.
        """
        self.stbar.busy()
        select,index = self.selectList(self.mom)
        if select:
            self.mom.listbox.delete(index)
            self.moment.remove(select)
        self.stbar.idle()


    def selectMoment(self, entry=None):
        """
        Return the selected item from the Hlist.
        """
        self.selectedEntry = self.list.hlist.info_selection()
        if not self.selectedEntry:
            self.lab.config(state=NORMAL)
            self.lab.delete(0,END)
            
            self.ste.config(state=NORMAL)
            self.ste.delete(0,END)
              
            self.rste.config(state=NORMAL)
            self.rste.delete(0,END)
          
        elif self.currentEntry != self.selectedEntry:
            self.currentEntry = self.selectedEntry
            imom, label, ntdmom, imoold, idfmom = self.momentInfo(self.currentEntry)
            self.lab.delete(0,END)
            self.lab.insert(0,label)
            self.lab.icursor(END)
            
            self.ste.delete(0,END)
            self.ste.insert(0,ntdmom)
            self.ste.icursor(END)
            
            if StartRestartModel(self.case).mGetRestart() != 'off':
                self.rste.delete(0,END)
                self.rste.insert(0,imoold)
                self.rste.icursor(END)
            for i in range(len(self.moment)):
                self.mom.listbox.delete(0)
            mom = idfmom[1:len(idfmom)-1]
            self.moment = string.split(mom,'*')
            for var in self.moment:
                self.mom.listbox.insert(END, var)


    def selectList(self, list):
        """
        Return the selected item from the list (one by one).
        """
        try:
            index = int(list.listbox.curselection()[0])
            select = list.listbox.get(index)
        except IndexError:
            select = None
            index = None
            
        return select, index


    def cancelSelection(self):
        """
        Fonction for the Popup Menu.
        Blank the selection.
        """
        self.list.hlist.selection_clear()
        self.currentEntry = None
        self.eraseEntries()


    def eraseEntries(self, event=None):
        """
        Delete all caracters in the entries.
        """
        t = PageText()
        self.lab.delete(0, END)
        self.ste.delete(0, END)
        if StartRestartModel(self.case).mGetRestart() == 'on':
            self.rste.delete(0, END)
        for i in range(len(self.moment)):
                self.mom.listbox.delete(0)
        self.moment = []
        self.varp.selection_clear()


    def insertListVarP(self, name):
        """
        Insert name of variable or property into the list (one by one).
        """
        self.varp.listbox.insert(END, name)


    def DoubleClick(self, event=None):
        """
        This method is correcting a Tkinter bug with the listbox widget.
        Note : The following Sequence does not work with the listbox widget:
        - Double-1
        - Return
        - Up
        - Down
        About events, see www.python.org 16.1.6.7
        """
        if abs(event.time - self.time) < 300:
            print "widget = ", event.widget
        self.time = event.time


    def _make_style(self):
        """
        Define beautyfull style in order to display columns in the Hlist.
        """
        self.style={}

        self.style['header'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.wl,
                                                anchor=CENTER,
                                                padx=5, pady=2,
                                                font=fB)
        self.style['header_left'] = Tix.DisplayStyle(Tix.TEXT,
                                                refwindow=self.wl,
                                                anchor=W,
                                                padx=5, pady=2,
                                                font=fB)
        self.style['left_normal'] = Tix.DisplayStyle(Tix.TEXT,
                                              refwindow=self.wl,
                                              anchor=W,
                                              padx=5, pady=2,
                                              font=fN)
        self.style['normal'] = Tix.DisplayStyle(Tix.TEXT,
                                              refwindow=self.wl,
                                              anchor=CENTER,
                                              padx=5, pady=2,
                                              font=fN)


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()

        # NoteBook and tabs layout
        #
        page = Tix.LabelFrame(self.myPage, bd=2,
                            label=t.MEANS, relief=FLAT)
        page.label.config(font=fT)
        page.pack(side=TOP, fill=X, padx=10)
        
        top = Tix.Frame(page.frame, relief=FLAT)
        top.pack(side=TOP, fill=X, padx=10)
        
        # Frame for means list
        self.wl = Tix.Frame(top, relief=FLAT)
        self.wl.pack(side=TOP, fill=X, pady=5)

        self.list = Tix.ScrolledHList(self.wl, width=400, options='hlist.columns 5 hlist.header 1 ' )
        self.list.config(scrollbar="auto +y +x")
        self.list.hlist.config(selectmode='single',
                               browsecmd=self.selectMoment,
                               bg=wm['hlistbackground'])
        self.list.vsb.config(width=10, bd=1)
        self.list.hsb.config(width=10, bd=1)
        self.list.grid(row=0, column=0, padx=5, pady=5)

        self._make_style()
       # Create the headers
        #
        self.list.hlist.header_create(0, itemtype=Tix.TEXT, text=t.IMOM,
                                      style=self.style['header'])
        self.list.hlist.header_create(1, itemtype=Tix.TEXT, text=t.NAME,
                                      style=self.style['header'])
        self.list.hlist.header_create(2, itemtype=Tix.TEXT, text=t.NTDMOM,
                                      style=self.style['header'])
        self.list.hlist.header_create(3, itemtype=Tix.TEXT, text=t.IMOOLD,
                                      style=self.style['header'])
        self.list.hlist.header_create(4, itemtype=Tix.TEXT, text=t.IDFMOM,
                                      style=self.style['header_left'])

        # Let configure the appearance of the Hlist subwidget 
        #
        self.list.hlist.config(separator='.', width=25, height=11, drawbranch=0, indent=10)

        # Let configure the width of the columns
        #
        self.list.hlist.column_width(0, chars=5)
        self.list.hlist.column_width(1, chars=9)
        self.list.hlist.column_width(2, chars=9)
        self.list.hlist.column_width(3, chars=9)
        self.list.hlist.column_width(4, chars=200)

        # Add and Kill buttons
        #
        self.b = Tix.ButtonBox(self.wl, orientation=HORIZONTAL, relief=FLAT)
        self.b.add('create', text=t.ADD,    command=self.addMean)
        self.b.add('modify', text=t.MODIFY, command=self.editMean)
        self.b.add('delete', text=t.DEL,    command=self.deleteMean)
        self.b.grid(row=1, column=0, columnspan=2, padx=10, pady=5)
        
        # Popup Menu of Tree
        #
        pm = Tix.PopupMenu(self.list.hlist, title=t.POPUP, state=NORMAL)
        pm.bind_widget(self.list.hlist)
        pm.menu.add_separator()
        pm.menu.add_command(label=t.UNSELECT, command=self.cancelSelection)
        pm.menu.m1 = Tix.Menu(pm.menu)
        
        # Separator
        #
        Tix.Frame(top, height=2, bd=2, relief=SUNKEN).pack(side=TOP, fill=X)

        # Paste meshes
        #
        wdef = Tix.Frame(top, relief=FLAT)
        wdef.pack(side=BOTTOM, fill=X, pady=5)

        self.varp = Tix.ScrolledListBox(wdef, scrollbar='auto +y', width=150, height=100)
        self.varp.vsb.config(width=10, bd=1)
        self.varp.hsb.config(width=10, bd=1)
        self.varp.grid(row=0, column= 0, padx=5, pady=5)
        
        # directions buttons
        #
        self.b = Tix.ButtonBox(wdef, orientation=VERTICAL, relief=FLAT)
        self.b.add('import', text=t.IMPORT, command=self.importVarProp)
        self.b.add('export', text=t.EXPORT, command=self.delVarProp)
        self.b.grid(row=0, column=1, padx=2, pady=5)
        
        self.mom = Tix.ScrolledListBox(wdef, scrollbar='auto +y', width=150, height=100)
        self.mom.vsb.config(width=10, bd=1)
        self.mom.hsb.config(width=10, bd=1)
        self.mom.grid(row=0, column= 2, padx=5, pady=5)
        self.balloon.bind_widget(self.mom, balloonmsg=t.KEYWORD+"IDFMOM(IMOM)")

        wt = Tix.Frame(top, relief=FLAT)
        wt.pack(side=BOTTOM, fill=X, pady=5)
        
        Tix.Label(wt, text=t.LABEL).grid(row=1, column=0, padx=5, pady=5, sticky=W)

        self.lab = Tix.Entry(wt, bd=2, width=12, textvariable=self.label)
        self.lab.grid(row=1, column=1, padx=5, pady=5, sticky=E)
        self.lab.bind('<Return>', self.addMean)
        self.balloon.bind_widget(self.lab, balloonmsg=t.LABEL_DEF)
        
        self.stl = Tix.Label(wt, text=t.START_E)
        self.stl.grid(row=2, column=0, padx=5, pady=5, sticky=W)
        self.balloon.bind_widget(self.stl, balloonmsg=t.KEYWORD+"NTDMOM(IMOM)")

        self.ste = Tix.Entry(wt, bd=2, width=12, textvariable=self.start)
        self.ste.grid(row=2, column=1, padx=5, pady=5, sticky=E)
        self.ste.bind('<Return>', self.addMean)
        self.balloon.bind_widget(self.ste, balloonmsg=t.KEYWORD+"NTDMOM(IMOM)")
        
        self.rstl = Tix.Label(wt, text=t.RESTART_E)
        self.rstl.grid(row=3, column=0, padx=5, pady=5, sticky=W)
        self.balloon.bind_widget(self.rstl, balloonmsg=t.KEYWORD+"IMOOLD(IMOM)")
        self.rste = Tix.Entry(wt, bd=2, width=12, textvariable=self.restart)
        self.rste.grid(row=3, column=1, padx=5, pady=5, sticky=E)
        self.rste.bind('<Return>', self.addMean)
        self.balloon.bind_widget(self.rste, balloonmsg=t.KEYWORD+"IMOOLD(IMOM)")


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        t = PageText()
        #update list of variables, properties, scalars ...
        self.mdl.updateDicoLabelName()
        for label in self.mdl.dicoLabel2Name.keys():
            self.insertListVarP(label)
            
        #is it a following calculation ?
##        from StartRestart import StartRestartModel
        if StartRestartModel(self.case).mGetRestart() == 'off':
            self.rstl.config(state=DISABLED)
            self.rste.config(state=DISABLED)
##        del StartRestartModel
        
        #update list of moments for view from xml file
        for nb in self.mdl.getMomentList():
            self.entriesNumber = self.entriesNumber + 1
            label, start, restart, list = self.mdl.getMomentImom(nb)
            self.insertMoment(nb, label, start, restart, list)
            self.mdl.LabelDico.append(label)



#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.ADD              = "Ajouter"
            self.DEL              = "Supprimer"
            self.EXPORT           = "<--"
            self.IDFMOM           = "Moyenne temporelle"
            self.IMOM             = "N"
            self.IMOOLD           = "Restart"
            self.IMPORT           = "-->"
            self.KEYWORD          = "Mot cl Code_Saturne : "
            self.LABEL            = "Nom de la moyenne temporelle : "
            self.LABEL_DEF        = "Entrez un nom pour la moyenne temporelle choisie"
            self.MEANS            = "Moyennes temporelles"
            self.MODIFY           = "Modifier"
            self.NAME             = "Nom"
            self.NTDMOM           = "Dbut"
            self.POPUP            = "Menu popup"
            self.RESTART_E        = "Si suite, numro dans le calcul prcdent\n de la moyenne  "\
                                    "utiliser pour initialiser \nla moyenne slectionne .\n"\
                                    "(-1 indique une rinitialisation)"
            self.START_E          = "Numro d'itration du dpart des moyennes"
            self.UNSELECT         = "Dslectionner"
            self.WARNING          = "Attention"

        else:
            self.ADD              = "Add"
            self.DEL              = "Delete"
            self.EXPORT           = "<--"
            self.IDFMOM           = "Time average"
            self.IMOM             = "N"
            self.IMOOLD           = "Restart"
            self.IMPORT           = "-->"
            self.KEYWORD          = "Code_Saturne key word: "
            self.LABEL            = "Name for time average : "
            self.LABEL_DEF        = "To give a name for the choosen time average"
            self.MEANS            = "Time averages"
            self.MODIFY           = "Modify"
            self.NAME             = "Name"
            self.NTDMOM           = "Start"
            self.POPUP            = "Menu popup"
            self.RESTART_E        = "If restart, number in preceding calculation\n"\
                                    "of the time average to use to initalize \n"\
                                    "the selected time average.\n"\
                                    "(-1 means that the time average is reset to zero)"
            self.START_E          = "Start iteration number for time average calculation"
            self.UNSELECT         = "Unselect"
            self.WARNING          = "Warning"

        # 2) Messages
        #
        if Tool.GuiParam.lang == 'fr':
            self.MSG_IDFMOM    = "Vous devez slectionner au moins une variable "\
                                 "ou proprit de la liste"
            self.MSG_IMOOLD    = "Attention, cette valeur doit tre :\n"\
                                 "soit comprise entre 1 et50\n"\
                                 "soit  gale  -1 "
            self.MSG_LABEL     = "Ce label existe dj : un label par dfaut est choisi"
            self.MSG_MODIFY    = "Pour modifier, il faut effectuer une slection"
            self.MSG_NEW_LABEL = "Ce label existe dj : l'ancien est conserv"
            self.MSG_NTDMOM    = "Attention, cette valeur doit tre au moins gale  1"

        else:
            self.MSG_IDFMOM    = "You must select at least one variable or property from list"
            self.MSG_IMOOLD    = "Warning, this value must be:\n"\
                                 "ever between 1 and 50\n"\
                                 "ever equal to -1"
            self.MSG_LABEL     = "This label already exists: another is choosen"
            self.MSG_MODIFY    = "To modify, you must select one time average"
            self.MSG_NEW_LABEL = "This label already exists: the old label is kept"
            self.MSG_NTDMOM    = "Warning, this value must be strictly up to 0"


#-------------------------------------------------------------------------------
# TimeAveragesModel test case
#-------------------------------------------------------------------------------


class TimeAveragesTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case()
        XMLinit(self.case)
        self.doc = XMLDocument()

    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case
        del self.doc

    def xmlNodeFromString(self, string):
        """Private method to return a xml node from string"""
##        return self.doc.parseString(string).root()
        n = self.doc.parseString(string)
        self.doc.xmlCleanAllBlank(n)
        return self.doc.root()
        
    def checkTimeAveragesInstantiation(self):
        """Check whether the TimeAveragesModel class could be instantiated"""
        model = None
        model = TimeAveragesModel(self.case)
        assert model != None, 'Could not instantiate TimeAveragesModel'
        
    def checkSetMoment(self):
        """Check whether the TimeAveragesModel class could be set a moment"""
        mdl = TimeAveragesModel(self.case)
        mdl.updateDicoLabelName()
        mdl.setMoment('1', 'moyenne', 10, 1, ['VelocitU', 'VelocitV'])
        mom = mdl.node_mean.toString()
        doc = '<time_averages>'\
                '<time_average id="1" label="moyenne">'\
                    '<var_prop name="velocity_U"/>'\
                    '<var_prop name="velocity_V"/>'\
                    '<time_step_start>10</time_step_start>'\
                    '<restart_from_time_average>1</restart_from_time_average>'\
                '</time_average>'\
              '</time_averages>'
        assert doc == mom, \
            'Could not set one moment'

def suite():
    testSuite = unittest.makeSuite(TimeAveragesTestCase, "check")
    return testSuite

def runTest():
    print "TimeAveragesTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
