# -*- coding: iso-8859-1 -*-
#
#-------------------------------------------------------------------------------
#                    Code_Saturne version 1.3
#                    ------------------------
#
#
#     This file is part of the Code_Saturne User Interface, element of the
#     Code_Saturne CFD tool.
#
#     Copyright (C) 1998-2007 EDF S.A., France
#
#     contact: saturne-support@edf.fr
#
#     The Code_Saturne User Interface is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of
#     the License, or (at your option) any later version.
#
#     The Code_Saturne User Interface is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the Code_Saturne Kernel; if not, write to the
#     Free Software Foundation, Inc.,
#     51 Franklin St, Fifth Floor,
#     Boston, MA  02110-1301  USA
#
#-------------------------------------------------------------------------------


"""
This module defines the values of reference.

This module contains the following classes and function:
- UserArraysModel
- UserArraysView
- PageText
- UserArraysTestCase
"""


#-------------------------------------------------------------------------------
# Library modules import
#-------------------------------------------------------------------------------


import Tix
from Tkconstants import *
import sys, unittest


#-------------------------------------------------------------------------------
# Application modules import
#-------------------------------------------------------------------------------


from Base.Common import *
import Base.Toolbox as Tool
import Base.Dialog as Dialog
import Base.TkPage as TkPage
from Base.XMLvariables import Variables


#-------------------------------------------------------------------------------
# User arrays model class
#-------------------------------------------------------------------------------

class UserArraysModel:
    """
    Manage the input/output markups in the xml doc about Integer and real user arrays
    """
    def __init__(self, case):
        """
        Constructor.
        """
        self.case = case

        self.node_calcul = self.case.xmlGetNode('calcul_management')
        self.node_int    = self.node_calcul.xmlInitNode('integer_user_array')
        self.node_real   = self.node_calcul.xmlInitNode('real_user_array')


    def defaultValues(self):
        """
        Return reference values by default
        """
        default = {}
        default['icel'] = 0
        default['ifac'] = 0
        default['ifab'] = 0
        default['idls'] = 0
        default['rcel'] = 0
        default['rfac'] = 0
        default['rfab'] = 0
        default['rdls'] = 0
        return default


    def setIntegerNcelet(self, value):
        """
        Set number of cells with halo into integer user's array.
        """ 
        self.node_int.xmlSetData('ncelet', value)


    def setIntegerNfac(self, value):
        """
        Set number of internal faces into integer user's array.
        """ 
        self.node_int.xmlSetData('nfac', value)


    def setIntegerNfabor(self, value):
        """
        Set number of boundary faces into integer user's array.
        """ 
        self.node_int.xmlSetData('nfabor', value)


    def setIntegerDimless(self, value):
        """
        Set integer value for integer user's array.
        """ 
        self.node_int.xmlSetData('dimless', value)


    def setRealNcelet(self, value):
        """
        Set number of cells with halo into real user's array.
        """ 
        self.node_real.xmlSetData('ncelet', value)


    def setRealNfac(self, value):
        """
        Set number of internal faces into real user's array.
        """ 
        self.node_real.xmlSetData('nfac', value)


    def setRealNfabor(self, value):
        """
        Set number of boundary faces into real user's array.
        """ 
        self.node_real.xmlSetData('nfabor', value)


    def setRealDimless(self, value):
        """
        Set integer value into real user's array.
        """ 
        self.node_real.xmlSetData('dimless', value)


    def getIntegerNcelet(self):
        """
        Return number of cells with halo from integer user's array.
        """ 
        value = self.defaultValues()['icel']
        if self.node_int.xmlGetInt('ncelet'):
            value = self.node_int.xmlGetInt('ncelet')
        else:
            self.setIntegerNcelet(value)
        return value


    def getIntegerNfac(self):
        """
        Return number of cells with halo from integer user's array.
        """ 
        value = self.defaultValues()['ifac']
        if self.node_int.xmlGetInt('nfac'):
            value = self.node_int.xmlGetInt('nfac')
        else:
            self.setIntegerNfac(value)
        return value


    def getIntegerNfabor(self):
        """
        Return number of cells with halo from integer user's array.
        """ 
        value = self.defaultValues()['ifab']
        if self.node_int.xmlGetInt('nfabor'):
            value = self.node_int.xmlGetInt('nfabor')
        else:
            self.setIntegerNfabor(value)
        return value


    def getIntegerDimless(self):
        """
        Return number of cells with halo from integer user's array.
        """ 
        value = self.defaultValues()['idls']
        if self.node_int.xmlGetInt('dimless'):
            value = self.node_int.xmlGetInt('dimless')
        else:
            self.setIntegerDimless(value)
        return value


    def getRealNcelet(self):
        """
        Return number of cells with halo from real user's array.
        """ 
        value = self.defaultValues()['rcel']
        if self.node_real.xmlGetInt('ncelet'):
            value = self.node_real.xmlGetInt('ncelet')
        else:
            self.setRealNcelet(value)
        return value


    def getRealNfac(self):
        """
        Return number of cells with halo from real user's array.
        """ 
        value = self.defaultValues()['rfac']
        if self.node_real.xmlGetInt('nfac'):
            value = self.node_real.xmlGetInt('nfac')
        else:
            self.setRealNfac(value)
        return value


    def getRealNfabor(self):
        """
        Return number of cells with halo from real user's array.
        """ 
        value = self.defaultValues()['rfab']
        if self.node_real.xmlGetInt('nfabor'):
            value = self.node_real.xmlGetInt('nfabor')
        else:
            self.setRealNfabor(value)
        return value


    def getRealDimless(self):
        """
        Return number of cells with halo from real user's array.
        """ 
        value = self.defaultValues()['rdls']
        if self.node_real.xmlGetInt('dimless'):
            value = self.node_real.xmlGetInt('dimless')
        else:
            self.setRealDimless(value)
        return value


#-------------------------------------------------------------------------------
# Main View class
#-------------------------------------------------------------------------------


class UserArraysView(TkPage.Page):
    """
    Class to open Reference Pressure Page.
    """

 
    def _dependsPages(self, name):
        """
        Construction of the list of dependencies Pages.
        """
        self.case[name]['depends'] = ['models/initia',
                                      'bounda/thermo']


    def _tkControlVariables(self):
        """
        Tkinter variables declaration.
        """
        self.icel   = Tix.IntVar()
        self.ifac   = Tix.IntVar()
        self.ifabor = Tix.IntVar()
        self.idimls = Tix.IntVar()
        self.rcel   = Tix.IntVar()
        self.rfac   = Tix.IntVar()
        self.rfabor = Tix.IntVar()
        self.rdimls = Tix.IntVar()


    def _pageControl(self):
        """
        Instantiate the turbulence modelling class.
        """
        self.mdl = UserArraysModel(self.case)


    def setNcelet(self, i, var, event=None):
        """
        Input number of cells with halo for (integer or real) array.
        """
        self.stbar.busy()

        if self.check2.isPositive(self.ecel[i], var):
            self.stbar.clear()
            value = var.get()
            if i == 0:
                self.mdl.setIntegerNcelet(value)
            else:
                self.mdl.setRealNcelet(value)

        self.stbar.idle()


    def setNfac(self, i, var, event=None):
        """
        Input number of internal faces for (integer or real) array.
        """
        self.stbar.busy()

        if self.check2.isPositive(self.efac[i], var):
            self.stbar.clear()
            value = var.get()
            if i == 0:
                self.mdl.setIntegerNfac(value)
            else:
                self.mdl.setRealNfac(value)

        self.stbar.idle()


    def setNfabor(self, i, var, event=None):
        """
        Input number of boundary faces for (integer or real) array.
        """
        self.stbar.busy()

        if self.check2.isPositive(self.efab[i], var):
            self.stbar.clear()
            value = var.get()
            if i == 0:
                self.mdl.setIntegerNfabor(value)
            else:
                self.mdl.setRealNfabor(value)

        self.stbar.idle()


    def setDimless(self, i, var, event=None):
        """
        Input integer value for (integer or real) array.
        """
        self.stbar.busy()

        if self.check2.isPositive(self.edls[i], var):
            self.stbar.clear()
            value = var.get()
            if i == 0:
                self.mdl.setIntegerDimless(value)
            else:
                self.mdl.setRealDimless(value)

        self.stbar.idle()


    def _createWidgets(self):
        """
        Create the Page layout.
        """
        t = PageText()
        
        self.w = [0]*2
        self.t = [0]*2
        self.ecel = [0]*2
        self.efac = [0]*2
        self.efab = [0]*2
        self.edls = [0]*2

        for (i,text, c,f,fb,tt,word) in [(0, t.INT,  self.icel, self.ifac, self.ifabor, self.idimls, 'NITUSE'),
                                         (1, t.REAL, self.rcel, self.rfac, self.rfabor, self.rdimls, 'NRTUSE')]:

            self.w[i]= Tix.LabelFrame(self.myPage, bd=2,
                                      label=text, relief=FLAT)
            self.w[i].label.config(font=fT)
            self.w[i].pack(side=TOP, fill=X, padx=10, pady=10)

            self.t[i] = Tix.Label(self.w[i].frame, text=word+' =')
            self.t[i].grid(row=i, column = 3, columnspan=4, padx=2, pady=5, sticky=W)
            
            self.ecel[i] = Tix.Entry(self.w[i].frame, bd=2, width=6, textvariable=c)
            self.ecel[i].grid(row=i+1, column=0, padx=2, pady=10, sticky=W)
            self.ecel[i].event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.ecel[i].bind("<<Event>>", TkPage.Callback(self.setNcelet, i, c))
            self.balloon.bind_widget(self.ecel[i], balloonmsg=t.SIZE_CELET)

            Tix.Label(self.w[i].frame, text='*NCELET + ').grid(row=i+1, column=1, pady=5, sticky=W)

            self.efac[i] = Tix.Entry(self.w[i].frame, bd=2, width=6, textvariable=f)
            self.efac[i].grid(row=i+1, column=2, padx=2, pady=10, sticky=W)
            self.efac[i].event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.efac[i].bind("<<Event>>", TkPage.Callback(self.setNfac, i, f))
            self.balloon.bind_widget(self.efac[i], balloonmsg=t.SIZE_INT_FACES)

            Tix.Label(self.w[i].frame, text='*NFAC + ').grid(row=i+1, column=3, pady=5, sticky=W)


            self.efab[i] = Tix.Entry(self.w[i].frame, bd=2, width=6, textvariable=fb)
            self.efab[i].grid(row=i+1, column=4, padx=2, pady=10, sticky=W)
            self.efab[i].event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.efab[i].bind("<<Event>>", TkPage.Callback(self.setNfabor, i, fb))
            self.balloon.bind_widget(self.efab[i], balloonmsg=t.SIZE_BOUND_FACES)

            Tix.Label(self.w[i].frame, text='*NFABOR + ').grid(row=i+1, column=5, pady=5, sticky=W)


            self.edls[i] = Tix.Entry(self.w[i].frame, bd=2, width=6, textvariable=tt)
            self.edls[i].grid(row=i+1, column=6, padx=2, pady=10, sticky=W)
            self.edls[i].event_add("<<Event>>", "<Return>", "<Leave>", "<FocusOut>")
            self.edls[i].bind("<<Event>>", TkPage.Callback(self.setDimless, i, tt))
            self.balloon.bind_widget(self.edls[i], balloonmsg=t.SIZE_TOT)


    def _initializeWidgets(self):
        """
        Extract resquested informations from XML document.
        This informations are used to initialize the widgets.
        For this page default values of all widgets are necessary
        included in the XML file.
        """
        self.icel.set(self.mdl.getIntegerNcelet())
        self.ifac.set(self.mdl.getIntegerNfac())
        self.ifabor.set(self.mdl.getIntegerNfabor())
        self.idimls.set(self.mdl.getIntegerDimless())
        
        self.rcel.set(self.mdl.getRealNcelet())
        self.rfac.set(self.mdl.getRealNfac())
        self.rfabor.set(self.mdl.getRealNfabor())
        self.rdimls.set(self.mdl.getRealDimless())
        
        
        
 

#-------------------------------------------------------------------------------
# Text and messages for this page
#-------------------------------------------------------------------------------


class PageText:
    """
    Storage of all texts and messages for this page.
    """
    def __init__(self):
        
        # 1) Texts
        #
        if Tool.GuiParam.lang == 'fr':
            self.INT               = "Tableau d'entiers"
            self.NITUSE            = "NITUSE = "
            self.NRTUSE            = "NRTUSE = "
            self.REAL              = "Tableau de rels"
            self.SIZE_TOT          = "Taille du tableau"
            self.SIZE_CELET        = "Longueur en nombre de cellules avec halo"
            self.SIZE_INT_FACES    = "Longueur en nombre de faces internes"
            self.SIZE_BOUND_FACES  = "Longueur en nombre de faces de bord"
            self.KEYWORD           = "Mot cl Code_Saturne : "
        else:
            self.INT               = "Integer array"
            self.NITUSE            = "NITUSE = "
            self.NRTUSE            = "NRTUSE = "
            self.REAL              = "Real array"
            self.SIZE_TOT          = "Array's size"
            self.SIZE_CELET        = "Size in a number of cells with halo"
            self.SIZE_INT_FACES    = "Size in a number of internal faces"
            self.SIZE_BOUND_FACES  = "Size in a number of boundary faces"

            self.KEYWORD           = "Code_Saturne key word: "

        # 2) Messages
        #

#-------------------------------------------------------------------------------
# UserArraysModel test case
#-------------------------------------------------------------------------------


class UserArraysTestCase(unittest.TestCase):
    """
    """
    def setUp(self):
        """This method is executed before all "check" methods."""
        from Base.XMLengine import Case, XMLDocument
        from Base.XMLinitialize import XMLinit
        Tool.GuiParam.lang = 'en'
        self.case = Case(None)
        XMLinit(self.case)
        self.doc = XMLDocument()

    def tearDown(self):
        """This method is executed after all "check" methods."""
        del self.case
        del self.doc

##    def xmlNodeFromString(self, string):
##        """Private method to return a xml node from string"""
##        return self.doc.parseString(string).root()
##
##    def checkTurbulenceInstantiation(self):
##        """Check whether the TurbulenceModel class could be instantiated"""
##        model = None
##        model = TurbulenceModel(self.case)
##        assert model != None, 'Could not instantiate TurbulenceModel'


def suite():
    testSuite = unittest.makeSuite(TurbulenceTestCase, "check")
    return testSuite

def runTest():
    print "TurbulenceTestCase"
    runner = unittest.TextTestRunner()
    runner.run(suite())

#-------------------------------------------------------------------------------
# End
#-------------------------------------------------------------------------------
